@@ -20,9 +20,10 @@
 #include "emailfunctions/email.h"
 
 #include <kcmdlineargs.h>
-#include <kapplication.h>
 #include <kdebug.h>
 
+#include <QCoreApplication>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -154,9 +155,7 @@ static bool checkQuoteIfNecessary( const QString& input, const QString& expResul
 
 int main(int argc, char *argv[])
 {
-  // KApplication::disableAutoDcopRegistration();
-  KCmdLineArgs::init( argc, argv, "testemail", 0, 0, 0, 0 );
-  KApplication app( false );
+  QCoreApplication app( argc, argv );
 
   // Empty input
   checkGetNameAndEmail( QString(), QString(), QString(), false );
@@ -365,6 +364,7 @@ int main(int argc, char *argv[])
   checkIsValidEmailAddress( "|matt@fruitsalad.org", "AddressOk" );
   checkIsValidEmailAddress( "}matt@fruitsalad.org", "AddressOk" );
   checkIsValidEmailAddress( "~matt@fruitsalad.org", "AddressOk" );
+  checkIsValidEmailAddress( "matt%matt@fruitsalad.org", "AddressOk" );
 
   //bug 105405
   checkIsValidEmailAddress( "[foobar] <matt@fruitsalad.org>", "InvalidDisplayName" );
