@@ -187,7 +187,7 @@ bool AddressBook::Entry::Address::nameOfField(const char* key, QString& value)
 	 ||
 	 !fields->insert
 	 (map<const char*, const QString, less<const char*> >::value_type
-	 (Fields[counter++], i18n("Address"))).second
+	 (Fields[counter++], i18nc("street/postal","Address"))).second
 	 ||
 	 !fields->insert
 	 (map<const char*, const QString, less<const char*> >::value_type
@@ -258,7 +258,7 @@ bool AddressBook::Entry::nameOfField(const char* key, QString& value)
       Q_CHECK_PTR(fields);
       if (!fields->insert
 	 (map<const char*, const QString, less<const char*> >::value_type
-	  (Fields[counter++], i18n("Title"))).second
+	  (Fields[counter++], i18nc("A person's title", "Title"))).second
 	 ||
 	 !fields->insert
 	 (map<const char*, const QString, less<const char*> >::value_type
@@ -577,7 +577,7 @@ AddressBook::ErrorCode AddressBook::load(const QString& filename)
 	           "Your file will be closed by default.",
 		  oldfile.absoluteFilePath()),
 	      i18n("File Error"),
-	      KStdGuiItem::close(), KStdGuiItem::save()))
+	      KStandardGuiItem::close(), KStandardGuiItem::save()))
 	    {
 	    case KMessageBox::No: // save
 	      if (!data->save(i18n("(Safety copy on file error)").toAscii(), true))
@@ -622,7 +622,7 @@ AddressBook::ErrorCode AddressBook::load(const QString& filename)
 	      i18n("The file \"%1\" cannot be found. "
 	           "Create a new one?", fname),
 	      i18n("No Such File"),
-	      i18n("Create"), KStdGuiItem::cancel())==KMessageBox::Yes)
+	      KGuiItem(i18n("Create")), KStandardGuiItem::cancel())==KMessageBox::Yes)
 	    {
 	      if (createNew(fname)==NoError)
 		{
