@@ -32,11 +32,12 @@
 #include <klocale.h>
 #include <kconfig.h>
 #include <kstringhandler.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <kio/netaccess.h>
 
-#include "kabc/ldif.h"
-#include "kabc/ldapurl.h"
+#include "kldap/ldif.h"
+#include "kldap/ldif.h"
+#include "kldap/ldapurl.h"
 
 #include "resourceldapkio.h"
 #include "resourceldapkioconfig.h"
@@ -46,7 +47,7 @@ using namespace KABC;
 class ResourceLDAPKIO::ResourceLDAPKIOPrivate
 {
   public:
-    LDIF mLdif;
+    KLDAP::Ldif mLdif;
     bool mTLS,mSSL,mSubTree;
     QString mResultDn;
     Addressee mAddr;
@@ -55,14 +56,14 @@ class ResourceLDAPKIO::ResourceLDAPKIOPrivate
     bool mSASL;
     QString mMech;
     QString mRealm, mBindDN;
-    LDAPUrl mLDAPUrl;
+    KLDAP::LdapUrl mLDAPUrl;
     int mVer, mSizeLimit, mTimeLimit, mRDNPrefix;
     int mError;
     int mCachePolicy;
     bool mReadOnly;
     bool mAutoCache;
     QString mCacheDst;
-    KTempFile *mTmp;
+    KTemporaryFile *mTmp;
 };
 
 ResourceLDAPKIO::ResourceLDAPKIO( const KConfig *config )
@@ -152,7 +153,7 @@ void ResourceLDAPKIO::listResult( KJob *job)
 
 QString ResourceLDAPKIO::findUid( const QString &uid )
 {
-  LDAPUrl url( d->mLDAPUrl );
+  KLDAP::LdapUrl url( d->mLDAPUrl );
   KIO::UDSEntry entry;
 
   mErrorMsg = d->mResultDn = "";
@@ -179,8 +180,8 @@ QByteArray ResourceLDAPKIO::addEntry( const QString &attr, const QString &value,
 {
   QByteArray tmp;
   if ( !attr.isEmpty() ) {
-    if ( mod ) tmp += LDIF::assembleLine( "replace", attr ) + '\n';
-    tmp += LDIF::assembleLine( attr, value ) + '\n';
+    if ( mod ) tmp += KLDAP::Ldif::assembleLine( "replace", attr ) + '\n';
+    tmp += KLDAP::Ldif::assembleLine( attr, value ) + '\n';
     if ( mod ) tmp += "-\n";
   }
   return ( tmp );
@@ -218,21 +219,21 @@ bool ResourceLDAPKIO::AddresseeToLDIF( QByteArray &ldif, const Addressee &addr,
     }
 
     if ( olddn.toLower() != dn.toLower() ) {
-      tmp = LDIF::assembleLine( "dn", olddn ) + '\n';
+      tmp = KLDAP::Ldif::assembleLine( "dn", olddn ) + '\n';
       tmp += "changetype: modrdn\n";
-      tmp += LDIF::assembleLine( "newrdn", dn.section( ',', 0, 0 ) ) + '\n';
+      tmp += KLDAP::Ldif::assembleLine( "newrdn", dn.section( ',', 0, 0 ) ) + '\n';
       tmp += "deleteoldrdn: 1\n\n";
     }
   }
 
 
-  tmp += LDIF::assembleLine( "dn", dn ) + '\n';
+  tmp += KLDAP::Ldif::assembleLine( "dn", dn ) + '\n';
   if ( mod ) tmp += "changetype: modify\n";
   if ( !mod ) {
     tmp += "objectClass: top\n";
     QStringList obclass = mAttributes[ "objectClass" ].split(',', QString::SkipEmptyParts);
     for ( QStringList::const_iterator it = obclass.constBegin(); it != obclass.constEnd(); ++it ) {
-      tmp += LDIF::assembleLine( "objectClass", *it ) + '\n';
+      tmp += KLDAP::Ldif::assembleLine( "objectClass", *it ) + '\n';
     }
   }
 
@@ -271,9 +272,9 @@ bool ResourceLDAPKIO::AddresseeToLDIF( QByteArray &ldif, const Addressee &addr,
 
   if ( !mAttributes[ "mail" ].isEmpty() ) {
     if ( mod ) tmp +=
-      LDIF::assembleLine( "replace", mAttributes[ "mail" ] ) + '\n';
+      KLDAP::Ldif::assembleLine( "replace", mAttributes[ "mail" ] ) + '\n';
     if ( mailIt != emails.end() ) {
-      tmp += LDIF::assembleLine( mAttributes[ "mail" ], *mailIt ) + '\n';
+      tmp += KLDAP::Ldif::assembleLine( mAttributes[ "mail" ], *mailIt ) + '\n';
       mailIt ++;
     }
     if ( mod && mAttributes[ "mail" ] != mAttributes[ "mailAlias" ] ) tmp += "-\n";
@@ -281,9 +282,9 @@ bool ResourceLDAPKIO::AddresseeToLDIF( QByteArray &ldif, const Addressee &addr,
 
   if ( !mAttributes[ "mailAlias" ].isEmpty() ) {
     if ( mod && mAttributes[ "mail" ] != mAttributes[ "mailAlias" ] ) tmp +=
-      LDIF::assembleLine( "replace", mAttributes[ "mailAlias" ] ) + '\n';
+      KLDAP::Ldif::assembleLine( "replace", mAttributes[ "mailAlias" ] ) + '\n';
     for ( ; mailIt != emails.end(); ++mailIt ) {
-      tmp += LDIF::assembleLine( mAttributes[ "mailAlias" ], *mailIt ) + '\n' ;
+      tmp += KLDAP::Ldif::assembleLine( mAttributes[ "mailAlias" ], *mailIt ) + '\n' ;
     }
     if ( mod ) tmp += "-\n";
   }
@@ -295,8 +296,8 @@ bool ResourceLDAPKIO::AddresseeToLDIF( QByteArray &ldif, const Addressee &addr,
     addr.photo().data().save( &buffer, "JPEG" );
 
     if ( mod ) tmp +=
-      LDIF::assembleLine( "replace", mAttributes[ "jpegPhoto" ] ) + '\n';
-    tmp += LDIF::assembleLine( mAttributes[ "jpegPhoto" ], pic, 76 ) + '\n';
+      KLDAP::Ldif::assembleLine( "replace", mAttributes[ "jpegPhoto" ] ) + '\n';
+    tmp += KLDAP::Ldif::assembleLine( mAttributes[ "jpegPhoto" ], pic, 76 ) + '\n';
     if ( mod ) tmp += "-\n";
   }
 
@@ -368,7 +369,7 @@ void ResourceLDAPKIO::init()
   if ( !mAttributes.contains("jpegPhoto") )
     mAttributes.insert( "jpegPhoto", "jpegPhoto" );
 
-  d->mLDAPUrl = KUrl();
+  d->mLDAPUrl = KLDAP::LdapUrl( KUrl() );
   if ( !mAnonymous ) {
     d->mLDAPUrl.setUser( mUser );
     d->mLDAPUrl.setPass( mPassword );
@@ -388,7 +389,7 @@ void ResourceLDAPKIO::init()
     d->mLDAPUrl.setAttributes( attr );
   }
 
-  d->mLDAPUrl.setScope( d->mSubTree ? LDAPUrl::Sub : LDAPUrl::One );
+  d->mLDAPUrl.setScope( d->mSubTree ? KLDAP::LdapUrl::Sub : KLDAP::LdapUrl::One );
   if ( !mFilter.isEmpty() && mFilter != "(objectClass=*)" )
     d->mLDAPUrl.setFilter( mFilter );
   d->mLDAPUrl.setExtension( "x-dir", "base" );
@@ -471,20 +472,20 @@ void ResourceLDAPKIO::createCache()
 {
   d->mTmp = NULL;
   if ( d->mCachePolicy == Cache_NoConnection && d->mAutoCache ) {
-    d->mTmp = new KTempFile( d->mCacheDst, "tmp" );
-    d->mTmp->setAutoDelete( true );
+    d->mTmp = new KTemporaryFile;
+    d->mTmp->setPrefix(d->mCacheDst);
+    d->mTmp->setSuffix("tmp");
+    d->mTmp->open();
   }
 }
 
 void ResourceLDAPKIO::activateCache()
 {
   if ( d->mTmp && d->mError == 0 ) {
-    d->mTmp->close();
-    rename( QFile::encodeName( d->mTmp->name() ), QFile::encodeName( d->mCacheDst ) );
-  }
-  if ( d->mTmp ) {
+    QString filename = d->mTmp->fileName();
     delete d->mTmp;
     d->mTmp = 0;
+    rename( QFile::encodeName( filename ), QFile::encodeName( d->mCacheDst ) );
   }
 }
 
@@ -578,24 +579,24 @@ bool ResourceLDAPKIO::asyncLoad()
 void ResourceLDAPKIO::data( KIO::Job *, const QByteArray &data )
 {
   if ( data.size() ) {
-    d->mLdif.setLDIF( data );
+    d->mLdif.setLdif( data );
     if ( d->mTmp ) {
-      d->mTmp->file()->write( data );
+      d->mTmp->write( data );
     }
   } else {
-    d->mLdif.endLDIF();
+    d->mLdif.endLdif();
   }
 
-  LDIF::ParseValue ret;
+  KLDAP::Ldif::ParseValue ret;
   QString name;
   QByteArray value;
   do {
     ret = d->mLdif.nextItem();
     switch ( ret ) {
-      case LDIF::NewEntry:
+      case KLDAP::Ldif::NewEntry:
         kDebug(7125) << "new entry: " << d->mLdif.dn() << endl;
         break;
-      case LDIF::Item:
+      case KLDAP::Ldif::Item:
         name = d->mLdif.attr().toLower();
         value = d->mLdif.value();
         if ( name == mAttributes[ "commonName" ].toLower() ) {
@@ -662,7 +663,7 @@ void ResourceLDAPKIO::data( KIO::Job *, const QByteArray &data )
         }
 
         break;
-      case LDIF::EndEntry: {
+      case KLDAP::Ldif::EndEntry: {
         d->mAddr.setResource( this );
         d->mAddr.insertAddress( d->mAd );
         d->mAddr.setChanged( false );
@@ -675,7 +676,7 @@ void ResourceLDAPKIO::data( KIO::Job *, const QByteArray &data )
       default:
         break;
     }
-  } while ( ret != LDIF::MoreData );
+  } while ( ret != KLDAP::Ldif::MoreData );
 }
 
 void ResourceLDAPKIO::loadCacheResult( KJob *job )
@@ -804,10 +805,10 @@ void ResourceLDAPKIO::removeAddressee( const Addressee& addr )
   }
   if ( !dn.isEmpty() ) {
     kDebug(7125) << "ResourceLDAPKIO: found uid: " << dn << endl;
-    LDAPUrl url( d->mLDAPUrl );
+    KLDAP::LdapUrl url( d->mLDAPUrl );
     url.setPath( '/' + dn );
     url.setExtension( "x-dir", "base" );
-    url.setScope( LDAPUrl::Base );
+    url.setScope( KLDAP::LdapUrl::Base );
     if ( KIO::NetAccess::del( url, NULL ) ) mAddrMap.remove( addr.uid() );
   } else {
     //maybe it's not saved yet
