@@ -23,7 +23,6 @@
 #include <QList>
 #include <QRegExp>
 
-#include <kapplication.h>
 #include <kdebug.h>
 #include <klocale.h>
 #include <krandom.h>
@@ -184,8 +183,13 @@ QString Addressee::uidLabel()
 
 --DEFINITIONS--
 
-void Addressee::setNameFromString( const QString &str )
+void Addressee::setNameFromString( const QString &s )
 {
+  QString str = s;
+  //remove enclosing quotes from string
+  if ( str.length() > 1  && s[ 0 ] == '"' && s[ s.length() - 1 ] == '"' )
+    str = s.mid( 1, s.length() - 2 );
+
   setFormattedName( str );
   setName( str );
 
