@@ -21,105 +21,236 @@
     Boston, MA 02110-1301, USA.
 */
 /**
-   @file calendar.cpp
-   Provides the main "calendar" object class.
+  @file
+  This file is part of the API for handling calendar data and
+  defines the Calendar class.
 
-   @author Preston Brown
-   @author Cornelius Schumacher
-   @author Reinhold Kainhofer
+  @author Preston Brown
+  @author Cornelius Schumacher
+  @author Reinhold Kainhofer
 */
-#include <stdlib.h>
-
 #include <kdebug.h>
 #include <klocale.h>
-#include <ktimezones.h>
+
+extern "C" {
+  #include <icaltimezone.h>
+}
 
 #include "exceptions.h"
 #include "calfilter.h"
-
+#include "icaltimezones.h"
 #include "calendar.h"
 
 using namespace KCal;
 
-Calendar::Calendar( const QString &timeZoneId )
-  : mTimeZones( new KTimeZones ),
-    mTimeZoneId( timeZoneId ),
-    mLocalTime( false ),
-    mModified( false ),
-    mDefaultFilter( new CalFilter ),
-    mFilter( mDefaultFilter ),
-    mNewObserver( false ),
-    mObserversEnabled( true )
+/**
+  Private class that helps to provide binary compatibility between releases.
+  @internal
+*/
+//@cond PRIVATE
+class KCal::Calendar::Private
 {
-  // Setup default filter, which does nothing
-  mFilter->setEnabled( false );
+  public:
+    Private()
+      : mTimeZones( new ICalTimeZones ),
+        mBuiltInTimeZone( 0 ),
+        mBuiltInViewTimeZone( 0 ),
+	mModified( false ),
+	mNewObserver( false ),
+	mObserversEnabled( true ),
+	mDefaultFilter( new CalFilter ),
+	mFilter( mDefaultFilter )
+    {
+        // Setup default filter, which does nothing
+        mFilter->setEnabled( false );
 
-  // user information...
-  setOwner( Person( i18n( "Unknown Name" ), i18n( "unknown@nowhere" ) ) );
+        // user information...
+        mOwner.setName( i18n( "Unknown Name" ) );
+        mOwner.setEmail( i18n( "unknown@nowhere" ) );
+    }
+
+    ~Private()
+    {
+        delete mTimeZones;
+        delete mBuiltInTimeZone;
+        if ( mBuiltInViewTimeZone != mBuiltInTimeZone )
+          delete mBuiltInViewTimeZone;
+        delete mDefaultFilter;
+    }
+    KDateTime::Spec timeZoneIdSpec( const QString &timeZoneId, bool view );
+
+    QString mProductId;
+    Person mOwner;
+    ICalTimeZones *mTimeZones; // collection of time zones used in this calendar
+    const ICalTimeZone *mBuiltInTimeZone;   // cached time zone lookup
+    const ICalTimeZone *mBuiltInViewTimeZone;   // cached viewing time zone lookup
+    KDateTime::Spec mTimeSpec;
+    mutable KDateTime::Spec mViewTimeSpec;
+    bool mModified;
+    bool mNewObserver;
+    bool mObserversEnabled;
+    QList<CalendarObserver*> mObservers;
+
+    CalFilter *mDefaultFilter;
+    CalFilter *mFilter;
+
+    // These lists are used to put together related To-dos
+    QMultiHash<QString, Incidence*> mOrphans;
+    QMultiHash<QString, Incidence*> mOrphanUids;
+};
+//@endcond
+
+Calendar::Calendar( const KDateTime::Spec &timeSpec )
+  : d( new KCal::Calendar::Private )
+{
+  d->mTimeSpec = timeSpec;
+  d->mViewTimeSpec = timeSpec;
+}
+
+Calendar::Calendar( const QString &timeZoneId )
+  : d( new KCal::Calendar::Private )
+{
+  setTimeZoneId( timeZoneId );
 }
 
 Calendar::~Calendar()
 {
-  delete mDefaultFilter;
-  delete mTimeZones;
+  delete d;
 }
 
-const Person &Calendar::getOwner() const
+const Person &Calendar::owner() const
 {
-  return mOwner;
+  return d->mOwner;
 }
 
 void Calendar::setOwner( const Person &owner )
 {
-  mOwner = owner;
+  d->mOwner = owner;
 
   setModified( true );
 }
 
+void Calendar::setTimeSpec( const KDateTime::Spec &timeSpec )
+{
+  d->mTimeSpec = timeSpec;
+  if ( d->mBuiltInViewTimeZone != d->mBuiltInTimeZone )
+    delete d->mBuiltInTimeZone;
+  d->mBuiltInTimeZone = 0;
+  setViewTimeSpec( timeSpec );
+
+  doSetTimeSpec( d->mTimeSpec );
+}
+
+KDateTime::Spec Calendar::timeSpec() const
+{
+  return d->mTimeSpec;
+}
+
 void Calendar::setTimeZoneId( const QString &timeZoneId )
 {
-  mTimeZoneId = timeZoneId;
-  mLocalTime = false;
+  d->mTimeSpec = d->timeZoneIdSpec( timeZoneId, false );
+  d->mViewTimeSpec = d->mTimeSpec;
+  if ( d->mBuiltInViewTimeZone != d->mBuiltInTimeZone )
+    delete d->mBuiltInViewTimeZone;
+  d->mBuiltInViewTimeZone = d->mBuiltInTimeZone;
 
-  setModified( true );
-  doSetTimeZoneId( timeZoneId );
+  doSetTimeSpec( d->mTimeSpec );
+}
+
+KDateTime::Spec Calendar::Private::timeZoneIdSpec( const QString &timeZoneId, bool view )
+{
+  if ( view ) {
+    if ( mBuiltInViewTimeZone != mBuiltInTimeZone )
+      delete mBuiltInViewTimeZone;
+    mBuiltInViewTimeZone = 0;
+  } else {
+    if ( mBuiltInViewTimeZone != mBuiltInTimeZone )
+      delete mBuiltInTimeZone;
+    mBuiltInTimeZone = 0;
+  }
+  if ( timeZoneId == QLatin1String("UTC") ) {
+    return( KDateTime::UTC );
+  }
+  const ICalTimeZone *tz = mTimeZones->zone(timeZoneId);
+  if (!tz) {
+    ICalTimeZoneSource tzsrc;
+    tz = tzsrc.parse(icaltimezone_get_builtin_timezone(timeZoneId.toLatin1()));
+    if ( view )
+      mBuiltInViewTimeZone = tz;
+    else
+      mBuiltInTimeZone = tz;
+  }
+  if (tz)
+    return tz;
+  else
+    return KDateTime::ClockTime;
 }
 
 QString Calendar::timeZoneId() const
 {
-  return mTimeZoneId;
+  const KTimeZone *tz = d->mTimeSpec.timeZone();
+  return tz ? tz->name() : QString();
 }
 
-KTimeZones *Calendar::timeZones() const
+void Calendar::setViewTimeSpec( const KDateTime::Spec &spec ) const
 {
-  return mTimeZones;
+  d->mViewTimeSpec = spec;
+  if ( d->mBuiltInViewTimeZone != d->mBuiltInTimeZone )
+    delete d->mBuiltInViewTimeZone;
+  d->mBuiltInViewTimeZone = 0;
 }
 
-void Calendar::setLocalTime()
+void Calendar::setViewTimeZoneId( const QString &timeZoneId ) const
 {
-  mLocalTime = true;
-  mTimeZoneId = "";
-
-  setModified( true );
+  d->mViewTimeSpec = d->timeZoneIdSpec( timeZoneId, true );
 }
 
-bool Calendar::isLocalTime() const
+KDateTime::Spec Calendar::viewTimeSpec() const
 {
-  return mLocalTime;
+  return d->mViewTimeSpec;
+}
+
+QString Calendar::viewTimeZoneId() const
+{
+  const KTimeZone *tz = d->mViewTimeSpec.timeZone();
+  return tz ? tz->name() : QString();
+}
+
+ICalTimeZones *Calendar::timeZones() const
+{
+  return d->mTimeZones;
+}
+
+void Calendar::shiftTimes(const KDateTime::Spec &oldSpec, const KDateTime::Spec &newSpec)
+{
+  setTimeSpec( newSpec );
+
+  int i, end;
+  Event::List ev = events();
+  for ( i = 0, end = ev.count();  i < end;  ++i )
+    ev[i]->shiftTimes( oldSpec, newSpec );
+
+  Todo::List to = todos();
+  for ( i = 0, end = to.count();  i < end;  ++i )
+    to[i]->shiftTimes( oldSpec, newSpec );
+
+  Journal::List jo = journals();
+  for ( i = 0, end = jo.count();  i < end;  ++i )
+    jo[i]->shiftTimes( oldSpec, newSpec );
 }
 
 void Calendar::setFilter( CalFilter *filter )
 {
   if ( filter ) {
-    mFilter = filter;
+    d->mFilter = filter;
   } else {
-    mFilter = mDefaultFilter;
+    d->mFilter = d->mDefaultFilter;
   }
 }
 
 CalFilter *Calendar::filter()
 {
-  return mFilter;
+  return d->mFilter;
 }
 
 QStringList Calendar::categories()
@@ -253,22 +384,22 @@ Event::List Calendar::events( const QDate &date,
                               SortDirection sortDirection )
 {
   Event::List el = rawEventsForDate( date, sortField, sortDirection );
-  mFilter->apply( &el );
+  d->mFilter->apply( &el );
   return el;
 }
 
-Event::List Calendar::events( const QDateTime &qdt )
+Event::List Calendar::events( const KDateTime &dt )
 {
-  Event::List el = rawEventsForDate( qdt );
-  mFilter->apply( &el );
+  Event::List el = rawEventsForDate( dt );
+  d->mFilter->apply( &el );
   return el;
 }
 
 Event::List Calendar::events( const QDate &start, const QDate &end,
-                              bool inclusive)
+                              bool inclusive )
 {
   Event::List el = rawEvents( start, end, inclusive );
-  mFilter->apply( &el );
+  d->mFilter->apply( &el );
   return el;
 }
 
@@ -276,7 +407,7 @@ Event::List Calendar::events( EventSortField sortField,
                               SortDirection sortDirection )
 {
   Event::List el = rawEvents( sortField, sortDirection );
-  mFilter->apply( &el );
+  d->mFilter->apply( &el );
   return el;
 }
 
@@ -284,7 +415,7 @@ bool Calendar::addIncidence( Incidence *incidence )
 {
   Incidence::AddVisitor<Calendar> v( this );
 
-  return incidence->accept(v);
+  return incidence->accept( v );
 }
 
 bool Calendar::deleteIncidence( Incidence *incidence )
@@ -294,19 +425,21 @@ bool Calendar::deleteIncidence( Incidence *incidence )
     bool result = incidence->accept( v );
     endChange( incidence );
     return result;
-  } else
+  } else {
     return false;
+  }
 }
 
-/** Dissociate a single occurrence or all future occurrences from a recurring sequence.
-    The new incidence is returned, but not automatically inserted into the calendar,
-    which is left to the calling application */
+// Dissociate a single occurrence or all future occurrences from a recurring
+// sequence. The new incidence is returned, but not automatically inserted
+// into the calendar, which is left to the calling application.
 Incidence *Calendar::dissociateOccurrence( Incidence *incidence,
                                            const QDate &date,
                                            bool single )
 {
-  if ( !incidence || !incidence->doesRecur() )
+  if ( !incidence || !incidence->doesRecur() ) {
     return 0;
+  }
 
   Incidence *newInc = incidence->clone();
   newInc->recreate();
@@ -315,15 +448,15 @@ Incidence *Calendar::dissociateOccurrence( Incidence *incidence,
   if ( single ) {
     recur->clear();
   } else {
-    // Adjust the recurrence for the future incidences. In particular
-    // adjust the "end after n occurrences" rules! "No end date" and "end by ..."
+    // Adjust the recurrence for the future incidences. In particular adjust
+    // the "end after n occurrences" rules! "No end date" and "end by ..."
     // don't need to be modified.
     int duration = recur->duration();
     if ( duration > 0 ) {
-      int doneduration = recur->durationTo( date.addDays(-1) );
+      int doneduration = recur->durationTo( date.addDays( -1 ) );
       if ( doneduration >= duration ) {
         kDebug(5850) << "The dissociated event already occurred more often "
-                      << "than it was supposed to ever occur. ERROR!" << endl;
+                     << "than it was supposed to ever occur. ERROR!" << endl;
         recur->clear();
       } else {
         recur->setDuration( duration - doneduration );
@@ -333,7 +466,7 @@ Incidence *Calendar::dissociateOccurrence( Incidence *incidence,
   // Adjust the date of the incidence
   if ( incidence->type() == "Event" ) {
     Event *ev = static_cast<Event *>( newInc );
-    QDateTime start( ev->dtStart() );
+    KDateTime start( ev->dtStart() );
     int daysTo = start.date().daysTo( date );
     ev->setDtStart( start.addDays( daysTo ) );
     ev->setDtEnd( ev->dtEnd().addDays( daysTo ) );
@@ -342,15 +475,16 @@ Incidence *Calendar::dissociateOccurrence( Incidence *incidence,
     bool haveOffset = false;
     int daysTo = 0;
     if ( td->hasDueDate() ) {
-      QDateTime due( td->dtDue() );
+      KDateTime due( td->dtDue() );
       daysTo = due.date().daysTo( date );
       td->setDtDue( due.addDays( daysTo ), true );
       haveOffset = true;
     }
     if ( td->hasStartDate() ) {
-      QDateTime start( td->dtStart() );
-      if ( !haveOffset )
+      KDateTime start( td->dtStart() );
+      if ( !haveOffset ) {
         daysTo = start.date().daysTo( date );
+      }
       td->setDtStart( start.addDays( daysTo ) );
       haveOffset = true;
     }
@@ -371,11 +505,15 @@ Incidence *Calendar::dissociateOccurrence( Incidence *incidence,
 Incidence *Calendar::incidence( const QString &uid )
 {
   Incidence *i = event( uid );
-  if ( i )
+  if ( i ) {
     return i;
+  }
+
   i = todo( uid );
-  if ( i )
+  if ( i ) {
     return i;
+  }
+
   i = journal( uid );
   return i;
 }
@@ -384,10 +522,13 @@ Incidence *Calendar::incidenceFromSchedulingID( const QString &sid )
 {
   Incidence::List incidences = rawIncidences();
   Incidence::List::iterator it = incidences.begin();
-  for ( ; it != incidences.end(); ++it )
-    if ( (*it)->schedulingID() == sid )
+  for ( ; it != incidences.end(); ++it ) {
+    if ( (*it)->schedulingID() == sid ) {
       // Touchdown, and the crowd goes wild
       return *it;
+    }
+  }
+
   // Not found
   return 0;
 }
@@ -405,7 +546,7 @@ Todo::List Calendar::sortTodos( Todo::List *todoList,
   // Notice we alphabetically presort Summaries first.
   // We do this so comparison "ties" stay in a nice order.
 
-  // Note that Todos may not have Start DateTimes nor due DateTimes.
+  // Note that To-dos may not have Start DateTimes nor due DateTimes.
 
   switch( sortField ) {
   case TodoSortUnsorted:
@@ -430,15 +571,15 @@ Todo::List Calendar::sortTodos( Todo::List *todoList,
         }
         todoListSorted.insert( sortIt, *eit );
       } else {
-        // Keep a list of the Todos without Start DateTimes
+        // Keep a list of the To-dos without Start DateTimes
         tempList.append( *eit );
       }
     }
     if ( sortDirection == SortDirectionAscending ) {
-      // Append the list of Todos without Start DateTimes
+      // Append the list of To-dos without Start DateTimes
       todoListSorted += tempList;
     } else {
-      // Prepend the list of Todos without Start DateTimes
+      // Prepend the list of To-dos without Start DateTimes
       tempList += todoListSorted;
       todoListSorted = tempList;
     }
@@ -462,15 +603,15 @@ Todo::List Calendar::sortTodos( Todo::List *todoList,
         }
         todoListSorted.insert( sortIt, *eit );
       } else {
-        // Keep a list of the Todos without Due DateTimes
+        // Keep a list of the To-dos without Due DateTimes
         tempList.append( *eit );
       }
     }
     if ( sortDirection == SortDirectionAscending ) {
-      // Append the list of Todos without Due DateTimes
+      // Append the list of To-dos without Due DateTimes
       todoListSorted += tempList;
     } else {
-      // Prepend the list of Todos without Due DateTimes
+      // Prepend the list of To-dos without Due DateTimes
       tempList += todoListSorted;
       todoListSorted = tempList;
     }
@@ -540,14 +681,14 @@ Todo::List Calendar::todos( TodoSortField sortField,
                             SortDirection sortDirection )
 {
   Todo::List tl = rawTodos( sortField, sortDirection );
-  mFilter->apply( &tl );
+  d->mFilter->apply( &tl );
   return tl;
 }
 
 Todo::List Calendar::todos( const QDate &date )
 {
   Todo::List el = rawTodosForDate( date );
-  mFilter->apply( &el );
+  d->mFilter->apply( &el );
   return el;
 }
 
@@ -608,49 +749,52 @@ Journal::List Calendar::journals( JournalSortField sortField,
                                   SortDirection sortDirection )
 {
   Journal::List jl = rawJournals( sortField, sortDirection );
-  mFilter->apply( &jl );
+  d->mFilter->apply( &jl );
   return jl;
 }
 
 Journal::List Calendar::journals( const QDate &date )
 {
   Journal::List el = rawJournalsForDate( date );
-  mFilter->apply( &el );
+  d->mFilter->apply( &el );
   return el;
 }
 
-// When this is called, the todo have already been added to the calendar.
-// This method is only about linking related todos
+// When this is called, the to-dos have already been added to the calendar.
+// This method is only about linking related to-dos.
 void Calendar::setupRelations( Incidence *forincidence )
 {
-  if ( !forincidence ) return;
-// kDebug(5850) << "Calendar::setupRelations for incidence " << forincidence << " with UID " << forincidence->uid() << ", summary: " << forincidence->summary() << endl;
+  if ( !forincidence ) {
+    return;
+  }
+
   QString uid = forincidence->uid();
 
   // First, go over the list of orphans and see if this is their parent
-  QList<Incidence*> l = mOrphans.values( uid );
-  mOrphans.remove( uid );
-  foreach ( Incidence *i, l ) {
-    i->setRelatedTo( forincidence );
-    forincidence->addRelation( i );
-    mOrphanUids.remove( i->uid() );
+  QList<Incidence*> l = d->mOrphans.values( uid );
+  d->mOrphans.remove( uid );
+  for ( int i = 0, end = l.count();  i < end;  ++i ) {
+    l[i]->setRelatedTo( forincidence );
+    forincidence->addRelation( l[i] );
+    d->mOrphanUids.remove( l[i]->uid() );
   }
 
   // Now see about this incidences parent
   if ( !forincidence->relatedTo() && !forincidence->relatedToUid().isEmpty() ) {
     // This incidence has a uid it is related to but is not registered to it yet
     // Try to find it
-    Incidence* parent = incidence( forincidence->relatedToUid() );
+    Incidence *parent = incidence( forincidence->relatedToUid() );
     if ( parent ) {
       // Found it
       forincidence->setRelatedTo( parent );
       parent->addRelation( forincidence );
     } else {
       // Not found, put this in the mOrphans list
-      // Note that the mOrphans dict might have several entries with the same key! That are
-      // multiple children that wait for the parent incidence to be inserted.
-      mOrphans.insert( forincidence->relatedToUid(), forincidence );
-      mOrphanUids.insert( forincidence->uid(), forincidence );
+      // Note that the mOrphans dict might contain multiple entries with the
+      // same key! which are multiple children that wait for the parent
+      // incidence to be inserted.
+      d->mOrphans.insert( forincidence->relatedToUid(), forincidence );
+      d->mOrphanUids.insert( forincidence->uid(), forincidence );
     }
   }
 }
@@ -658,32 +802,28 @@ void Calendar::setupRelations( Incidence *forincidence )
 // If a task with subtasks is deleted, move it's subtasks to the orphans list
 void Calendar::removeRelations( Incidence *incidence )
 {
-  if( !incidence ) {
+  if ( !incidence ) {
     kDebug(5800) << "Warning: Calendar::removeRelations( 0 )!\n";
     return;
   }
 
-// kDebug(5850) << "Calendar::removeRelations for incidence " << forincidence << " with UID " << forincidence->uid() << ", summary: " << forincidence->summary() << endl;
   QString uid = incidence->uid();
-
-  Incidence::List relations = incidence->relations();
-  Incidence::List::ConstIterator it;
-  for ( it = relations.begin(); it != relations.end(); ++it ) {
-    Incidence *i = *it;
-    if ( !mOrphanUids.contains( i->uid() ) ) {
-      mOrphans.insert( uid, i );
-      mOrphanUids.insert( i->uid(), i );
+  foreach ( Incidence *i, incidence->relations() ) {
+    if ( !d->mOrphanUids.contains( i->uid() ) ) {
+      d->mOrphans.insert( uid, i );
+      d->mOrphanUids.insert( i->uid(), i );
       i->setRelatedTo( 0 );
       i->setRelatedToUid( uid );
     }
   }
 
   // If this incidence is related to something else, tell that about it
-  if ( incidence->relatedTo() )
+  if ( incidence->relatedTo() ) {
     incidence->relatedTo()->removeRelation( incidence );
+  }
 
   // Remove this one from the orphans list
-  if ( mOrphanUids.remove( uid ) ) {
+  if ( d->mOrphanUids.remove( uid ) ) {
     // This incidence is located in the orphans list - it should be removed
     // Since the mOrphans dict might contain the same key (with different
     // child incidence pointers!) multiple times, take care that we remove
@@ -694,9 +834,12 @@ void Calendar::removeRelations( Incidence *incidence )
     // changed before its parent is inserted. This might happen with
     // groupware servers....). Remove them, too
     QStringList relatedToUids;
-    // First get the list of all keys in the mOrphans list that point to the removed item
+
+    // First, create a list of all keys in the mOrphans list which point
+    // to the removed item
     relatedToUids << incidence->relatedToUid();
-    for ( QMultiHash<QString, Incidence*>::Iterator it = mOrphans.begin(); it != mOrphans.end(); ++it ) {
+    for ( QMultiHash<QString, Incidence*>::Iterator it = d->mOrphans.begin();
+          it != d->mOrphans.end(); ++it ) {
       if ( it.value()->uid() == uid ) {
         relatedToUids << it.key();
       }
@@ -707,46 +850,55 @@ void Calendar::removeRelations( Incidence *incidence )
           uidit != relatedToUids.end(); ++uidit ) {
       Incidence::List tempList;
       // Remove all to get access to the remaining entries
-      QList<Incidence*> l = mOrphans.values( *uidit );
-      mOrphans.remove( *uidit );
-      foreach ( Incidence* i, l ) {
-        if ( i != incidence ) tempList.append( i );
+      QList<Incidence*> l = d->mOrphans.values( *uidit );
+      d->mOrphans.remove( *uidit );
+      foreach ( Incidence *i, l ) {
+        if ( i != incidence ) {
+          tempList.append( i );
+        }
       }
       // Readd those that point to a different orphan incidence
       for ( Incidence::List::Iterator incit = tempList.begin();
             incit != tempList.end(); ++incit ) {
-        mOrphans.insert( *uidit, *incit );
+        d->mOrphans.insert( *uidit, *incit );
       }
     }
   }
 }
 
-void Calendar::registerObserver( Observer *observer )
+void Calendar::registerObserver( CalendarObserver *observer )
 {
-  if( !mObservers.contains( observer ) )
-    mObservers.append( observer );
-  mNewObserver = true;
+  if ( !d->mObservers.contains( observer ) ) {
+    d->mObservers.append( observer );
+  }
+  d->mNewObserver = true;
 }
 
-void Calendar::unregisterObserver( Observer *observer )
+void Calendar::unregisterObserver( CalendarObserver *observer )
 {
-  mObservers.removeAll( observer );
+  d->mObservers.removeAll( observer );
 }
 
 void Calendar::setModified( bool modified )
 {
-  if ( modified != mModified || mNewObserver ) {
-    mNewObserver = false;
-    foreach ( Observer *observer, mObservers )
+  if ( modified != d->mModified || d->mNewObserver ) {
+    d->mNewObserver = false;
+    foreach ( CalendarObserver *observer, d->mObservers ) {
       observer->calendarModified( modified, this );
-    mModified = modified;
+    }
+    d->mModified = modified;
   }
 }
 
+bool Calendar::isModified() const
+{
+  return d->mModified;
+}
+
 void Calendar::incidenceUpdated( IncidenceBase *incidence )
 {
   incidence->setSyncStatus( Event::SYNCMOD );
-  incidence->setLastModified( QDateTime::currentDateTime() );
+  incidence->setLastModified( KDateTime::currentUtcDateTime() );
   // we should probably update the revision number here,
   // or internally in the Event itself when certain things change.
   // need to verify with ical documentation.
@@ -759,39 +911,50 @@ void Calendar::incidenceUpdated( IncidenceBase *incidence )
 
 void Calendar::notifyIncidenceAdded( Incidence *i )
 {
-  if ( !mObserversEnabled )
+  if ( !d->mObserversEnabled ) {
     return;
+  }
 
-  foreach ( Observer *observer, mObservers )
+  foreach ( CalendarObserver *observer, d->mObservers ) {
     observer->calendarIncidenceAdded( i );
+  }
 }
 
 void Calendar::notifyIncidenceChanged( Incidence *i )
 {
-  if ( !mObserversEnabled )
+  if ( !d->mObserversEnabled ) {
     return;
+  }
 
-  foreach ( Observer *observer, mObservers )
+  foreach ( CalendarObserver *observer, d->mObservers ) {
     observer->calendarIncidenceChanged( i );
+  }
 }
 
 void Calendar::notifyIncidenceDeleted( Incidence *i )
 {
-  if ( !mObserversEnabled )
+  if ( !d->mObserversEnabled ) {
     return;
+  }
 
-  foreach ( Observer *observer, mObservers )
+  foreach ( CalendarObserver *observer, d->mObservers ) {
     observer->calendarIncidenceDeleted( i );
+  }
+}
+
+void Calendar::customPropertyUpdated()
+{
+  setModified( true );
 }
 
 void Calendar::setProductId( const QString &productId )
 {
-  mProductId = productId;
+  d->mProductId = productId;
 }
 
-QString Calendar::productId()
+QString Calendar::productId() const
 {
-  return mProductId;
+  return d->mProductId;
 }
 
 Incidence::List Calendar::mergeIncidenceList( const Event::List &events,
@@ -800,17 +963,18 @@ Incidence::List Calendar::mergeIncidenceList( const Event::List &events,
 {
   Incidence::List incidences;
 
-  Event::List::ConstIterator it1;
-  for ( it1 = events.begin(); it1 != events.end(); ++it1 )
-    incidences.append( *it1 );
+  int i, end;
+  for ( i = 0, end = events.count();  i < end;  ++i ) {
+    incidences.append( events[i] );
+  }
 
-  Todo::List::ConstIterator it2;
-  for ( it2 = todos.begin(); it2 != todos.end(); ++it2 )
-    incidences.append( *it2 );
+  for ( i = 0, end = todos.count();  i < end;  ++i ) {
+    incidences.append( todos[i] );
+  }
 
-  Journal::List::ConstIterator it3;
-  for ( it3 = journals.begin(); it3 != journals.end(); ++it3 )
-    incidences.append( *it3 );
+  for ( i = 0, end = journals.count();  i < end;  ++i ) {
+    incidences.append( journals[i] );
+  }
 
   return incidences;
 }
@@ -827,7 +991,123 @@ bool Calendar::endChange( Incidence * )
 
 void Calendar::setObserversEnabled( bool enabled )
 {
-  mObserversEnabled = enabled;
+  d->mObserversEnabled = enabled;
+}
+
+void Calendar::appendAlarms( Alarm::List &alarms, Incidence *incidence,
+                             const KDateTime &from, const KDateTime &to )
+{
+  KDateTime preTime = from.addSecs(-1);
+
+  Alarm::List alarmlist = incidence->alarms();
+  for ( int i = 0, iend = alarmlist.count();  i < iend;  ++i ) {
+    if ( alarmlist[i]->enabled() ) {
+      KDateTime dt = alarmlist[i]->nextRepetition( preTime );
+      if ( dt.isValid() && dt <= to ) {
+        kDebug(5800) << "Calendar::appendAlarms() '"
+                     << incidence->summary() << "': "
+                     << dt.toString() << endl;
+        alarms.append( alarms[i] );
+      }
+    }
+  }
+}
+
+void Calendar::appendRecurringAlarms( Alarm::List &alarms,
+                                      Incidence *incidence,
+                                      const KDateTime &from,
+                                      const KDateTime &to )
+{
+  KDateTime dt;
+  int  endOffset = 0;
+  bool endOffsetValid = false;
+  int  period = from.secsTo( to );
+
+  Alarm::List alarmlist = incidence->alarms();
+  for ( int i = 0, iend = alarmlist.count();  i < iend;  ++i ) {
+    Alarm *a = alarmlist[i];
+    if ( a->enabled() ) {
+      if ( a->hasTime() ) {
+        // The alarm time is defined as an absolute date/time
+        dt = a->nextRepetition( from.addSecs(-1) );
+        if ( !dt.isValid() || dt > to ) {
+          continue;
+        }
+      } else {
+        // Alarm time is defined by an offset from the event start or end time.
+        // Find the offset from the event start time, which is also used as the
+        // offset from the recurrence time.
+        int offset = 0;
+        if ( a->hasStartOffset() ) {
+          offset = a->startOffset().asSeconds();
+        } else if ( a->hasEndOffset() ) {
+          if ( !endOffsetValid ) {
+            endOffset = incidence->dtStart().secsTo( incidence->dtEnd() );
+            endOffsetValid = true;
+          }
+          offset = a->endOffset().asSeconds() + endOffset;
+        }
+
+        // Find the incidence's earliest alarm
+        KDateTime fromStart = incidence->dtStart().addSecs( offset );
+        if ( fromStart > to ) {
+          continue;
+        }
+        if ( from > fromStart ) {
+          fromStart = from;   // don't look earlier than the earliest alarm
+        }
+
+        // Adjust the 'fromStart' date/time and find the next recurrence at or after it
+        dt = incidence->recurrence()->getNextDateTime( fromStart.addSecs(-offset - 1) );
+        if ( !dt.isValid() ||
+             ( dt = dt.addSecs( offset ) ) > to ) // adjust 'dt' to get the alarm time
+        {
+          // The next recurrence is too late.
+          if ( !a->repeatCount() ) {
+            continue;
+          }
+
+          // The alarm has repetitions, so check whether repetitions of previous
+          // recurrences fall within the time period.
+          bool found = false;
+          dt = fromStart.addSecs( -offset );
+          while ( (dt = incidence->recurrence()->getPreviousDateTime( dt )).isValid() ) {
+            int toFrom = dt.secsTo( fromStart ) - offset;
+            if ( toFrom > a->duration() ) {
+              break;  // this recurrence's last repetition is too early, so give up
+            }
+
+            // The last repetition of this recurrence is at or after 'fromStart' time.
+            // Check if a repetition occurs between 'fromStart' and 'to'.
+            int snooze = a->snoozeTime() * 60;   // in seconds
+            if ( period >= snooze ||
+                 toFrom % snooze == 0 ||
+                 ( toFrom / snooze + 1 ) * snooze <= toFrom + period ) {
+              found = true;
+#ifndef NDEBUG
+              // for debug output
+              dt = dt.addSecs( offset + ( ( toFrom - 1 ) / snooze + 1 ) * snooze );
+#endif
+              break;
+            }
+          }
+          if ( !found ) {
+            continue;
+          }
+        }
+      }
+      kDebug(5800) << "Calendar::appendAlarms() '" << incidence->summary()
+                   << "': " << dt.toString() << endl;
+      alarms.append( a );
+    }
+  }
 }
 
 #include "calendar.moc"
+
+// DEPRECATED methods
+Event::List Calendar::events( const QDateTime &qdt )
+{ return events(KDateTime(qdt, timeSpec())); }
+
+Event::List Calendar::rawEventsForDate( const QDateTime &qdt )
+{ return rawEventsForDate(KDateTime(qdt, timeSpec())); }
