@@ -19,24 +19,47 @@
     Boston, MA 02110-1301, USA.
 */
 /**
- @file calendarnull.cpp
- A null calendar class with does nothing.
+  @file
+  This file is part of the API for handling calendar data and
+  defines the CalendarNull class.
 
- @author Cornelius Schumacher
+  @author Cornelius Schumacher
 */
 #include "calendarnull.h"
 
 using namespace KCal;
 
-CalendarNull::CalendarNull( const QString &timeZoneId )
-  : Calendar( timeZoneId )
+/**
+  Private class that helps to provide binary compatibility between releases.
+  @internal
+*/
+//@cond PRIVATE
+class KCal::CalendarNull::Private
+{
+};
+static CalendarNull *mSelf = 0;
+//@endcond
+
+CalendarNull::CalendarNull( const KDateTime::Spec &timeSpec )
+  : Calendar( timeSpec ),
+    d( new KCal::CalendarNull::Private )
 {}
 
-CalendarNull *CalendarNull::mSelf = 0;
+CalendarNull::CalendarNull( const QString &timeZoneId )
+  : Calendar( timeZoneId ),
+    d( new KCal::CalendarNull::Private )
+{}
+
+CalendarNull::~CalendarNull()
+{
+  delete d;
+}
 
 CalendarNull *CalendarNull::self()
 {
-  if ( !mSelf ) mSelf = new CalendarNull( QLatin1String( "UTC" ) );
+  if ( !mSelf ) {
+    mSelf = new CalendarNull( KDateTime::UTC );
+  }
 
   return mSelf;
 }
