@@ -20,19 +20,20 @@
     Boston, MA 02110-1301, USA.
 */
 /**
-   @file calendarresources.cpp
-   Provides a calendar composed of several calendar resources.
+  @file
+  This file is part of the API for handling calendar data and
+  defines the CalendarResources class.
 
-   @author Cornelius Schumacher
-   @author Reinhold Kainhofer
+  @author Cornelius Schumacher
+  @author Reinhold Kainhofer
 */
 #include <stdlib.h>
 
-#include <QDateTime>
 #include <QString>
 #include <QList>
 
 #include <kdebug.h>
+#include <kdatetime.h>
 #include <kstandarddirs.h>
 #include <klocale.h>
 
@@ -40,28 +41,24 @@
 #include "kresources/selectdialog.h"
 #include "kabc/lock.h"
 
-#include "vcaldrag.h"
-#include "vcalformat.h"
-#include "icalformat.h"
-#include "exceptions.h"
 #include "incidence.h"
 #include "journal.h"
-#include "filestorage.h"
 #include "resourcecalendar.h"
-#include "resourcelocal.h"
 #include "calendarresources.h"
 
 using namespace KCal;
 
 ResourceCalendar
-*CalendarResources::StandardDestinationPolicy::destination( Incidence * )
+*CalendarResources::StandardDestinationPolicy::destination( Incidence *i )
 {
+  Q_UNUSED( i );
   return resourceManager()->standardResource();
 }
 
 ResourceCalendar
-*CalendarResources::AskDestinationPolicy::destination( Incidence * )
+*CalendarResources::AskDestinationPolicy::destination( Incidence *i )
 {
+  Q_UNUSED( i );
   QList<KRES::Resource*> list;
 
   CalendarResourceManager::ActiveIterator it;
@@ -69,18 +66,26 @@ ResourceCalendar
         it != resourceManager()->activeEnd(); ++it ) {
     if ( !(*it)->readOnly() ) {
       //Insert the first the Standard resource to get be the default selected.
-      if ( resourceManager()->standardResource() == *it )
+      if ( resourceManager()->standardResource() == *it ) {
         list.insert( 0, *it );
-      else
+      } else {
         list.append( *it );
+      }
     }
   }
 
   KRES::Resource *r;
-  r = KRES::SelectDialog::getResource( list, mParent );
+  r = KRES::SelectDialog::getResource( list, parent() );
   return static_cast<ResourceCalendar *>( r );
 }
 
+CalendarResources::CalendarResources( const KDateTime::Spec &timeSpec,
+                                      const QString &family )
+  : Calendar( timeSpec )
+{
+  init( family );
+}
+
 CalendarResources::CalendarResources( const QString &timeZoneId,
                                       const QString &family )
   : Calendar( timeZoneId )
@@ -104,6 +109,8 @@ CalendarResources::~CalendarResources()
 {
   close();
   delete mManager;
+  delete mStandardPolicy;
+  delete mAskPolicy;
 }
 
 void CalendarResources::readConfig( KConfig *config )
@@ -128,7 +135,7 @@ void CalendarResources::load()
   // troubles ;-((
   CalendarResourceManager::Iterator i1;
   for ( i1 = mManager->begin(); i1 != mManager->end(); ++i1 ) {
-    (*i1)->setTimeZoneId( timeZoneId() );
+    (*i1)->setTimeSpec( timeSpec() );
   }
 
   QList<ResourceCalendar *> failed;
@@ -156,11 +163,10 @@ void CalendarResources::load()
   mOpen = true;
 }
 
-bool CalendarResources::reload( const QString &tz )
+bool CalendarResources::reload()
 {
   save();
   close();
-  setTimeZoneId( tz );
   load();
   return true;
 }
@@ -175,6 +181,16 @@ void CalendarResources::setAskDestinationPolicy()
   mDestinationPolicy = mAskPolicy;
 }
 
+QWidget *CalendarResources::dialogParentWidget()
+{
+  return mDestinationPolicy->parent();
+}
+void CalendarResources::setDialogParentWidget( QWidget *parent )
+{
+  mDestinationPolicy->setParent( parent );
+}
+
+
 void CalendarResources::close()
 {
   kDebug(5800) << "CalendarResources::close" << endl;
@@ -199,7 +215,6 @@ void CalendarResources::save()
     for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
       (*it)->save();
     }
-
     setModified( false );
   }
 }
@@ -212,7 +227,6 @@ bool CalendarResources::isSaving()
       return true;
     }
   }
-
   return false;
 }
 
@@ -223,9 +237,11 @@ bool CalendarResources::addIncidence( Incidence *incidence,
   bool validRes = false;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    if ( (*it) == resource )
+    if ( (*it) == resource ) {
       validRes = true;
+    }
   }
+
   ResourceCalendar *oldResource = 0;
   if ( mResourceMap.contains( incidence ) ) {
     oldResource = mResourceMap[incidence];
@@ -240,10 +256,11 @@ bool CalendarResources::addIncidence( Incidence *incidence,
     endChange( incidence );
     return true;
   } else {
-    if ( oldResource )
+    if ( oldResource ) {
       mResourceMap[incidence] = oldResource;
-    else
+    } else {
       mResourceMap.remove( incidence );
+    }
   }
 
   return false;
@@ -262,7 +279,6 @@ bool CalendarResources::addIncidence( Incidence *incidence )
       incidence->registerObserver( this );
       notifyIncidenceAdded( incidence );
 
-
       mResourceMap[ incidence ] = resource;
       setModified( true );
       endChange( incidence );
@@ -270,8 +286,9 @@ bool CalendarResources::addIncidence( Incidence *incidence )
     } else {
       mResourceMap.remove( incidence );
     }
-  } else
+  } else {
     kDebug(5800) << "CalendarResources::addIncidence(): no resource" << endl;
+  }
 
   return false;
 }
@@ -294,8 +311,9 @@ bool CalendarResources::deleteEvent( Event *event )
   bool status;
   if ( mResourceMap.find( event ) != mResourceMap.end() ) {
     status = mResourceMap[event]->deleteEvent( event );
-    if ( status )
+    if ( status ) {
       mResourceMap.remove( event );
+    }
   } else {
     status = false;
     CalendarResourceManager::ActiveIterator it;
@@ -341,8 +359,9 @@ bool CalendarResources::deleteTodo( Todo *todo )
   bool status;
   if ( mResourceMap.find( todo ) != mResourceMap.end() ) {
     status = mResourceMap[todo]->deleteTodo( todo );
-    if ( status )
+    if ( status ) {
       mResourceMap.remove( todo );
+    }
   } else {
     CalendarResourceManager::ActiveIterator it;
     status = false;
@@ -362,7 +381,8 @@ Todo::List CalendarResources::rawTodos( TodoSortField sortField,
 
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    appendIncidences<Todo::List>( result, (*it)->rawTodos( TodoSortUnsorted ), *it );
+    appendIncidences<Todo::List>( result,
+                                  (*it)->rawTodos( TodoSortUnsorted ), *it );
   }
   return sortTodos( &result, sortField, sortDirection );
 }
@@ -392,11 +412,10 @@ Todo::List CalendarResources::rawTodosForDate( const QDate &date )
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
     appendIncidences<Todo::List>( result, (*it)->rawTodosForDate( date ), *it );
   }
-
   return result;
 }
 
-Alarm::List CalendarResources::alarmsTo( const QDateTime &to )
+Alarm::List CalendarResources::alarmsTo( const KDateTime &to )
 {
   kDebug(5800) << "CalendarResources::alarmsTo" << endl;
 
@@ -408,8 +427,8 @@ Alarm::List CalendarResources::alarmsTo( const QDateTime &to )
   return result;
 }
 
-Alarm::List CalendarResources::alarms( const QDateTime &from,
-                                       const QDateTime &to )
+Alarm::List CalendarResources::alarms( const KDateTime &from,
+                                       const KDateTime &to )
 {
   Alarm::List result;
   CalendarResourceManager::ActiveIterator it;
@@ -428,7 +447,8 @@ Event::List CalendarResources::rawEventsForDate( const QDate &date,
   Event::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    appendIncidences<Event::List>( result, (*it)->rawEventsForDate( date ), *it );
+    appendIncidences<Event::List>( result,
+                                   (*it)->rawEventsForDate( date ), *it );
   }
   return sortEvents( &result, sortField, sortDirection );
 }
@@ -441,12 +461,13 @@ Event::List CalendarResources::rawEvents( const QDate &start, const QDate &end,
   Event::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    appendIncidences<Event::List>( result, (*it)->rawEvents( start, end, inclusive ), *it );
+    appendIncidences<Event::List>( result,
+                                   (*it)->rawEvents( start, end, inclusive ), *it );
   }
   return result;
 }
 
-Event::List CalendarResources::rawEventsForDate( const QDateTime &qdt )
+Event::List CalendarResources::rawEventsForDate( const KDateTime &qdt )
 {
   kDebug(5800) << "CalendarResources::rawEventsForDate(qdt)" << endl;
 
@@ -454,7 +475,8 @@ Event::List CalendarResources::rawEventsForDate( const QDateTime &qdt )
   Event::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    appendIncidences<Event::List>( result, (*it)->rawEventsForDate( qdt ), *it );
+    appendIncidences<Event::List>( result,
+                                   (*it)->rawEventsForDate( qdt ), *it );
   }
   return result;
 }
@@ -467,12 +489,12 @@ Event::List CalendarResources::rawEvents( EventSortField sortField,
   Event::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    appendIncidences<Event::List>( result, (*it)->rawEvents( EventSortUnsorted ), *it );
+    appendIncidences<Event::List>( result,
+                                   (*it)->rawEvents( EventSortUnsorted ), *it );
   }
   return sortEvents( &result, sortField, sortDirection );
 }
 
-
 bool CalendarResources::addJournal( Journal *journal )
 {
   kDebug(5800) << "CalendarResources::addJournal" << endl;
@@ -486,8 +508,9 @@ bool CalendarResources::deleteJournal( Journal *journal )
   bool status;
   if ( mResourceMap.find( journal ) != mResourceMap.end() ) {
     status = mResourceMap[journal]->deleteJournal( journal );
-    if ( status )
+    if ( status ) {
       mResourceMap.remove( journal );
+    }
   } else {
     CalendarResourceManager::ActiveIterator it;
     status = false;
@@ -501,8 +524,7 @@ bool CalendarResources::deleteJournal( Journal *journal )
 }
 
 bool CalendarResources::addJournal( Journal *journal,
-                                    ResourceCalendar *resource
-  )
+                                    ResourceCalendar *resource )
 {
   return addIncidence( journal, resource );
 }
@@ -532,7 +554,8 @@ Journal::List CalendarResources::rawJournals( JournalSortField sortField,
   Journal::List result;
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    appendIncidences<Journal::List>( result, (*it)->rawJournals( JournalSortUnsorted ), *it );
+    appendIncidences<Journal::List>( result,
+                                     (*it)->rawJournals( JournalSortUnsorted ), *it );
   }
   return sortJournals( &result, sortField, sortDirection );
 }
@@ -544,17 +567,23 @@ Journal::List CalendarResources::rawJournalsForDate( const QDate &date )
 
   CalendarResourceManager::ActiveIterator it;
   for ( it = mManager->activeBegin(); it != mManager->activeEnd(); ++it ) {
-    appendIncidences<Journal::List>( result, (*it)->rawJournalsForDate( date ), *it );
+    appendIncidences<Journal::List>( result,
+                                     (*it)->rawJournalsForDate( date ), *it );
   }
   return result;
 }
 
 template< class IncidenceList >
-void CalendarResources::appendIncidences(IncidenceList &result, const IncidenceList &extra, ResourceCalendar *resource)
+void CalendarResources::appendIncidences( IncidenceList &result,
+                                          const IncidenceList &extra,
+                                          ResourceCalendar *resource )
 {
   result += extra;
-  for (typename IncidenceList::ConstIterator it = extra.begin();  it != extra.end();  ++it)
+  for ( typename IncidenceList::ConstIterator it = extra.begin();
+        it != extra.end();
+        ++it ) {
     mResourceMap[ *it ] = resource;
+  }
 }
 
 void CalendarResources::connectResource( ResourceCalendar *resource )
@@ -584,8 +613,9 @@ void CalendarResources::resourceAdded( ResourceCalendar *resource )
 {
   kDebug(5800) << "Resource added: " << resource->resourceName() << endl;
 
-  if ( !resource->isActive() )
+  if ( !resource->isActive() ) {
     return;
+  }
 
   if ( resource->open() ) {
     resource->load();
@@ -610,41 +640,39 @@ void CalendarResources::resourceDeleted( ResourceCalendar *resource )
   emit signalResourceDeleted( resource );
 }
 
-void CalendarResources::doSetTimeZoneId( const QString &timeZoneId )
+void CalendarResources::doSetTimeSpec( const KDateTime::Spec &timeSpec )
 {
   // set the timezone for all resources. Otherwise we'll have those terrible
   // tz troubles ;-((
   CalendarResourceManager::Iterator i1;
   for ( i1 = mManager->begin(); i1 != mManager->end(); ++i1 ) {
-    (*i1)->setTimeZoneId( timeZoneId );
+    (*i1)->setTimeSpec( timeSpec );
   }
 }
 
-void CalendarResources::setTimeZoneIdViewOnly( const QString &timeZoneId )
-{
-  reload( timeZoneId );
-}
-
 CalendarResources::Ticket
 *CalendarResources::requestSaveTicket( ResourceCalendar *resource )
 {
   kDebug(5800) << "CalendarResources::requestSaveTicket()" << endl;
 
   KABC::Lock *lock = resource->lock();
-  if ( !lock )
+  if ( !lock ) {
     return 0;
-  if ( lock->lock() )
+  }
+  if ( lock->lock() ) {
     return new Ticket( resource );
-  else
+  } else {
     return 0;
+  }
 }
 
 bool CalendarResources::save( Ticket *ticket, Incidence *incidence )
 {
   kDebug(5800) << "CalendarResources::save( Ticket *)" << endl;
 
-  if ( !ticket || !ticket->resource() )
+  if ( !ticket || !ticket->resource() ) {
     return false;
+  }
 
   kDebug(5800) << "tick " << ticket->resource()->resourceName() << endl;
 
@@ -698,8 +726,9 @@ bool CalendarResources::endChange( Incidence *incidence )
   kDebug(5800) << "CalendarResource::endChange()" << endl;
 
   ResourceCalendar *r = resource( incidence );
-  if ( !r )
+  if ( !r ) {
     return false;
+  }
 
   int count = decrementChangeCount( r );
 
@@ -746,14 +775,29 @@ int CalendarResources::decrementChangeCount( ResourceCalendar *r )
   return count;
 }
 
-void CalendarResources::slotLoadError( ResourceCalendar *, const QString &err )
+void CalendarResources::slotLoadError( ResourceCalendar *r, const QString &err )
 {
+  Q_UNUSED( r );
   emit signalErrorMessage( err );
 }
 
-void CalendarResources::slotSaveError( ResourceCalendar *, const QString &err )
+void CalendarResources::slotSaveError( ResourceCalendar *r, const QString &err )
 {
+  Q_UNUSED( r );
   emit signalErrorMessage( err );
 }
 
 #include "calendarresources.moc"
+
+// DEPRECATED methods
+Alarm::List CalendarResources::alarmsTo( const QDateTime &to )
+{ return alarmsTo(KDateTime(to, timeSpec())); }
+
+bool CalendarResources::reload( const QString &tz )
+{
+  save();
+  close();
+  setTimeZoneId( tz );
+  load();
+  return true;
+}
