@@ -20,11 +20,12 @@
     Boston, MA 02110-1301, USA.
 */
 /**
-   @file calendarresources.h
-   Provides a Calendar composed of several Calendar Resources.
+  @file
+  This file is part of the API for handling calendar data and
+  defines the CalendarResources class.
 
-   @author Cornelius Schumacher
-   @author Reinhold Kainhofer
+  @author Cornelius Schumacher
+  @author Reinhold Kainhofer
  */
 #ifndef KCAL_CALENDARRESOURCES_H
 #define KCAL_CALENDARRESOURCES_H
@@ -40,18 +41,12 @@
 
 class QWidget;
 
-/**
-   @namespace KCal
-   Namespace KCal is for global classes, objects and/or functions for the
-   kcal library.
-*/
 namespace KCal {
 
 class CalFormat;
 
 /**
-   @class CalendarResources
-
+   @brief
    This class provides a Calendar which is composed of other Calendars
    known as "Resources".
 
@@ -73,9 +68,13 @@ class KCAL_EXPORT CalendarResources :
     class DestinationPolicy
     {
       public:
-        DestinationPolicy( CalendarResourceManager *manager ) :
-          mManager( manager ) {}
-		virtual ~DestinationPolicy(){}
+        explicit DestinationPolicy( CalendarResourceManager *manager,
+                                    QWidget *parent = 0  ) :
+          mManager( manager ), mParent( parent ) {}
+        virtual ~DestinationPolicy(){}
+
+        virtual QWidget *parent() { return mParent; }
+        virtual void setParent( QWidget *newparent ) { mParent = newparent; }
         virtual ResourceCalendar *destination( Incidence *incidence ) = 0;
 
       protected:
@@ -84,6 +83,7 @@ class KCAL_EXPORT CalendarResources :
 
       private:
         CalendarResourceManager *mManager;
+        QWidget *mParent;
     };
 
     /**
@@ -92,9 +92,10 @@ class KCAL_EXPORT CalendarResources :
     class StandardDestinationPolicy : public DestinationPolicy
     {
       public:
-        StandardDestinationPolicy( CalendarResourceManager *manager ) :
-          DestinationPolicy( manager ) {}
-		virtual ~StandardDestinationPolicy(){}
+        explicit StandardDestinationPolicy( CalendarResourceManager *manager,
+                                            QWidget *parent = 0 ) :
+          DestinationPolicy( manager, parent ) {}
+        virtual ~StandardDestinationPolicy(){}
         ResourceCalendar *destination( Incidence *incidence );
 
       private:
@@ -108,15 +109,13 @@ class KCAL_EXPORT CalendarResources :
     class AskDestinationPolicy : public DestinationPolicy
     {
       public:
-        AskDestinationPolicy( CalendarResourceManager *manager,
-                              QWidget *parent = 0 ) :
-          DestinationPolicy( manager ), mParent( parent ) {}
-		virtual ~AskDestinationPolicy(){}
+        explicit AskDestinationPolicy( CalendarResourceManager *manager,
+                                       QWidget *parent = 0 ) :
+          DestinationPolicy( manager, parent ) {}
+        virtual ~AskDestinationPolicy(){}
         ResourceCalendar *destination( Incidence *incidence );
 
       private:
-        QWidget *mParent;
-
         class Private;
         Private *d;
     };
@@ -141,18 +140,25 @@ class KCAL_EXPORT CalendarResources :
     };
 
     /**
-       Construct CalendarResource object using a Time Zone and a Family name.
+       Construct CalendarResource object using a time specification (time
+       zone, etc.) and a Family name.
 
-       @param timeZoneId is a string containing a Time Zone ID, which is
-       assumed to be valid. The Time Zone Id is used to set the time zone
-       for viewing Incidence dates.\n
-       On some systems, /usr/share/zoneinfo/zone.tab may be available for
-       reference.\n
-       @e Example: "Europe/Berlin"
+       @param timeSpec is a time specification which is used for creating
+       or modifying incidences in the Calendar. It is also used for viewing
+       incidences (see setViewTimeSpec()).
 
-       @warning
-       Do Not pass an empty timeZoneId string as this may cause unintended
-       consequences when storing Incidences into the Calendar.
+       @param family is any QString representing a unique name.
+    */
+    CalendarResources(
+      const KDateTime::Spec &timeSpec,
+      const QString &family = QLatin1String( "calendar" ) );
+
+    /**
+       Construct CalendarResource object using a time zone ID and a Family name.
+
+       @param timeZoneId is used for creating or modifying incidences in the
+       Calendar. It is also used for viewing incidences. The time zone does
+       not alter existing incidences.
 
        @param family is any QString representing a unique name.
     */
@@ -174,10 +180,11 @@ class KCAL_EXPORT CalendarResources :
 
     /**
      * Reloads all incidences from all resources.
-     * @par tz The timezone to set.
      * @return success or failure
      */
-    bool reload( const QString &tz );
+    bool reload();
+    /** Use reload() plus setTimeZoneId() instead. */
+    KDE_DEPRECATED bool reload( const QString &tz );
 
     /**
        Clear out the current Calendar, freeing all used memory etc.
@@ -251,6 +258,23 @@ class KCAL_EXPORT CalendarResources :
     */
     void setAskDestinationPolicy();
 
+    /**
+       Returns the current parent for new dialogs. This is a bad hack, but we need
+       to properly set the parent for the resource selection dialog. Otherwise
+       the dialog will not be modal to the editor dialog in korganizer and
+       the user can still work in the editor dialog (and thus crash korganizer).
+       Afterwards we need to reset it (to avoid pointers to widgets that are
+       already deleted) so we also need the accessor
+    */
+    QWidget *dialogParentWidget();
+    /**
+       Set the widget parent for new dialogs. This is a bad hack, but we need
+       to properly set the parent for the resource selection dialog. Otherwise
+       the dialog will not be modal to the editor dialog in korganizer and
+       the user can still work in the editor dialog (and thus crash korganizer).
+    */
+    void setDialogParentWidget( QWidget *parent );
+
     /**
        Request ticket for saving the Calendar.  If a ticket is returned the
        Calendar is locked for write access until save() or releaseSaveTicket()
@@ -370,12 +394,12 @@ class KCAL_EXPORT CalendarResources :
        Return an unfiltered list of all Events which occur on the given
        timestamp.
 
-       @param qdt request unfiltered Event list for this QDateTime only.
+       @param dt request unfiltered Event list for this KDateTime only.
 
        @return the list of unfiltered Events occurring on the specified
        timestamp.
     */
-    Event::List rawEventsForDate( const QDateTime &qdt );
+    Event::List rawEventsForDate( const KDateTime &dt );
 
     /**
        Return an unfiltered list of all Events occurring within a date range.
@@ -567,27 +591,19 @@ class KCAL_EXPORT CalendarResources :
 
        @return the list of Alarms for the for the specified time range.
     */
-    Alarm::List alarms( const QDateTime &from, const QDateTime &to );
+    Alarm::List alarms( const KDateTime &from, const KDateTime &to );
 
     /**
        Return a list of Alarms that occur before the specified timestamp.
 
        @param to is the ending timestamp.
 
-       @return the list of Alarms occurring before the specified QDateTime.
+       @return the list of Alarms occurring before the specified KDateTime.
     */
-    Alarm::List alarmsTo( const QDateTime &to );
+    Alarm::List alarmsTo( const KDateTime &to );
+    KDE_DEPRECATED Alarm::List alarmsTo( const QDateTime &to );
 
-    /**
-     * Set the viewing time zone, which requires that all resources are saved,
-     * and then reloaded such that the event times are re-interpreted in the new
-     * timezone. Note that the absolute times of events do not change with this.
-     * If you want to change the times of the contents of the resources, use
-     * setTimeZoneId
-     */
-    void setTimeZoneIdViewOnly( const QString& tz );
-
-  signals:
+  Q_SIGNALS:
     /**
        Signal that the Resource has been modified.
     */
@@ -614,18 +630,14 @@ class KCAL_EXPORT CalendarResources :
     void resourceDeleted( ResourceCalendar *resource );
 
     /**
-       Let CalendarResource subclasses set the Time Zone ID.
+       Let CalendarResource subclasses set the time specification (time
+       zone, etc.).
 
-       First parameter is a string containing a Time Zone ID, which is
-       assumed to be valid. On some systems, /usr/share/zoneinfo/zone.tab
-       may be available for reference.\n
-       @e Example: "Europe/Berlin"
-
-       @warning
-       Do Not pass an empty timeZoneId string as this may cause unintended
-       consequences when storing Incidences into the Calendar.
+       @param timeSpec is a time specification which is used for creating
+       or modifying incidences in the Calendar. It is also used for viewing
+       incidences (see setViewTimeSpec()).
     */
-    virtual void doSetTimeZoneId( const QString &timeZoneId );
+    virtual void doSetTimeSpec( const KDateTime::Spec &timeSpec );
 
     /**
        Increment the number of times this Resource has been changed by 1.
@@ -645,7 +657,7 @@ class KCAL_EXPORT CalendarResources :
     */
     int decrementChangeCount( ResourceCalendar *resource );
 
-  protected slots:
+  protected Q_SLOTS:
     void slotLoadError( ResourceCalendar *resource, const QString &err );
     void slotSaveError( ResourceCalendar *resource, const QString &err );
 
@@ -655,7 +667,8 @@ class KCAL_EXPORT CalendarResources :
     */
     void init( const QString &family );
     template< class IncidenceList >
-    void appendIncidences(IncidenceList &result, const IncidenceList &extra, ResourceCalendar *);
+    void appendIncidences( IncidenceList &result, const IncidenceList &extra,
+                           ResourceCalendar * );
 
     bool mOpen;
 
