@@ -33,14 +33,14 @@ class Calendar;
 class KCAL_EXPORT CalStorage
 {
   public:
-    CalStorage( Calendar *calendar )
+    explicit CalStorage( Calendar *calendar ) : d ( 0 )
     {
       mCalendar = calendar;
     }
     virtual ~CalStorage() {}
 
     Calendar *calendar() const { return mCalendar; }
-  
+
     virtual bool open() = 0;
     virtual bool load() = 0;
     virtual bool save() = 0;
@@ -50,7 +50,7 @@ class KCAL_EXPORT CalStorage
     Calendar *mCalendar;
 
     class Private;
-    Private *d;
+    Private *const d;
 };
 
 }
