@@ -7,12 +7,12 @@
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
-    
+
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
-    
+
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
@@ -38,15 +38,16 @@ class KCAL_EXPORT FreeBusyUrlStore
     void writeUrl( const QString &email, const QString &url );
 
     QString readUrl( const QString &email );
-  
+
     void sync();
-  
+
   private:
     FreeBusyUrlStore();
-  
     static FreeBusyUrlStore *mSelf;
-
     KConfig *mConfig;
+
+    class Private;
+    Private *const d;
 };
 
 }
