@@ -44,7 +44,7 @@
 using namespace KCal;
 
 HtmlExport::HtmlExport( Calendar *calendar, HTMLExportSettings *settings ) :
-  mCalendar( calendar ), mSettings( settings )
+  mCalendar( calendar ), mSettings( settings ), d( 0 )
 {
 }
 
@@ -267,7 +267,7 @@ void HtmlExport::createEvent (QTextStream *ts, Event *event,
   kDebug(5850) << "HtmlExport::createEvent(): " << event->summary() << endl;
   *ts << "  <tr>\n";
 
-  if (!event->doesFloat()) {
+  if (!event->floats()) {
     if (event->isMultiDay() && (event->dtStart().date() != date)) {
       *ts << "    <td>&nbsp;</td>\n";
     } else {
@@ -489,19 +489,22 @@ void HtmlExport::createTodo (QTextStream *ts,Todo *todo)
   *ts << "</tr>\n";
 }
 
-void HtmlExport::createWeekView( QTextStream */*ts*/ )
+void HtmlExport::createWeekView( QTextStream *ts )
 {
+  Q_UNUSED( ts );
   // FIXME: Implement this!
 }
 
-void HtmlExport::createJournalView( QTextStream */*ts*/ )
+void HtmlExport::createJournalView( QTextStream *ts )
 {
+  Q_UNUSED( ts );
 //   Journal::List rawJournalList = mCalendar->journals();
   // FIXME: Implement this!
 }
 
-void HtmlExport::createFreeBusyView( QTextStream */*ts*/ )
+void HtmlExport::createFreeBusyView( QTextStream *ts )
 {
+  Q_UNUSED( ts );
   // FIXME: Implement this!
 }
 
