@@ -40,6 +40,7 @@ extern "C" {
 #include "calendar.h"
 #include "calendarlocal.h"
 #include "journal.h"
+#include "icaltimezones.h"
 
 #include "icalformat.h"
 #include "icalformatimpl.h"
@@ -51,12 +52,11 @@ extern "C" {
 
 using namespace KCal;
 
-ICalFormat::ICalFormat() : mImpl(0)
+ICalFormat::ICalFormat() : mImpl(0), d( 0 )
 {
   setImplementation( new ICalFormatImpl( this ) );
 
-  mTimeZoneId = "UTC";
-  mUtc = true;
+  mTimeSpec = KDateTime::UTC;
 }
 
 ICalFormat::~ICalFormat()
@@ -112,9 +112,8 @@ bool ICalFormat::save( Calendar *calendar, const QString &fileName )
   KSaveFile::backupFile( fileName );
 
   KSaveFile file( fileName );
-  if ( file.status() != 0 ) {
-    kDebug(5800) << "ICalFormat::save() errno: " << strerror( file.status() )
-              << endl;
+  if ( !file.open() ) {
+    kDebug(5800) << "ICalFormat::save() err: " << file.errorString() << endl;
     setException( new ErrorFormat( ErrorFormat::SaveError,
                   i18n( "Error saving to '%1'.", fileName ) ) );
     return false;
@@ -122,9 +121,10 @@ bool ICalFormat::save( Calendar *calendar, const QString &fileName )
 
   // Convert to UTF8 and save
   QByteArray textUtf8 = text.toUtf8();
-  file.file()->write( textUtf8.data(), textUtf8.size() - 1 );
+  file.write( textUtf8.data(), textUtf8.size() - 1 );
 
-  if ( !file.close() ) {
+  if ( !file.finalize() ) {
+    kDebug(5800) << "ICalFormat::save() err: " << file.errorString() << endl;
     setException(new ErrorFormat(ErrorFormat::SaveError,
                  i18n("Could not save '%1'", fileName)));
     return false;
@@ -140,8 +140,6 @@ bool ICalFormat::fromString( Calendar *cal, const QString &text )
 
 bool ICalFormat::fromRawString( Calendar *cal, const QByteArray &text )
 {
-  setTimeZone( cal->timeZoneId(), !cal->isLocalTime() );
-
   // Get first VCALENDAR component.
   // TODO: Handle more than one VCALENDAR or non-VCALENDAR top components
   icalcomponent *calendar;
@@ -160,7 +158,7 @@ bool ICalFormat::fromRawString( Calendar *cal, const QByteArray &text )
   if (icalcomponent_isa(calendar) == ICAL_XROOT_COMPONENT) {
     icalcomponent *comp;
     for ( comp = icalcomponent_get_first_component(calendar, ICAL_VCALENDAR_COMPONENT);
-          comp != 0; comp = icalcomponent_get_next_component(calendar, ICAL_VCALENDAR_COMPONENT) ) {
+          comp; comp = icalcomponent_get_next_component(calendar, ICAL_VCALENDAR_COMPONENT) ) {
       // put all objects into their proper places
       if ( !mImpl->populate( cal, comp ) ) {
         kDebug(5800) << "ICalFormat::load(): Could not populate calendar" << endl;
@@ -195,7 +193,7 @@ bool ICalFormat::fromRawString( Calendar *cal, const QByteArray &text )
 
 Incidence *ICalFormat::fromString( const QString &text )
 {
-  CalendarLocal cal( mTimeZoneId );
+  CalendarLocal cal( mTimeSpec );
   fromString(&cal, text);
 
   Incidence *ical = 0;
@@ -219,19 +217,20 @@ Incidence *ICalFormat::fromString( const QString &text )
 
 QString ICalFormat::toString( Calendar *cal )
 {
-  setTimeZone( cal->timeZoneId(), !cal->isLocalTime() );
-
   icalcomponent *calendar = mImpl->createCalendarComponent(cal);
 
   icalcomponent *component;
 
+  ICalTimeZones *tzlist = cal->timeZones();   // time zones possibly used in the calendar
+  ICalTimeZones tzUsedList;                   // time zones actually used in the calendar
+
   // todos
   Todo::List todoList = cal->rawTodos();
   Todo::List::ConstIterator it;
   for( it = todoList.begin(); it != todoList.end(); ++it ) {
 //    kDebug(5800) << "ICalFormat::toString() write todo "
 //                  << (*it)->uid() << endl;
-    component = mImpl->writeTodo( *it );
+    component = mImpl->writeTodo( *it, tzlist, &tzUsedList );
     icalcomponent_add_component( calendar, component );
   }
 
@@ -244,7 +243,7 @@ QString ICalFormat::toString( Calendar *cal )
     {
       // kDebug(5800) << "ICalFormat::toString() write event "
       //             << (*it2)->uid() << endl;
-      component = mImpl->writeEvent( *it2 );
+      component = mImpl->writeEvent( *it2, tzlist, &tzUsedList );
       icalcomponent_add_component( calendar, component );
     }
   }
@@ -255,10 +254,23 @@ QString ICalFormat::toString( Calendar *cal )
   for( it3 = journals.begin(); it3 != journals.end(); ++it3 ) {
     kDebug(5800) << "ICalFormat::toString() write journal "
                   << (*it3)->uid() << endl;
-    component = mImpl->writeJournal( *it3 );
+    component = mImpl->writeJournal( *it3, tzlist, &tzUsedList );
     icalcomponent_add_component( calendar, component );
   }
 
+  // time zones
+  const ICalTimeZones::ZoneMap zones = tzUsedList.zones();
+  for ( ICalTimeZones::ZoneMap::ConstIterator it = zones.begin();  it != zones.end();  ++it) {
+    icaltimezone *tz = (*it)->icalTimezone();
+    if ( !tz ) {
+      kError(5800) << "ICalFormat::toString(): bad time zone" << endl;
+    } else {
+      component = icalcomponent_new_clone( icaltimezone_get_component( tz ) );
+      icalcomponent_add_component( calendar, component );
+      icaltimezone_free( tz, 1 );
+    }
+  }
+
   QString text = QString::fromUtf8( icalcomponent_as_ical_string( calendar ) );
 
   icalcomponent_free( calendar );
@@ -275,7 +287,7 @@ QString ICalFormat::toString( Calendar *cal )
 
 QString ICalFormat::toICalString( Incidence *incidence )
 {
-  CalendarLocal cal( mTimeZoneId );
+  CalendarLocal cal( mTimeSpec );
   cal.addIncidence( incidence->clone() );
   return toString( &cal );
 }
@@ -391,7 +403,7 @@ FreeBusy *ICalFormat::parseFreeBusy( const QString &str )
 ScheduleMessage *ICalFormat::parseScheduleMessage( Calendar *cal,
                                                    const QString &messageText )
 {
-  setTimeZone( cal->timeZoneId(), !cal->isLocalTime() );
+  setTimeSpec( cal->timeSpec() );
   clearException();
 
   if (messageText.isEmpty())
@@ -417,26 +429,30 @@ ScheduleMessage *ICalFormat::parseScheduleMessage( Calendar *cal,
     return 0;
   }
 
+  // Populate the message's time zone collection with all VTIMEZONE components
+  ICalTimeZones tzlist;
+  ICalTimeZoneSource tzs;
+  tzs.parse(message, tzlist);
+
   icalcomponent *c;
 
   IncidenceBase *incidence = 0;
   c = icalcomponent_get_first_component(message,ICAL_VEVENT_COMPONENT);
   if (c) {
-    icalcomponent *ctz = icalcomponent_get_first_component(message,ICAL_VTIMEZONE_COMPONENT);
-    incidence = mImpl->readEvent(c, ctz);
+    incidence = mImpl->readEvent(c, &tzlist);
   }
 
   if (!incidence) {
     c = icalcomponent_get_first_component(message,ICAL_VTODO_COMPONENT);
     if (c) {
-      incidence = mImpl->readTodo(c);
+      incidence = mImpl->readTodo(c, &tzlist);
     }
   }
 
   if (!incidence) {
     c = icalcomponent_get_first_component(message,ICAL_VJOURNAL_COMPONENT);
     if (c) {
-      incidence = mImpl->readJournal(c);
+      incidence = mImpl->readJournal(c, &tzlist);
     }
   }
 
@@ -563,18 +579,18 @@ ScheduleMessage *ICalFormat::parseScheduleMessage( Calendar *cal,
   return new ScheduleMessage(incidence,method,status);
 }
 
-void ICalFormat::setTimeZone( const QString &id, bool utc )
+void ICalFormat::setTimeSpec( const KDateTime::Spec &timeSpec )
 {
-  mTimeZoneId = id;
-  mUtc = utc;
+  mTimeSpec = timeSpec;
+}
+
+KDateTime::Spec ICalFormat::timeSpec() const
+{
+  return mTimeSpec;
 }
 
 QString ICalFormat::timeZoneId() const
 {
-  return mTimeZoneId;
-}
-
-bool ICalFormat::utc() const
-{
-  return mUtc;
+  const KTimeZone *tz = mTimeSpec.timeZone();
+  return tz ? tz->name() : QString();
 }
