@@ -22,12 +22,14 @@
 #include <QDateTime>
 #include <QString>
 #include <QList>
+#include <QVector>
 #include <QSet>
 #include <QFile>
 #include <QTextStream>
 
-#include <kdebug.h>
+#include <ksystemtimezone.h>
 #include <kdatetime.h>
+#include <kdebug.h>
 
 extern "C" {
   #include <ical.h>
@@ -42,6 +44,10 @@ extern "C" {
 using namespace KCal;
 
 
+// Minimum repetition counts for VTIMEZONE RRULEs
+static const int minRuleCount = 5;   // for any RRULE
+static const int minPhaseCount = 8;  // for separate STANDARD/DAYLIGHT component
+
 // Convert an ical time to QDateTime, preserving the UTC indicator
 static QDateTime toQDateTime(const icaltimetype &t)
 {
@@ -205,20 +211,26 @@ const ICalTimeZone *ICalTimeZones::utc()
 /******************************************************************************/
 
 ICalTimeZone::ICalTimeZone(ICalTimeZoneSource *source, const QString &name, ICalTimeZoneData *data)
-  : KTimeZone(source, name)
+  : KTimeZone(source, name), d( 0 )
 {
   setData(data);
 }
 
-ICalTimeZone::ICalTimeZone(const KTimeZone &tz)
-  : KTimeZone(0, tz.name(), tz.countryCode(), tz.latitude(), tz.longitude(), tz.comment())
+ICalTimeZone::ICalTimeZone(const KTimeZone &tz, const QDate &earliest)
+  : KTimeZone(0, tz.name(), tz.countryCode(), tz.latitude(), tz.longitude(), tz.comment()), d( 0 )
 {
-  if (tz.data(true))
-    setData( new ICalTimeZoneData(*tz.data(), tz) );
+  const KTimeZoneData *data = tz.data(true);
+  if (data) {
+    const ICalTimeZoneData *icaldata = dynamic_cast<const ICalTimeZoneData*>(data);
+    if ( icaldata )
+      setData( new ICalTimeZoneData(*icaldata) );
+    else
+      setData( new ICalTimeZoneData(*data, tz, earliest) );
+  }
 }
 
 ICalTimeZone::ICalTimeZone(const ICalTimeZone &tz)
-  : KTimeZone(tz)
+  : KTimeZone(tz), d( 0 )
 {
 }
 
@@ -262,6 +274,11 @@ icaltimezone *ICalTimeZone::icalTimezone() const
   return dat ? dat->icalTimezone() : 0;
 }
 
+bool ICalTimeZone::hasTransitions() const
+{
+    return true;
+}
+
 
 /******************************************************************************/
 
@@ -274,6 +291,7 @@ public:
       if (icalComponent)
         icalcomponent_free(icalComponent);
     }
+    icalcomponent *component() const  { return icalComponent; }
     void setComponent(icalcomponent *c)
     {
       if (icalComponent)
@@ -283,6 +301,7 @@ public:
     QString       location;       // name of city for this time zone
     QByteArray    url;            // URL of published VTIMEZONE definition (optional)
     QDateTime     lastModified;   // time of last modification of the VTIMEZONE component (optional)
+private:
     icalcomponent *icalComponent; // ical component representing this time zone
 };
 
@@ -299,90 +318,256 @@ ICalTimeZoneData::ICalTimeZoneData(const ICalTimeZoneData &rhs)
   d->location      = rhs.d->location;
   d->url           = rhs.d->url;
   d->lastModified  = rhs.d->lastModified;
-  d->icalComponent = icalcomponent_new_clone( rhs.d->icalComponent );
+  d->setComponent( icalcomponent_new_clone( rhs.d->component() ) );
 }
 
-struct Transition {
-  QDateTime start;
-  int phase;
-  bool done;
-  bool operator<(const Transition &rhs) const { return start < rhs.start; }   // for qSort()
-};
-
-ICalTimeZoneData::ICalTimeZoneData(const KTimeZoneData &rhs, const KTimeZone &tz)
+ICalTimeZoneData::ICalTimeZoneData(const KTimeZoneData &rhs, const KTimeZone &tz, const QDate &earliest)
   : KTimeZoneData(rhs),
     d(new ICalTimeZoneDataPrivate())
 {
+  // VTIMEZONE RRULE types
+  enum {
+    DAY_OF_MONTH          = 0x01,
+    WEEKDAY_OF_MONTH      = 0x02,
+    LAST_WEEKDAY_OF_MONTH = 0x04
+  };
+
   if (dynamic_cast<const KSystemTimeZone*>(&tz)) {
-    icaltimezone *itz = icaltimezone_get_builtin_timezone( tz.name().toUtf8() );
-    d->icalComponent = icalcomponent_new_clone( icaltimezone_get_component( itz ) );
+    // Try to fetch a system time zone in preference, on the grounds
+    // that system time zones are more likely to be up to date than
+    // built-in libical ones.
+    icalcomponent *c = 0;
+    KTimeZone *ktz = KSystemTimeZones::readZone( tz.name() );
+    if ( ktz ) {
+      if ( ktz->data(true) ) {
+        ICalTimeZone icaltz( *ktz, earliest );
+        icaltimezone *itz = icaltz.icalTimezone();
+        c = icalcomponent_new_clone( icaltimezone_get_component( itz ) );
+        icaltimezone_free( itz, 1 );
+      }
+      delete ktz;
+    }
+    if ( !c ) {
+      // Try to fetch a built-in libical time zone.
+      icaltimezone *itz = icaltimezone_get_builtin_timezone( tz.name().toUtf8() );
+      c = icalcomponent_new_clone( icaltimezone_get_component( itz ) );
+    }
+    if ( c ) {
+      // TZID in built-in libical time zones has a standard prefix.
+      // To make the VTIMEZONE TZID match TZID references in incidences
+      // (as required by RFC2445), strip off the prefix.
+      icalproperty *prop = icalcomponent_get_first_property( c, ICAL_TZID_PROPERTY );
+      if ( prop ) {
+        icalvalue* value = icalproperty_get_value( prop );
+        const char *tzid = icalvalue_get_text( value );
+        QByteArray icalprefix = ICalTimeZoneSource::icalTzidPrefix();
+        int len = icalprefix.size();
+        if ( !strncmp( icalprefix, tzid, len ) ) {
+          const char *s = strchr( tzid + len, '/' );    // find third '/'
+          if ( s ) {
+            QByteArray tzidShort( s + 1 );    // deep copy of string (needed by icalvalue_set_text())
+            icalvalue_set_text( value, tzidShort );
+
+            // Remove the X-LIC-LOCATION property, which is only used by libical
+            prop = icalcomponent_get_first_property( c, ICAL_X_PROPERTY );
+            const char *xname = icalproperty_get_x_name( prop );
+            if ( xname  &&  !strcmp(xname, "X-LIC-LOCATION") )
+              icalcomponent_remove_property( c, prop );
+          }
+        }
+      }
+    }
+    d->setComponent( c );
   }
   else {
     // Write the time zone data into an iCal component
-    icalcomponent *comp = icalcomponent_new(ICAL_VTIMEZONE_COMPONENT);
-#warning tzid or location?
-    icalcomponent_add_property(comp, icalproperty_new_tzid( tz.name().toUtf8() ));
-//    icalcomponent_add_property(comp, icalproperty_new_tzurl( ??.toUtf8() ));
-//    icalcomponent_add_property(comp, icalproperty_new_location( ??.toUtf8() ));
+    icalcomponent *tzcomp = icalcomponent_new(ICAL_VTIMEZONE_COMPONENT);
+    icalcomponent_add_property(tzcomp, icalproperty_new_tzid( tz.name().toUtf8() ));
+//    icalcomponent_add_property(tzcomp, icalproperty_new_location( tz.name().toUtf8() ));
 
     // Compile an ordered list of transitions so that we can know the phases
     // which occur before and after each transition.
-    Transition transition;
-    transition.done = false;
-    QList<Transition> transitions;
-    QList<KTimeZonePhase> tzphases = rhs.phases();
-    for (int p = 0, pend = tzphases.count();  p < pend;  ++p) {
-      transition.phase = p;
-      QList<QDateTime> starts = tzphases[p].starts();
-      for (int t = 0, tend = starts.count();  t < tend;  ++t) {
-        transition.start = starts[t];
-        transitions += transition;
+    QList<KTimeZone::Transition> transits = transitions();
+    if ( earliest.isValid() ) {
+      // Remove all transitions earlier than those we are interested in
+      for ( int i = 0, end = transits.count();  i < end;  ++i ) {
+        if ( transits[i].time().date() >= earliest) {
+          if ( i > 0 )
+            transits.erase(transits.begin(), transits.begin() + i);
+          break;
+        }
       }
     }
-    qSort(transitions);
+    int trcount = transits.count();
+    QVector<bool> transitionsDone(trcount);
+    transitionsDone.fill(false);
 
-    // Go through the list of transitions and create an iCal component for
-    // each distinct combination of phases before and after the transition.
-    bool found;
+    // Go through the list of transitions and create an iCal component for each
+    // distinct combination of phase after and UTC offset before the transition.
     icaldatetimeperiodtype dtperiod;
     dtperiod.period = icalperiodtype_null_period();
-    do {
-      found = false;
-      for (int i = 0, end = transitions.count();  i < end;  ++i) {
-        if ( transitions[i].done )
-          continue;
-        found = true;
-        int preOffset = (i > 0) ? tzphases[ transitions[i-1].phase ].utcOffset() : rhs.previousUtcOffset();
-        int postPhase = transitions[i].phase;
-        KTimeZonePhase &phase = tzphases[postPhase];
-        icalcomponent *phcomp = icalcomponent_new( phase.isDst() ?
-                                   ICAL_XDAYLIGHT_COMPONENT : ICAL_XSTANDARD_COMPONENT );
-        QList<QByteArray> abbrevs = phase.abbreviations();
-        for (int a = 0, aend = abbrevs.count();  a < aend;  ++a) {
-          icalcomponent_add_property(phcomp, icalproperty_new_tzname( static_cast<const char*>(abbrevs[a]) ));
-        }
-        if ( !phase.comment().isEmpty() )
-          icalcomponent_add_property(phcomp, icalproperty_new_comment( phase.comment().toUtf8() ));
-        icalcomponent_add_property(phcomp, icalproperty_new_tzoffsetfrom( preOffset ));
-        icalcomponent_add_property(phcomp, icalproperty_new_tzoffsetto( phase.utcOffset() ));
-        icalcomponent_add_property(phcomp, icalproperty_new_dtstart(
-                          writeLocalICalDateTime( transitions[i].start, preOffset ) ));
-        transitions[i].done = true;
+    for ( ; ; ) {
+      int i = 0;
+      for ( ;  i < trcount && transitionsDone[i];  ++i) ;
+      if ( i >= trcount )
+        break;
+      // Found a phase combination which hasn't yet been processed
+      int preOffset = (i > 0) ? transits[i-1].phase().utcOffset() : rhs.previousUtcOffset();
+      KTimeZone::Phase phase = transits[i].phase();
+      icalcomponent *phaseComp = icalcomponent_new( phase.isDst() ?
+                                 ICAL_XDAYLIGHT_COMPONENT : ICAL_XSTANDARD_COMPONENT );
+      QList<QByteArray> abbrevs = phase.abbreviations();
+      for (int a = 0, aend = abbrevs.count();  a < aend;  ++a) {
+        icalcomponent_add_property(phaseComp, icalproperty_new_tzname( static_cast<const char*>(abbrevs[a]) ));
+      }
+      if ( !phase.comment().isEmpty() )
+        icalcomponent_add_property(phaseComp, icalproperty_new_comment( phase.comment().toUtf8() ));
+      icalcomponent_add_property(phaseComp, icalproperty_new_tzoffsetfrom( preOffset ));
+      icalcomponent_add_property(phaseComp, icalproperty_new_tzoffsetto( phase.utcOffset() ));
+      // Create a component to hold initial RRULE if any, plus all RDATEs
+      icalcomponent *phaseComp1 = icalcomponent_new_clone( phaseComp );
+      icalcomponent_add_property(phaseComp1, icalproperty_new_dtstart(
+                        writeLocalICalDateTime( transits[i].time(), preOffset ) ));
+      bool useNewRRULE = false;
 
-        while (++i < end) {
-          if (!transitions[i].done 
-          &&  transitions[i].phase == postPhase
-          &&  tzphases[ transitions[i-1].phase ].utcOffset() == preOffset) {
-            dtperiod.time = writeLocalICalDateTime( transitions[i].start, preOffset );
-            icalcomponent_add_property(phcomp, icalproperty_new_rdate( dtperiod ));
-            transitions[i].done = true;
+      // Compile the list of UTC transition dates/times, and
+      // check if the list can be reduced to an RRULE instead of multiple RDATEs.
+      QTime time;
+      QDate date;
+      int year = 0, month = 0, daysInMonth = 0, dayOfMonth = 0;  // initialise to avoid compiler warnings
+      int dayOfWeek = 0;      // Monday = 1
+      int nthFromStart = 0;   // nth (weekday) of month
+      int nthFromEnd = 0;     // nth last (weekday) of month
+      int newRule;
+      int rule = 0;
+      QList<QDateTime> rdates;   // dates which (probably) need to be written as RDATEs
+      QList<QDateTime> times;
+      QDateTime qdt = transits[i].time();   // set 'qdt' for start of loop
+      times += qdt;
+      transitionsDone[i] = true;
+      do {
+        if (!rule) {
+          // Initialise data for detecting a new rule
+          rule         = DAY_OF_MONTH | WEEKDAY_OF_MONTH | LAST_WEEKDAY_OF_MONTH;
+          time         = qdt.time();
+          date         = qdt.date();
+          year         = date.year();
+          month        = date.month();
+          daysInMonth  = date.daysInMonth();
+          dayOfWeek    = date.dayOfWeek();   // Monday = 1
+          dayOfMonth   = date.day();
+          nthFromStart = (dayOfMonth - 1)/7 + 1;   // nth (weekday) of month
+          nthFromEnd   = (daysInMonth - dayOfMonth)/7 + 1;   // nth last (weekday) of month
+        }
+        if (++i >= trcount) {
+          newRule = 0;
+          times += QDateTime();   // append a dummy value since last value in list is ignored
+        } else {
+          if (transitionsDone[i]
+          ||  transits[i].phase() != phase
+          ||  transits[i-1].phase().utcOffset() != preOffset)
+            continue;
+          newRule = rule;
+          transitionsDone[i] = true;
+          qdt = transits[i].time();
+          times += qdt;
+          date = qdt.date();
+          if (qdt.time() != time
+          ||  date.month() != month
+          ||  date.year() != ++year) {
+            newRule = 0;
+          } else {
+            int day = date.day();
+            if ((newRule & DAY_OF_MONTH)  &&  day != dayOfMonth)
+              newRule &= ~DAY_OF_MONTH;
+            if (newRule & (WEEKDAY_OF_MONTH | LAST_WEEKDAY_OF_MONTH)) {
+              if (date.dayOfWeek() != dayOfWeek) {
+                newRule &= ~(WEEKDAY_OF_MONTH | LAST_WEEKDAY_OF_MONTH);
+              } else {
+                if ((newRule & WEEKDAY_OF_MONTH)  &&  (day - 1)/7 + 1 != nthFromStart)
+                  newRule &= ~WEEKDAY_OF_MONTH;
+                if ((newRule & LAST_WEEKDAY_OF_MONTH)  &&  (daysInMonth - day)/7 + 1 != nthFromEnd)
+                  newRule &= ~LAST_WEEKDAY_OF_MONTH;
+              }
+            }
           }
         }
-        icalcomponent_add_component(comp, phcomp);
-      }
-    } while (found);
+        if (!newRule) {
+          // The previous rule (if any) no longer applies.
+          // Write all the times up to but not including the current one.
+          // First check whether any of the last RDATE values fit this rule.
+          int yr = times[0].date().year();
+	  while (!rdates.isEmpty()) {
+            qdt = rdates.last();
+	    date = qdt.date();
+	    if (qdt.time() != time
+            ||  date.month() != month
+	    ||  date.year() != --yr)
+              break;
+	    int day  = date.day();
+            if (rule & DAY_OF_MONTH) {
+              if (day != dayOfMonth)
+                break;
+	    } else {
+              if (date.dayOfWeek() != dayOfWeek
+              ||  (rule & WEEKDAY_OF_MONTH)  &&  (day - 1)/7 + 1 != nthFromStart
+              ||  (rule & LAST_WEEKDAY_OF_MONTH)  &&  (daysInMonth - day)/7 + 1 != nthFromEnd)
+		break;
+	    }
+	    times.prepend(qdt);
+	    rdates.pop_back();
+	  }
+          if (times.count() > (useNewRRULE ? minPhaseCount : minRuleCount)) {
+            // There are enough dates to combine into an RRULE
+            icalrecurrencetype r;
+            icalrecurrencetype_clear(&r);
+            r.freq = ICAL_YEARLY_RECURRENCE;
+            r.count = (year >= 2030) ? 0 : times.count() - 1;
+            r.by_month[0] = month;
+            if (rule & DAY_OF_MONTH)
+              r.by_month_day[0] = dayOfMonth;
+            else if (rule & WEEKDAY_OF_MONTH)
+              r.by_day[0] = (dayOfWeek % 7 + 1) + (nthFromStart * 8);   // Sunday = 1
+            else if (rule & LAST_WEEKDAY_OF_MONTH)
+              r.by_day[0] = -(dayOfWeek % 7 + 1) - (nthFromEnd * 8);   // Sunday = 1
+            icalproperty *prop = icalproperty_new_rrule(r);
+            if (useNewRRULE) {
+              // This RRULE doesn't start from the phase start date, so set it into
+              // a new STANDARD/DAYLIGHT component in the VTIMEZONE.
+              icalcomponent *c = icalcomponent_new_clone( phaseComp );
+              icalcomponent_add_property(c, icalproperty_new_dtstart(
+                                writeLocalICalDateTime( times[0], preOffset ) ));
+              icalcomponent_add_property(c, prop);
+              icalcomponent_add_component(tzcomp, c);
+            } else {
+              icalcomponent_add_property(phaseComp1, prop);
+            }
+          } else {
+            // Save dates for writing as RDATEs
+            for ( int t = 0, tend = times.count() - 1;  t < tend;  ++t ) {
+              rdates += times[t];
+            }
+          }
+          useNewRRULE = true;
+          // All date/time values but the last have been added to the VTIMEZONE.
+          // Remove them from the list.
+          qdt = times.last();   // set 'qdt' for start of loop
+          times.clear();
+          times += qdt;
+        }
+        rule = newRule;
+      } while (i < trcount);
 
-    d->icalComponent = comp;
+      // Write remaining dates as RDATEs
+      for ( int rd = 0, rdend = rdates.count();  rd < rdend;  ++rd ) {
+        dtperiod.time = writeLocalICalDateTime( rdates[rd], preOffset );
+        icalcomponent_add_property(phaseComp1, icalproperty_new_rdate( dtperiod ));
+      }
+      icalcomponent_add_component(tzcomp, phaseComp1);
+      icalcomponent_free(phaseComp);
+    }
+
+    d->setComponent( tzcomp );
   }
 }
 
@@ -397,7 +582,7 @@ ICalTimeZoneData &ICalTimeZoneData::operator=(const ICalTimeZoneData &rhs)
   d->location      = rhs.d->location;
   d->url           = rhs.d->url;
   d->lastModified  = rhs.d->lastModified;
-  d->setComponent( icalcomponent_new_clone(rhs.d->icalComponent) );
+  d->setComponent( icalcomponent_new_clone(rhs.d->component()) );
   return *this;
 }
 
@@ -423,7 +608,7 @@ QDateTime ICalTimeZoneData::lastModified() const
 
 QByteArray ICalTimeZoneData::vtimezone() const
 {
-  return icalcomponent_as_ical_string( d->icalComponent );
+  return icalcomponent_as_ical_string( d->component() );
 }
 
 icaltimezone *ICalTimeZoneData::icalTimezone() const
@@ -431,25 +616,33 @@ icaltimezone *ICalTimeZoneData::icalTimezone() const
   icaltimezone *icaltz = icaltimezone_new();
   if ( !icaltz )
     return 0;
-  if ( !icaltimezone_set_component(icaltz, d->icalComponent) )
-  {
-    icaltimezone_free(icaltz, 1);
+  icalcomponent *c = icalcomponent_new_clone( d->component() );
+  if ( !icaltimezone_set_component( icaltz, c ) ) {
+    icalcomponent_free( c );
+    icaltimezone_free( icaltz, 1 );
     return 0;
   }
   return icaltz;
 }
 
+bool ICalTimeZoneData::hasTransitions() const
+{
+    return true;
+}
+
 
 /******************************************************************************/
 
 class ICalTimeZoneSourcePrivate
 {
   public:
-    static KTimeZonePhase *parsePhase(icalcomponent*, bool daylight, int &prevOffset);
+    static QList<QDateTime> parsePhase(icalcomponent*, bool daylight, int &prevOffset, KTimeZone::Phase&);
+    static QByteArray icalTzidPrefix;
 };
+QByteArray ICalTimeZoneSourcePrivate::icalTzidPrefix;
 
 
-ICalTimeZoneSource::ICalTimeZoneSource()
+ICalTimeZoneSource::ICalTimeZoneSource() : d( 0 )
 {
 }
 
@@ -506,7 +699,6 @@ ICalTimeZone *ICalTimeZoneSource::parse(icalcomponent *vtimezone)
     switch (kind) {
 
       case ICAL_TZID_PROPERTY:
-#warning Should location be used for name rather than tzid?
         name = QString::fromUtf8(icalproperty_get_tzid(p));
         break;
 
@@ -547,6 +739,13 @@ ICalTimeZone *ICalTimeZoneSource::parse(icalcomponent *vtimezone)
   }
   if (data->d->location.isEmpty()  &&  !xlocation.isEmpty())
     data->d->location = xlocation;
+  QString prefix = QString::fromUtf8(icalTzidPrefix());
+  if (name.startsWith(prefix)) {
+    // Remove the prefix from libical built in time zone TZID
+    int i = name.indexOf('/', prefix.length());
+    if (i > 0)
+      name = name.mid(i + 1);
+  }
   //kDebug(5800) << "---zoneId: \"" << name << '"' << endl;
 
   /*
@@ -554,42 +753,59 @@ ICalTimeZone *ICalTimeZoneSource::parse(icalcomponent *vtimezone)
    * and create a Phase object containing details for each one.
    */
   int prevOffset = 0;
+  QList<KTimeZone::Transition> transitions;
   QDateTime earliest;
-  QList<KTimeZonePhase> phases;
+  QList<KTimeZone::Phase> phases;
   for (icalcomponent *c = icalcomponent_get_first_component(vtimezone, ICAL_ANY_COMPONENT);
        c;  c = icalcomponent_get_next_component(vtimezone, ICAL_ANY_COMPONENT))
   {
     int prevoff;
-    KTimeZonePhase *phase = 0;
+    KTimeZone::Phase phase;
+    QList<QDateTime> times;
     icalcomponent_kind kind = icalcomponent_isa(c);
     switch (kind) {
 
       case ICAL_XSTANDARD_COMPONENT:
         //kDebug(5800) << "---standard phase: found" << endl;
-        phase = ICalTimeZoneSourcePrivate::parsePhase(c, false, prevoff);
+        times = ICalTimeZoneSourcePrivate::parsePhase(c, false, prevoff, phase);
         break;
 
       case ICAL_XDAYLIGHT_COMPONENT:
         //kDebug(5800) << "---daylight phase: found" << endl;
-        phase = ICalTimeZoneSourcePrivate::parsePhase(c, true, prevoff);
+        times = ICalTimeZoneSourcePrivate::parsePhase(c, true, prevoff, phase);
         break;
 
       default:
         kDebug(5800) << "ICalTimeZoneSource::parse(): Unknown component: " << kind << endl;
         break;
     }
-    if (phase  &&  phase->isValid()) {
-      phases += *phase;
-      if (!earliest.isValid()  ||  phase->start(0) < earliest) {
+    int tcount = times.count();
+    if (tcount) {
+      phases += phase;
+      for (int t = 0;  t < tcount;  ++t)
+        transitions += KTimeZone::Transition(times[t], phase);
+      if (!earliest.isValid()  ||  times[0] < earliest) {
         prevOffset = prevoff;
-        earliest = phase->start(0);
+        earliest = times[0];
       }
     }
-    delete phase;
   }
   data->setPhases(phases, prevOffset);
+  // Remove any "duplicate" transitions, i.e. those where two consecutive
+  // transitions have the same phase.
+  qSort(transitions);
+  for (int t = 1, tend = transitions.count();  t < tend; ) {
+    if (transitions[t].phase() == transitions[t-1].phase()) {
+      transitions.removeAt(t);
+      --tend;
+    } else {
+      ++t;
+    }
+  }
+  data->setTransitions(transitions);
 
   data->d->setComponent( icalcomponent_new_clone(vtimezone) );
+  kDebug(5800) << "ICalTimeZoneSource::parse(): VTIMEZONE " << name << endl;
   return new ICalTimeZone(this, name, data);
 }
 
@@ -602,8 +818,11 @@ ICalTimeZone *ICalTimeZoneSource::parse(icaltimezone *tz)
   return parse(icaltimezone_get_component(tz));
 }
 
-KTimeZonePhase *ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool daylight, int &prevOffset)
+QList<QDateTime> ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool daylight, int &prevOffset,
+                                      KTimeZone::Phase &phase)
 {
+  QList<QDateTime> transitions;
+
   // Read the observance data for this standard/daylight savings phase
   QList<QByteArray> abbrevs;
   QString comment;
@@ -625,14 +844,16 @@ KTimeZonePhase *ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool day
       {
         // TZNAME can appear multiple times in order to provide language
         // translations of the time zone offset name.
+#ifdef __GNUC__
 #warning Does this cope with multiple language specifications?
+#endif
         QByteArray tzname = icalproperty_get_tzname(p);
         // Outlook (2000) places "Standard Time" and "Daylight Time" in the TZNAME
         // strings, which is totally useless. So ignore those.
         if (!daylight  &&  tzname == "Standard Time"
         ||  daylight  &&  tzname == "Daylight Time")
           break;
-        if (abbrevs.indexOf(tzname))
+        if (!abbrevs.contains(tzname))
           abbrevs += tzname;
         break;
       }
@@ -670,7 +891,7 @@ KTimeZonePhase *ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool day
   // Validate the phase data
   if (!found_dtstart || !found_tzoffsetfrom || !found_tzoffsetto) {
     kDebug(5800) << "ICalTimeZoneSource::readPhase(): DTSTART/TZOFFSETFROM/TZOFFSETTO missing" << endl;
-    return 0;
+    return transitions;
   }
 
   // Convert DTSTART to QDateTime, and from local time to UTC
@@ -679,8 +900,7 @@ KTimeZonePhase *ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool day
   dtstart.is_utc = 1;
   QDateTime utcStart = toQDateTime(icaltime_normalize(dtstart));   // UTC
 
-  QList<QDateTime> times;
-  times += utcStart;
+  transitions += utcStart;
   if (recurs) {
     /* RDATE or RRULE is specified. There should only be one or the other, but
      * it doesn't really matter - the code can cope with both.
@@ -713,7 +933,7 @@ KTimeZonePhase *ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool day
             t.is_utc = 1;
             t = icaltime_normalize(t);
           }
-          times += toQDateTime(t);
+          transitions += toQDateTime(t);
           break;
         }
         case ICAL_RRULE_PROPERTY:
@@ -722,26 +942,21 @@ KTimeZonePhase *ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool day
           ICalFormat icf;
           ICalFormatImpl impl(&icf);
           impl.readRecurrence(icalproperty_get_rrule(p), &r);
-#warning temporary build hack
-          r.setStartDt(klocalStart.dateTime());
+          r.setStartDt(klocalStart);
           // The end date time specified in an RRULE should be in UTC.
-          // Convert to local time to avoid datesInInterval() getting things wrong.
+          // Convert to local time to avoid timesInInterval() getting things wrong.
           if (r.duration() == 0) {
-            KDateTime end = r.endDt();
+            KDateTime end(r.endDt());
             if (end.timeSpec() == KDateTime::Spec::UTC) {
               end.setTimeSpec(KDateTime::Spec::ClockTime);
-#warning temporary build hack
-              r.setEndDt( end.addSecs(prevOffset).dateTime() );
+              r.setEndDt( end.addSecs(prevOffset) );
             }
           }
-#warning temporary build hack
-          DateTimeList dts = r.datesInInterval(klocalStart.dateTime(), maxTime.dateTime());
+          DateTimeList dts = r.timesInInterval(klocalStart, maxTime);
           for ( int i = 0, end = dts.count();  i < end;  ++i) {
-#warning temporary build hack
-            // QDateTime utc = dts[i].dateTime();
-	    QDateTime utc = dts[i];
+            QDateTime utc = dts[i].dateTime();
             utc.setTimeSpec(Qt::UTC);
-            times += utc.addSecs(-prevOffset);
+            transitions += utc.addSecs(-prevOffset);
           }
           break;
         }
@@ -750,11 +965,65 @@ KTimeZonePhase *ICalTimeZoneSourcePrivate::parsePhase(icalcomponent *c, bool day
       }
       p = icalcomponent_get_next_property(c, ICAL_ANY_PROPERTY);
     }
-    qSortUnique(times);
+    qSortUnique(transitions);
   }
 
-  return new KTimeZonePhase(times, utcOffset, abbrevs, daylight, comment);
+  phase = KTimeZone::Phase(utcOffset, abbrevs, daylight, comment);
+  return transitions;
 }
 
+ICalTimeZone *ICalTimeZoneSource::standardZone(const QString &zone, bool icalBuiltIn)
+{
+  if ( !icalBuiltIn ) {
+    // Try to fetch a system time zone in preference, on the grounds
+    // that system time zones are more likely to be up to date than
+    // built-in libical ones.
+    QString tzid = zone;
+    QString prefix = QString::fromUtf8( icalTzidPrefix() );
+    if ( zone.startsWith( prefix ) ) {
+      int i = zone.indexOf('/', prefix.length() );
+      if ( i > 0 )
+        tzid = zone.mid( i + 1 );   // strip off the libical prefix
+    }
+    KTimeZone *ktz = KSystemTimeZones::readZone( tzid );
+    if ( ktz ) {
+      if ( ktz->data(true) ) {
+        ICalTimeZone *icaltz = new ICalTimeZone( *ktz );
+        delete ktz;
+        kDebug(5800) << "ICalTimeZoneSource::standardZone(" << zone << "): read from system database" << endl;
+        return icaltz;
+      }
+      delete ktz;
+    }
+  }
+  // Try to fetch a built-in libical time zone.
+  // First try to look it up as a geographical location (e.g. Europe/London)
+  QByteArray zoneName = zone.toUtf8();
+  icaltimezone *icaltz = icaltimezone_get_builtin_timezone( zoneName );
+  if ( !icaltz ) {
+    // This will find it if it includes the libical prefix
+    icaltz = icaltimezone_get_builtin_timezone_from_tzid( zoneName );
+    if ( !icaltz )
+      return 0;
+  }
+  return parse( icaltz );
+}
+
+QByteArray ICalTimeZoneSource::icalTzidPrefix()
+{
+  if (ICalTimeZoneSourcePrivate::icalTzidPrefix.isEmpty()) {
+    icaltimezone *icaltz = icaltimezone_get_builtin_timezone("Europe/London");
+    QByteArray tzid = icaltimezone_get_tzid( icaltz );
+    if (tzid.right(13) == "Europe/London") {
+      int i = tzid.indexOf('/', 1);
+      if (i > 0) {
+        ICalTimeZoneSourcePrivate::icalTzidPrefix = tzid.left(i + 1);
+        return ICalTimeZoneSourcePrivate::icalTzidPrefix;
+      }
+    }
+    kError(5800) << "ICalTimeZoneSource: failed to get libical TZID prefix" << endl;
+  }
+  return ICalTimeZoneSourcePrivate::icalTzidPrefix;
+}
 
 }  // namespace KCal
