@@ -27,14 +27,15 @@
 #include <kdebug.h>
 
 #include "calformat.h"
+
 #include "incidence.h"
 
 using namespace KCal;
 
 Incidence::Incidence() :
   IncidenceBase(),
-  mRelatedTo(0), mStatus(StatusNone), mSecrecy(SecrecyPublic),
-  mPriority(5), mRecurrence(0)
+  mRelatedTo( 0 ), mStatus( StatusNone ), mSecrecy( SecrecyPublic ),
+  mPriority( 0 ), mRecurrence( 0 ), d( 0 )
 {
   recreate();
 
@@ -42,7 +43,8 @@ Incidence::Incidence() :
   mAttachments.setAutoDelete(true);
 }
 
-Incidence::Incidence( const Incidence &i ) : IncidenceBase( i ),Recurrence::Observer()
+Incidence::Incidence( const Incidence &i ) :
+  IncidenceBase( i ), Recurrence::RecurrenceObserver(), d( 0 )
 {
 // TODO: reenable attributes currently commented out.
   mRevision = i.mRevision;
@@ -93,7 +95,7 @@ Incidence::~Incidence()
     Incidence::List Relations = mRelations;
     List::ConstIterator it;
     for ( it = Relations.begin(); it != Relations.end(); ++it ) {
-        if ( (*it)->relatedTo() == this ) (*it)->setRelatedTo( 0 );
+      if ( (*it)->relatedTo() == this ) (*it)->mRelatedTo = 0;
     }
     if ( relatedTo() ) relatedTo()->removeRelation( this );
 
@@ -103,9 +105,7 @@ Incidence::~Incidence()
 // A string comparison that considers that null and empty are the same
 static bool stringCompare( const QString& s1, const QString& s2 )
 {
-    if ( s1.isEmpty() && s2.isEmpty() )
-        return true;
-    return s1 == s2;
+  return ( s1.isEmpty() && s2.isEmpty() ) || (s1 == s2);
 }
 
 bool Incidence::operator==( const Incidence& i2 ) const
@@ -156,14 +156,15 @@ bool Incidence::operator==( const Incidence& i2 ) const
 
 void Incidence::recreate()
 {
-  setCreated(QDateTime::currentDateTime());
+  KDateTime nowUTC = KDateTime::currentUtcDateTime();
+  setCreated(nowUTC);
 
   setUid(CalFormat::createUniqueId());
   setSchedulingID( QString() );
 
   setRevision(0);
 
-  setLastModified(QDateTime::currentDateTime());
+  setLastModified(nowUTC);
   setPilotId( 0 );
   setSyncStatus( SYNCNONE );
 }
@@ -183,16 +184,16 @@ void Incidence::setFloats(bool f)
   IncidenceBase::setFloats( f );
 }
 
-void Incidence::setCreated( const QDateTime &created )
+void Incidence::setCreated( const KDateTime &created )
 {
   if (mReadOnly) return;
-  mCreated = created;
+  mCreated = created.toUtc();
 
 // FIXME: Shouldn't we call updated for the creation date, too?
 //  updated();
 }
 
-QDateTime Incidence::created() const
+KDateTime Incidence::created() const
 {
   return mCreated;
 }
@@ -210,15 +211,24 @@ int Incidence::revision() const
   return mRevision;
 }
 
-void Incidence::setDtStart(const QDateTime &dtStart)
+void Incidence::setDtStart(const KDateTime &dtStart)
 {
   if ( mRecurrence ) {
     mRecurrence->setStartDateTime( dtStart );
-    mRecurrence->setFloats( doesFloat() );
+    mRecurrence->setFloats( floats() );
   }
   IncidenceBase::setDtStart( dtStart );
 }
 
+void Incidence::shiftTimes(const KDateTime::Spec &oldSpec, const KDateTime::Spec &newSpec)
+{
+  IncidenceBase::shiftTimes( oldSpec, newSpec );
+  if ( mRecurrence )
+    mRecurrence->shiftTimes( oldSpec, newSpec );
+  for ( int i = 0, end = mAlarms.count();  i < end;  ++i )
+    mAlarms[i]->shiftTimes( oldSpec, newSpec );
+}
+
 void Incidence::setDescription(const QString &description)
 {
   if (mReadOnly) return;
@@ -338,7 +348,7 @@ Recurrence *Incidence::recurrence() const
   {
     mRecurrence = new Recurrence();
     mRecurrence->setStartDateTime( IncidenceBase::dtStart() );
-    mRecurrence->setFloats( doesFloat() );
+    mRecurrence->setFloats( floats() );
     mRecurrence->setRecurReadOnly( mReadOnly );
     mRecurrence->addObserver( const_cast<KCal::Incidence*>(this) );
   }
@@ -364,12 +374,12 @@ bool Incidence::doesRecur() const
   else return false;
 }
 
-bool Incidence::recursOn(const QDate &qd) const
+bool Incidence::recursOn(const QDate &qd, const KDateTime::Spec &timeSpec) const
 {
-  return ( mRecurrence && mRecurrence->recursOn(qd) );
+  return ( mRecurrence && mRecurrence->recursOn(qd, timeSpec) );
 }
 
-bool Incidence::recursAt(const QDateTime &qdt) const
+bool Incidence::recursAt(const KDateTime &qdt) const
 {
   return ( mRecurrence && mRecurrence->recursAt(qdt) );
 }
@@ -382,13 +392,13 @@ bool Incidence::recursAt(const QDateTime &qdt) const
   @return the start date/time of all occurrences that overlap with the given
       date. Empty list if the incidence does not overlap with the date at all
 */
-QList<QDateTime> Incidence::startDateTimesForDate( const QDate &date ) const
+QList<KDateTime> Incidence::startDateTimesForDate( const QDate &date, const KDateTime::Spec &timeSpec ) const
 {
 //kDebug(5800) << "Incidence::startDateTimesForDate " << date << ", incidence=" << summary() << endl;
-  QDateTime start = dtStart();
-  QDateTime end = endDateRecurrenceBase();
+  KDateTime start = dtStart();
+  KDateTime end = endDateRecurrenceBase();
 
-  QList<QDateTime> result;
+  QList<KDateTime> result;
 
   // TODO_Recurrence: Also work if only due date is given...
   if ( !start.isValid() && ! end.isValid() ) {
@@ -396,8 +406,9 @@ QList<QDateTime> Incidence::startDateTimesForDate( const QDate &date ) const
   }
 
   // if the incidence doesn't recur,
+  KDateTime kdate( date, timeSpec );
   if ( !doesRecur() ) {
-    if ( !(start.date() > date || end.date() < date ) ) {
+    if ( !(start > kdate || end < kdate ) ) {
       result << start;
     }
     return result;
@@ -406,13 +417,13 @@ QList<QDateTime> Incidence::startDateTimesForDate( const QDate &date ) const
   int days = start.daysTo( end );
   // Account for possible recurrences going over midnight, while the original event doesn't
   QDate tmpday( date.addDays( -days - 1 ) );
-  QDateTime tmp;
+  KDateTime tmp;
   while ( tmpday <= date ) {
-    if ( recurrence()->recursOn( tmpday ) ) {
-      QList<QTime> times = recurrence()->recurTimesOn( tmpday );
+    if ( recurrence()->recursOn( tmpday, timeSpec ) ) {
+      QList<QTime> times = recurrence()->recurTimesOn( tmpday, timeSpec );
       for ( QList<QTime>::ConstIterator it = times.begin(); it != times.end(); ++it ) {
-        tmp = QDateTime( tmpday, *it );
-        if ( endDateForStart( tmp ).date() >= date )
+        tmp = KDateTime( tmpday, *it, start.timeSpec() );
+        if ( endDateForStart( tmp ) >= kdate )
           result << tmp;
       }
     }
@@ -429,13 +440,13 @@ QList<QDateTime> Incidence::startDateTimesForDate( const QDate &date ) const
       date/time. Empty list if the incidence does not happen at the given
       time at all.
 */
-QList<QDateTime> Incidence::startDateTimesForDateTime( const QDateTime &datetime ) const
+QList<KDateTime> Incidence::startDateTimesForDateTime( const KDateTime &datetime ) const
 {
 // kDebug(5800) << "Incidence::startDateTimesForDateTime " << datetime << ", incidence=" << summary() << endl;
-  QDateTime start = dtStart();
-  QDateTime end = endDateRecurrenceBase();
+  KDateTime start = dtStart();
+  KDateTime end = endDateRecurrenceBase();
 
-  QList<QDateTime> result;
+  QList<KDateTime> result;
 
   // TODO_Recurrence: Also work if only due date is given...
   if ( !start.isValid() && ! end.isValid() ) {
@@ -453,12 +464,13 @@ QList<QDateTime> Incidence::startDateTimesForDateTime( const QDateTime &datetime
   int days = start.daysTo( end );
   // Account for possible recurrences going over midnight, while the original event doesn't
   QDate tmpday( datetime.date().addDays( -days - 1 ) );
-  QDateTime tmp;
+  KDateTime tmp;
   while ( tmpday <= datetime.date() ) {
-    if ( recurrence()->recursOn( tmpday ) ) {
-      QList<QTime> times = recurrence()->recurTimesOn( tmpday );
+    if ( recurrence()->recursOn( tmpday, datetime.timeSpec() ) ) {
+      // Get the times during the day (in start date's time zone) when recurrences happen
+      QList<QTime> times = recurrence()->recurTimesOn( tmpday, start.timeSpec() );
       for ( QList<QTime>::ConstIterator it = times.begin(); it != times.end(); ++it ) {
-        tmp = QDateTime( tmpday, *it );
+        tmp = KDateTime( tmpday, *it, start.timeSpec() );
         if ( !(tmp > datetime || endDateForStart( tmp ) < datetime ) )
           result << tmp;
       }
@@ -469,10 +481,10 @@ QList<QDateTime> Incidence::startDateTimesForDateTime( const QDateTime &datetime
 }
 
 /** Return the end time of the occurrence if it starts at the given date/time */
-QDateTime Incidence::endDateForStart( const QDateTime &startDt ) const
+KDateTime Incidence::endDateForStart( const KDateTime &startDt ) const
 {
-  QDateTime start = dtStart();
-  QDateTime end = endDateRecurrenceBase();
+  KDateTime start = dtStart();
+  KDateTime end = endDateRecurrenceBase();
   if ( !end.isValid() ) return start;
   if ( !start.isValid() ) return end;
 
@@ -511,7 +523,7 @@ void Incidence::setExDateTimes( const DateTimeList &exDates )
   updated();
 }
 
-void Incidence::addExDateTime( const QDateTime &date )
+void Incidence::addExDateTime( const KDateTime &date )
 {
   if ( mReadOnly ) return;
   recurrence()->addExDateTime( date );
@@ -555,7 +567,7 @@ void Incidence::setRDateTimes( const DateTimeList &exDates )
   updated();
 }
 
-void Incidence::addRDateTime( const QDateTime &date )
+void Incidence::addRDateTime( const KDateTime &date )
 {
   if ( mReadOnly ) return;
   recurrence()->addRDateTime( date );
@@ -797,4 +809,3 @@ void Incidence::recurrenceUpdated( Recurrence *recurrence )
   if ( recurrence == mRecurrence )
     updated();
 }
-
