@@ -24,19 +24,16 @@
 #include <time.h>
 
 #include <QBuffer>
-#include <QDateTime>
 #include <QList>
 #include <QTextDocument>
 
 #include <kapplication.h>
+#include <kdatetime.h>
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <klocale.h>
 
 #include "emailfunctions/email.h"
-#include "ktnef/ktnefparser.h"
-#include "ktnef/ktnefmessage.h"
-#include "ktnef/ktnefdefs.h"
 #include "kabc/phonenumber.h"
 #include "kabc/vcardconverter.h"
 #include "kabc/stdaddressbook.h"
@@ -52,7 +49,6 @@
 #include "freebusy.h"
 
 using namespace KCal;
-using namespace KTnef;
 
 /*******************************************************************
  *  Helper functions for the extensive display (event viewer)
@@ -159,8 +155,8 @@ static QString eventViewerFormatAttendees( Incidence *event )
   QString tmpStr;
   Attendee::List attendees = event->attendees();
   if ( attendees.count() ) {
-    KIconLoader iconLoader;
-    const QString iconPath = iconLoader.iconPath( "mail_generic",
+    KIconLoader* iconLoader = KIconLoader::global();
+    const QString iconPath = iconLoader->iconPath( "mail_generic",
                                                   K3Icon::Small );
 
     // Add organizer link
@@ -212,8 +208,8 @@ static QString eventViewerFormatBirthday( Event *event )
   QString name_1 = event->customProperty("KABC","NAME-1");
   QString email_1= event->customProperty("KABC","EMAIL-1");
 
-  KIconLoader iconLoader;
-  const QString iconPath = iconLoader.iconPath( "mail_generic",
+  KIconLoader* iconLoader = KIconLoader::global();
+  const QString iconPath = iconLoader->iconPath( "mail_generic",
                                                   K3Icon::Small );
   //TODO: add a tart icon
   QString tmpString = "<ul>";
@@ -236,28 +232,27 @@ static QString eventViewerFormatHeader( Incidence *incidence )
 
   // show icons
   {
-    KIconLoader iconLoader;
-
+    KIconLoader* iconLoader = KIconLoader::global();
     tmpStr += "<td>";
 
     if ( incidence->type() == "Todo" ) {
       tmpStr += "<img src=\"" +
-                iconLoader.iconPath( "todo", K3Icon::Small ) +
+                iconLoader->iconPath( "todo", K3Icon::Small ) +
                 "\">";
     }
     if ( incidence->isAlarmEnabled() ) {
       tmpStr += "<img src=\"" +
-                iconLoader.iconPath( "bell", K3Icon::Small ) +
+                iconLoader->iconPath( "bell", K3Icon::Small ) +
                 "\">";
     }
     if ( incidence->doesRecur() ) {
       tmpStr += "<img src=\"" +
-                iconLoader.iconPath( "recur", K3Icon::Small ) +
+                iconLoader->iconPath( "recur", K3Icon::Small ) +
                 "\">";
     }
     if ( incidence->isReadOnly() ) {
       tmpStr += "<img src=\"" +
-                iconLoader.iconPath( "readonlyevent", K3Icon::Small ) +
+                iconLoader->iconPath( "readonlyevent", K3Icon::Small ) +
                 "\">";
     }
 
@@ -284,7 +279,7 @@ static QString eventViewerFormatEvent( Event *event )
   }
 
   tmpStr += "<tr>";
-  if ( event->doesFloat() ) {
+  if ( event->floats() ) {
     if ( event->isMultiDay() ) {
       tmpStr += "<td align=\"right\"><b>" + i18n( "Time" ) + "</b></td>";
       tmpStr += "<td>" + i18nc("<beginTime> - <endTime>","%1 - %2",
@@ -335,17 +330,17 @@ static QString eventViewerFormatEvent( Event *event )
 
   if ( event->categories().count() > 0 ) {
     tmpStr += "<tr>";
-    tmpStr += "<td align=\"right\"><b>" + i18np( "1 category", "%n categories", event->categories().count() )+ "</b></td>";
+    tmpStr += "<td align=\"right\"><b>" + i18np( "1 category", "%1 categories", event->categories().count() )+ "</b></td>";
     tmpStr += "<td>" + event->categoriesStr() + "</td>";
     tmpStr += "</tr>";
   }
 
   if ( event->doesRecur() ) {
-    QDateTime dt = event->recurrence()->getNextDateTime(
-                                          QDateTime::currentDateTime() );
+    KDateTime dt = event->recurrence()->getNextDateTime(
+                                          KDateTime::currentUtcDateTime() );
     tmpStr += "<tr>";
     tmpStr += "<td align=\"right\"><b>" + i18n( "Next Occurrence" )+ "</b></td>";
-    tmpStr += "<td>" + KGlobal::locale()->formatDateTime( dt, true ) + "</td>";
+    tmpStr += "<td>" + KGlobal::locale()->formatDateTime( dt.dateTime(), true ) + "</td>";
     tmpStr += "</tr>";
   }
 
@@ -356,14 +351,14 @@ static QString eventViewerFormatEvent( Event *event )
   int attachmentCount = event->attachments().count();
   if ( attachmentCount > 0 ) {
     tmpStr += "<tr>";
-    tmpStr += "<td align=\"right\"><b>" + i18np( "1 attachment", "%n attachments", attachmentCount )+ "</b></td>";
+    tmpStr += "<td align=\"right\"><b>" + i18np( "1 attachment", "%1 attachments", attachmentCount )+ "</b></td>";
     tmpStr += "<td>" + eventViewerFormatAttachments( event ) + "</td>";
     tmpStr += "</tr>";
   }
 
   tmpStr += "</table>";
   tmpStr += "<p><em>" + i18n( "Creation date: %1.",
-    KGlobal::locale()->formatDateTime( event->created() , true ) ) + "</em>";
+    KGlobal::locale()->formatDateTime( event->created().dateTime(), true ) ) + "</em>";
   return tmpStr;
 }
 
@@ -386,24 +381,24 @@ static QString eventViewerFormatTodo( Todo *todo )
   tmpStr += eventViewerFormatCategories( todo );
 
   if ( todo->priority() > 0 )
-    tmpStr += i18n("<p><b>Priority:</b> %2</p>", todo->priority());
+    tmpStr += i18n("<p><b>Priority:</b> %1</p>", todo->priority());
   else
-    tmpStr += i18n("<p><b>Priority:</b> %2</p>", i18n("Unspecified"));
+    tmpStr += i18n("<p><b>Priority:</b> %1</p>", i18n("Unspecified"));
 
   tmpStr += i18n("<p><i>%1 % completed</i></p>",
                   todo->percentComplete() );
 
   if ( todo->doesRecur() ) {
-    QDateTime dt = todo->recurrence()->getNextDateTime(
-                                         QDateTime::currentDateTime() );
+    KDateTime dt = todo->recurrence()->getNextDateTime(
+                                         KDateTime::currentUtcDateTime() );
     tmpStr += eventViewerAddTag( "p", "<em>" +
       i18n("This is a recurring to-do. The next occurrence will be on %1.",
-      KGlobal::locale()->formatDateTime( dt, true ) ) + "</em>" );
+      KGlobal::locale()->formatDateTime( dt.dateTime(), true ) ) + "</em>" );
   }
   tmpStr += eventViewerFormatAttendees( todo );
   tmpStr += eventViewerFormatAttachments( todo );
   tmpStr += "<p><em>" + i18n( "Creation date: %1.",
-    KGlobal::locale()->formatDateTime( todo->created() , true ) ) + "</em>";
+    KGlobal::locale()->formatDateTime( todo->created().dateTime(), true ) ) + "</em>";
   return tmpStr;
 }
 
@@ -437,18 +432,18 @@ static QString eventViewerFormatFreeBusy( FreeBusy *fb )
       int dur = per.duration().asSeconds();
       QString cont;
       if ( dur >= 3600 ) {
-        cont += i18np("1 hour ", "%n hours ", dur / 3600 );
+        cont += i18np("1 hour ", "%1 hours ", dur / 3600 );
         dur %= 3600;
       }
       if ( dur >= 60 ) {
-        cont += i18np("1 minute ", "%n minutes ", dur / 60);
+        cont += i18np("1 minute ", "%1 minutes ", dur / 60);
         dur %= 60;
       }
       if ( dur > 0 ) {
-        cont += i18np("1 second", "%n seconds", dur);
+        cont += i18np("1 second", "%1 seconds", dur);
       }
       text += i18nc("startDate for duration", "%1 for %2",
-            KGlobal::locale()->formatDateTime( per.start(), false ) ,
+            KGlobal::locale()->formatDateTime( per.start().dateTime(), false ) ,
             cont );
       text += "<br>";
     } else {
@@ -459,8 +454,8 @@ static QString eventViewerFormatFreeBusy( FreeBusy *fb )
               KGlobal::locale()->formatTime( per.end().time() ) );
       } else {
         text += i18nc("fromDateTime - toDateTime", "%1 - %2",
-            KGlobal::locale()->formatDateTime( per.start(), false ) ,
-            KGlobal::locale()->formatDateTime( per.end(), false ) );
+            KGlobal::locale()->formatDateTime( per.start().dateTime(), false ) ,
+            KGlobal::locale()->formatDateTime( per.end().dateTime(), false ) );
       }
       text += "<br>";
     }
@@ -469,7 +464,7 @@ static QString eventViewerFormatFreeBusy( FreeBusy *fb )
   return tmpStr;
 }
 
-class IncidenceFormatter::EventViewerVisitor : public IncidenceBase::Visitor
+class KCal::IncidenceFormatter::EventViewerVisitor : public IncidenceBase::Visitor
 {
   public:
     EventViewerVisitor() { mResult = ""; }
@@ -520,7 +515,7 @@ QString IncidenceFormatter::extensiveDisplayString( IncidenceBase *incidence )
 
 static QString string2HTML( const QString& str )
 {
-  return Qt::convertFromPlainText(str);
+  return Qt::convertFromPlainText( str, Qt::WhiteSpaceNormal );
 }
 
 static QString invitationRow( const QString &cell1, const QString &cell2 )
@@ -557,7 +552,7 @@ static QString invitationDetailsEvent( Event* event )
   html += invitationRow( i18n( "Where:" ), sLocation );
 
   // Meeting Start Time Row
-  if ( ! event->doesFloat() ) {
+  if ( ! event->floats() ) {
     tmp =  i18nc("%1: Start Date, %2: Start Time", "%1 %2",
                  event->dtStartDateStr(), event->dtStartTimeStr() );
   } else {
@@ -568,7 +563,7 @@ static QString invitationDetailsEvent( Event* event )
 
   // Meeting End Time Row
   if ( event->hasEndDate() ) {
-    if ( ! event->doesFloat() ) {
+    if ( ! event->floats() ) {
       tmp =  i18nc("%1: End Date, %2: End Time", "%1 %2",
                    event->dtEndDateStr(), event->dtEndTimeStr() );
     } else {
@@ -581,16 +576,16 @@ static QString invitationDetailsEvent( Event* event )
   html += invitationRow( i18n( "End Time:" ), tmp );
 
   // Meeting Duration Row
-  if ( !event->doesFloat() && event->hasEndDate() ) {
+  if ( !event->floats() && event->hasEndDate() ) {
     tmp.clear();
     QTime sDuration(0,0,0), t;
     int secs = event->dtStart().secsTo( event->dtEnd() );
     t = sDuration.addSecs( secs );
     if ( t.hour() > 0 ) {
-      tmp += i18np( "1 hour ", "%n hours ", t.hour() );
+      tmp += i18np( "1 hour ", "%1 hours ", t.hour() );
     }
     if ( t.minute() > 0 ) {
-      tmp += i18np( "1 minute ", "%n minutes ", t.minute() );
+      tmp += i18np( "1 minute ", "%1 minutes ", t.minute() );
     }
 
     html += invitationRow( i18n( "Duration:" ), tmp );
@@ -668,18 +663,18 @@ static QString invitationDetailsFreeBusy( FreeBusy *fb )
       int dur = per.duration().asSeconds();
       QString cont;
       if ( dur >= 3600 ) {
-        cont += i18np("1 hour ", "%n hours ", dur / 3600);
+        cont += i18np("1 hour ", "%1 hours ", dur / 3600);
         dur %= 3600;
       }
       if ( dur >= 60 ) {
-        cont += i18np("1 minute", "%n minutes ", dur / 60);
+        cont += i18np("1 minute", "%1 minutes ", dur / 60);
         dur %= 60;
       }
       if ( dur > 0 ) {
-        cont += i18np("1 second", "%n seconds", dur);
+        cont += i18np("1 second", "%1 seconds", dur);
       }
       html += invitationRow( QString(), i18nc("startDate for duration", "%1 for %2",
-            KGlobal::locale()->formatDateTime( per.start(), false ) ,
+            KGlobal::locale()->formatDateTime( per.start().dateTime(), false ) ,
            cont) );
     } else {
       QString cont;
@@ -690,8 +685,8 @@ static QString invitationDetailsFreeBusy( FreeBusy *fb )
               KGlobal::locale()->formatTime( per.end().time() ) );
       } else {
         cont = i18nc("fromDateTime - toDateTime", "%1 - %2",
-            KGlobal::locale()->formatDateTime( per.start(), false ) ,
-            KGlobal::locale()->formatDateTime( per.end(), false ) );
+            KGlobal::locale()->formatDateTime( per.start().dateTime(), false ) ,
+            KGlobal::locale()->formatDateTime( per.end().dateTime(), false ) );
       }
 
       html += invitationRow( QString(), cont );
@@ -899,7 +894,7 @@ static QString invitationHeaderFreeBusy( FreeBusy *fb, ScheduleMessage *msg )
   }
 }
 
-class IncidenceFormatter::ScheduleMessageVisitor : public IncidenceBase::Visitor
+class KCal::IncidenceFormatter::ScheduleMessageVisitor : public IncidenceBase::Visitor
 {
   public:
     ScheduleMessageVisitor() : mMessage(0) { mResult = ""; }
@@ -911,7 +906,7 @@ class IncidenceFormatter::ScheduleMessageVisitor : public IncidenceBase::Visitor
     ScheduleMessage *mMessage;
 };
 
-class IncidenceFormatter::InvitationHeaderVisitor :
+class KCal::IncidenceFormatter::InvitationHeaderVisitor :
       public IncidenceFormatter::ScheduleMessageVisitor
 {
   protected:
@@ -937,7 +932,7 @@ class IncidenceFormatter::InvitationHeaderVisitor :
     }
 };
 
-class IncidenceFormatter::InvitationBodyVisitor :
+class KCal::IncidenceFormatter::InvitationBodyVisitor :
       public IncidenceFormatter::ScheduleMessageVisitor
 {
   protected:
@@ -1088,470 +1083,11 @@ QString IncidenceFormatter::formatICalInvitation( QString invitation, Calendar *
 }
 
 
-
-
-/*******************************************************************
- *  Helper functions for the msTNEF -> VPart converter
- *******************************************************************/
-
-
-//-----------------------------------------------------------------------------
-
-static QString stringProp( KTNEFMessage* tnefMsg, const quint32& key,
-                           const QString& fallback = QString())
-{
-  return tnefMsg->findProp( key < 0x10000 ? key & 0xFFFF : key >> 16,
-                            fallback );
-}
-
-static QString sNamedProp( KTNEFMessage* tnefMsg, const QString& name,
-                           const QString& fallback = QString() )
-{
-  return tnefMsg->findNamedProp( name, fallback );
-}
-
-struct save_tz { char* old_tz; char* tz_env_str; };
-
-/* temporarily go to a different timezone */
-static struct save_tz set_tz( const char* _tc )
-{
-  const char *tc = _tc?_tc:"UTC";
-
-  struct save_tz rv;
-
-  rv.old_tz = 0;
-  rv.tz_env_str = 0;
-
-  //kDebug(5006) << "set_tz(), timezone before = " << timezone << endl;
-
-  char* tz_env = 0;
-  if( getenv( "TZ" ) ) {
-    tz_env = strdup( getenv( "TZ" ) );
-    rv.old_tz = tz_env;
-  }
-  char* tmp_env = (char*)malloc( strlen( tc ) + 4 );
-  strcpy( tmp_env, "TZ=" );
-  strcpy( tmp_env+3, tc );
-  putenv( tmp_env );
-
-  rv.tz_env_str = tmp_env;
-
-  /* tmp_env is not free'ed -- it is part of the environment */
-
-  tzset();
-  //kDebug(5006) << "set_tz(), timezone after = " << timezone << endl;
-
-  return rv;
-}
-
-/* restore previous timezone */
-static void unset_tz( struct save_tz old_tz )
-{
-  if( old_tz.old_tz ) {
-    char* tmp_env = (char*)malloc( strlen( old_tz.old_tz ) + 4 );
-    strcpy( tmp_env, "TZ=" );
-    strcpy( tmp_env+3, old_tz.old_tz );
-    putenv( tmp_env );
-    /* tmp_env is not free'ed -- it is part of the environment */
-    free( old_tz.old_tz );
-  } else {
-    /* clear TZ from env */
-    putenv( strdup("TZ") );
-  }
-  tzset();
-
-  /* is this OK? */
-  if( old_tz.tz_env_str ) free( old_tz.tz_env_str );
-}
-
-static QDateTime utc2Local( const QDateTime& utcdt )
-{
-  struct tm tmL;
-
-  save_tz tmp_tz = set_tz("UTC");
-  time_t utc = utcdt.toTime_t();
-  unset_tz( tmp_tz );
-
-  localtime_r( &utc, &tmL );
-  return QDateTime( QDate( tmL.tm_year+1900, tmL.tm_mon+1, tmL.tm_mday ),
-                    QTime( tmL.tm_hour, tmL.tm_min, tmL.tm_sec ) );
-}
-
-
-static QDateTime pureISOToLocalQDateTime( const QString& dtStr,
-                                          bool bDateOnly = false )
-{
-  QDate tmpDate;
-  QTime tmpTime;
-  int year, month, day, hour, minute, second;
-
-  if( bDateOnly ) {
-    year = dtStr.left( 4 ).toInt();
-    month = dtStr.mid( 4, 2 ).toInt();
-    day = dtStr.mid( 6, 2 ).toInt();
-    hour = 0;
-    minute = 0;
-    second = 0;
-  } else {
-    year = dtStr.left( 4 ).toInt();
-    month = dtStr.mid( 4, 2 ).toInt();
-    day = dtStr.mid( 6, 2 ).toInt();
-    hour = dtStr.mid( 9, 2 ).toInt();
-    minute = dtStr.mid( 11, 2 ).toInt();
-    second = dtStr.mid( 13, 2 ).toInt();
-  }
-  tmpDate.setYMD( year, month, day );
-  tmpTime.setHMS( hour, minute, second );
-
-  if( tmpDate.isValid() && tmpTime.isValid() ) {
-    QDateTime dT = QDateTime( tmpDate, tmpTime );
-
-    if( !bDateOnly ) {
-      // correct for GMT ( == Zulu time == UTC )
-      if (dtStr.at(dtStr.length()-1) == 'Z') {
-        //dT = dT.addSecs( 60 * KRFCDate::localUTCOffset() );
-        //localUTCOffset( dT ) );
-        dT = utc2Local( dT );
-      }
-    }
-    return dT;
-  } else
-    return QDateTime();
-}
-
-
-
-QString IncidenceFormatter::msTNEFToVPart( const QByteArray& tnef )
-{
-  bool bOk = false;
-
-  KTNEFParser parser;
-  QByteArray b( tnef );
-  QBuffer buf( &b );
-  CalendarLocal cal ( QLatin1String( "UTC" ) );
-  KABC::Addressee addressee;
-  ICalFormat calFormat;
-  Event* event = new Event();
-
-  if( parser.openDevice( &buf ) ) {
-    KTNEFMessage* tnefMsg = parser.message();
-    //QMap<int,KTNEFProperty*> props = parser.message()->properties();
-
-    // Everything depends from property PR_MESSAGE_CLASS
-    // (this is added by KTNEFParser):
-    QString msgClass = tnefMsg->findProp( 0x001A, QString(), true )
-      .toUpper();
-    if( !msgClass.isEmpty() ) {
-      // Match the old class names that might be used by Outlook for
-      // compatibility with Microsoft Mail for Windows for Workgroups 3.1.
-      bool bCompatClassAppointment = false;
-      bool bCompatMethodRequest = false;
-      bool bCompatMethodCancled = false;
-      bool bCompatMethodAccepted = false;
-      bool bCompatMethodAcceptedCond = false;
-      bool bCompatMethodDeclined = false;
-      if( msgClass.startsWith( "IPM.MICROSOFT SCHEDULE." ) ) {
-        bCompatClassAppointment = true;
-        if( msgClass.endsWith( ".MTGREQ" ) )
-          bCompatMethodRequest = true;
-        if( msgClass.endsWith( ".MTGCNCL" ) )
-          bCompatMethodCancled = true;
-        if( msgClass.endsWith( ".MTGRESPP" ) )
-          bCompatMethodAccepted = true;
-        if( msgClass.endsWith( ".MTGRESPA" ) )
-          bCompatMethodAcceptedCond = true;
-        if( msgClass.endsWith( ".MTGRESPN" ) )
-          bCompatMethodDeclined = true;
-      }
-      bool bCompatClassNote = ( msgClass == "IPM.MICROSOFT MAIL.NOTE" );
-
-      if( bCompatClassAppointment || "IPM.APPOINTMENT" == msgClass ) {
-        // Compose a vCal
-        bool bIsReply = false;
-        QString prodID = "-//Microsoft Corporation//Outlook ";
-        prodID += tnefMsg->findNamedProp( "0x8554", "9.0" );
-        prodID += "MIMEDIR/EN\n";
-        prodID += "VERSION:2.0\n";
-        calFormat.setApplication( "Outlook", prodID );
-
-        Scheduler::Method method;
-        if( bCompatMethodRequest )
-          method = Scheduler::Request;
-        else if( bCompatMethodCancled )
-          method = Scheduler::Cancel;
-        else if( bCompatMethodAccepted || bCompatMethodAcceptedCond ||
-                 bCompatMethodDeclined ) {
-          method = Scheduler::Reply;
-          bIsReply = true;
-        } else {
-          // pending(khz): verify whether "0x0c17" is the right tag ???
-          //
-          // at the moment we think there are REQUESTS and UPDATES
-          //
-          // but WHAT ABOUT REPLIES ???
-          //
-          //
-
-          if( tnefMsg->findProp(0x0c17) == "1" )
-            bIsReply = true;
-          method = Scheduler::Request;
-        }
-
-        /// ###  FIXME Need to get this attribute written
-        ScheduleMessage schedMsg(event, method, ScheduleMessage::Unknown );
-
-        QString sSenderSearchKeyEmail( tnefMsg->findProp( 0x0C1D ) );
-
-        if( !sSenderSearchKeyEmail.isEmpty() ) {
-          int colon = sSenderSearchKeyEmail.indexOf( ':' );
-          // May be e.g. "SMTP:KHZ@KDE.ORG"
-          if( sSenderSearchKeyEmail.indexOf( ':' ) == -1 )
-            sSenderSearchKeyEmail.remove( 0, colon+1 );
-        }
-
-        QString s( tnefMsg->findProp( 0x0e04 ) );
-        QStringList attendees = s.split( ';' );
-        if( attendees.count() ) {
-          for( QStringList::Iterator it = attendees.begin();
-               it != attendees.end(); ++it ) {
-            // Skip all entries that have no '@' since these are
-            // no mail addresses
-            if( (*it).indexOf('@') == -1 ) {
-              s = (*it).trimmed();
-
-              Attendee *attendee = new Attendee( s, s, true );
-              if( bIsReply ) {
-                if( bCompatMethodAccepted )
-                  attendee->setStatus( Attendee::Accepted );
-                if( bCompatMethodDeclined )
-                  attendee->setStatus( Attendee::Declined );
-                if( bCompatMethodAcceptedCond )
-                  attendee->setStatus(Attendee::Tentative);
-              } else {
-                attendee->setStatus( Attendee::NeedsAction );
-                attendee->setRole( Attendee::ReqParticipant );
-              }
-              event->addAttendee(attendee);
-            }
-          }
-        } else {
-          // Oops, no attendees?
-          // This must be old style, let us use the PR_SENDER_SEARCH_KEY.
-          s = sSenderSearchKeyEmail;
-          if( !s.isEmpty() ) {
-            Attendee *attendee = new Attendee( QString(), QString(),
-                                               true );
-            if( bIsReply ) {
-              if( bCompatMethodAccepted )
-                attendee->setStatus( Attendee::Accepted );
-              if( bCompatMethodAcceptedCond )
-                attendee->setStatus( Attendee::Declined );
-              if( bCompatMethodDeclined )
-                attendee->setStatus( Attendee::Tentative );
-            } else {
-              attendee->setStatus(Attendee::NeedsAction);
-              attendee->setRole(Attendee::ReqParticipant);
-            }
-            event->addAttendee(attendee);
-          }
-        }
-        s = tnefMsg->findProp( 0x0c1f ); // look for organizer property
-        if( s.isEmpty() && !bIsReply )
-          s = sSenderSearchKeyEmail;
-        // TODO: Use the common name?
-        if( !s.isEmpty() )
-          event->setOrganizer( s );
-
-        s = tnefMsg->findProp( 0x8516 ).replace( QChar( '-' ), QString() )
-          .replace( QChar( ':' ), QString() );
-        event->setDtStart( QDateTime::fromString( s ) ); // ## Format??
-
-        s = tnefMsg->findProp( 0x8517 ).replace( QChar( '-' ), QString() )
-          .replace( QChar( ':' ), QString() );
-        event->setDtEnd( QDateTime::fromString( s ) );
-
-        s = tnefMsg->findProp( 0x8208 );
-        event->setLocation( s );
-
-        // is it OK to set this to OPAQUE always ??
-        //vPart += "TRANSP:OPAQUE\n"; ###FIXME, portme!
-        //vPart += "SEQUENCE:0\n";
-
-        // is "0x0023" OK  -  or should we look for "0x0003" ??
-        s = tnefMsg->findProp( 0x0023 );
-        event->setUid( s );
-
-        // PENDING(khz): is this value in local timezone? Must it be
-        // adjusted? Most likely this is a bug in the server or in
-        // Outlook - we ignore it for now.
-        s = tnefMsg->findProp( 0x8202 ).replace( QChar( '-' ), QString() )
-          .replace( QChar( ':' ), QString() );
-        // ### kcal always uses currentDateTime()
-        // event->setDtStamp(QDateTime::fromString(s));
-
-        s = tnefMsg->findNamedProp( "Keywords" );
-        event->setCategories( s );
-
-        s = tnefMsg->findProp( 0x1000 );
-        event->setDescription( s );
-
-        s = tnefMsg->findProp( 0x0070 );
-        event->setSummary( s );
-
-        s = tnefMsg->findProp( 0x0026 );
-        event->setPriority( s.toInt() );
-
-        // is reminder flag set ?
-        if(!tnefMsg->findProp(0x8503).isEmpty()) {
-          Alarm *alarm = new Alarm(event);
-          QDateTime highNoonTime =
-            pureISOToLocalQDateTime( tnefMsg->findProp( 0x8502 )
-                                     .replace( QChar( '-' ), "" )
-                                     .replace( QChar( ':' ), "" ) );
-          QDateTime wakeMeUpTime =
-            pureISOToLocalQDateTime( tnefMsg->findProp( 0x8560, "" )
-                                     .replace( QChar( '-' ), "" )
-                                     .replace( QChar( ':' ), "" ) );
-          alarm->setTime(wakeMeUpTime);
-
-          if( highNoonTime.isValid() && wakeMeUpTime.isValid() )
-            alarm->setStartOffset( Duration( highNoonTime, wakeMeUpTime ) );
-          else
-            // default: wake them up 15 minutes before the appointment
-            alarm->setStartOffset( Duration( 15*60 ) );
-          alarm->setDisplayAlarm( i18n( "Reminder" ) );
-
-          // Sorry: the different action types are not known (yet)
-          //        so we always set 'DISPLAY' (no sounds, no images...)
-          event->addAlarm( alarm );
-        }
-        cal.addEvent( event );
-        bOk = true;
-        // we finished composing a vCal
-      } else if( bCompatClassNote || "IPM.CONTACT" == msgClass ) {
-        addressee.setUid( stringProp( tnefMsg, attMSGID ) );
-        addressee.setFormattedName( stringProp( tnefMsg, MAPI_TAG_PR_DISPLAY_NAME ) );
-        addressee.insertEmail( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_EMAIL1EMAILADDRESS ), true );
-        addressee.insertEmail( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_EMAIL2EMAILADDRESS ), false );
-        addressee.insertEmail( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_EMAIL3EMAILADDRESS ), false );
-        addressee.insertCustom( "KADDRESSBOOK", "X-IMAddress", sNamedProp( tnefMsg, MAPI_TAG_CONTACT_IMADDRESS ) );
-        addressee.insertCustom( "KADDRESSBOOK", "X-SpousesName", stringProp( tnefMsg, MAPI_TAG_PR_SPOUSE_NAME ) );
-        addressee.insertCustom( "KADDRESSBOOK", "X-ManagersName", stringProp( tnefMsg, MAPI_TAG_PR_MANAGER_NAME ) );
-        addressee.insertCustom( "KADDRESSBOOK", "X-AssistantsName", stringProp( tnefMsg, MAPI_TAG_PR_ASSISTANT ) );
-        addressee.insertCustom( "KADDRESSBOOK", "X-Department", stringProp( tnefMsg, MAPI_TAG_PR_DEPARTMENT_NAME ) );
-        addressee.insertCustom( "KADDRESSBOOK", "X-Office", stringProp( tnefMsg, MAPI_TAG_PR_OFFICE_LOCATION ) );
-        addressee.insertCustom( "KADDRESSBOOK", "X-Profession", stringProp( tnefMsg, MAPI_TAG_PR_PROFESSION ) );
-
-        QString s = tnefMsg->findProp( MAPI_TAG_PR_WEDDING_ANNIVERSARY )
-          .replace( QChar( '-' ), QString() )
-          .replace( QChar( ':' ), QString() );
-        if( !s.isEmpty() )
-          addressee.insertCustom( "KADDRESSBOOK", "X-Anniversary", s );
-
-        addressee.setUrl( KUrl( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_WEBPAGE )  ) );
-
-        // collect parts of Name entry
-        addressee.setFamilyName( stringProp( tnefMsg, MAPI_TAG_PR_SURNAME ) );
-        addressee.setGivenName( stringProp( tnefMsg, MAPI_TAG_PR_GIVEN_NAME ) );
-        addressee.setAdditionalName( stringProp( tnefMsg, MAPI_TAG_PR_MIDDLE_NAME ) );
-        addressee.setPrefix( stringProp( tnefMsg, MAPI_TAG_PR_DISPLAY_NAME_PREFIX ) );
-        addressee.setSuffix( stringProp( tnefMsg, MAPI_TAG_PR_GENERATION ) );
-
-        addressee.setNickName( stringProp( tnefMsg, MAPI_TAG_PR_NICKNAME ) );
-        addressee.setRole( stringProp( tnefMsg, MAPI_TAG_PR_TITLE ) );
-        addressee.setOrganization( stringProp( tnefMsg, MAPI_TAG_PR_COMPANY_NAME ) );
-        /*
-        the MAPI property ID of this (multiline) )field is unknown:
-        vPart += stringProp(tnefMsg, "\n","NOTE", ... , "" );
-        */
-
-        KABC::Address adr;
-        adr.setPostOfficeBox( stringProp( tnefMsg, MAPI_TAG_PR_HOME_ADDRESS_PO_BOX ) );
-        adr.setStreet( stringProp( tnefMsg, MAPI_TAG_PR_HOME_ADDRESS_STREET ) );
-        adr.setLocality( stringProp( tnefMsg, MAPI_TAG_PR_HOME_ADDRESS_CITY ) );
-        adr.setRegion( stringProp( tnefMsg, MAPI_TAG_PR_HOME_ADDRESS_STATE_OR_PROVINCE ) );
-        adr.setPostalCode( stringProp( tnefMsg, MAPI_TAG_PR_HOME_ADDRESS_POSTAL_CODE ) );
-        adr.setCountry( stringProp( tnefMsg, MAPI_TAG_PR_HOME_ADDRESS_COUNTRY ) );
-        adr.setType(KABC::Address::Home);
-        addressee.insertAddress(adr);
-
-        adr.setPostOfficeBox( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_BUSINESSADDRESSPOBOX ) );
-        adr.setStreet( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_BUSINESSADDRESSSTREET ) );
-        adr.setLocality( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_BUSINESSADDRESSCITY ) );
-        adr.setRegion( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_BUSINESSADDRESSSTATE ) );
-        adr.setPostalCode( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_BUSINESSADDRESSPOSTALCODE ) );
-        adr.setCountry( sNamedProp( tnefMsg, MAPI_TAG_CONTACT_BUSINESSADDRESSCOUNTRY ) );
-        adr.setType( KABC::Address::Work );
-        addressee.insertAddress( adr );
-
-        adr.setPostOfficeBox( stringProp( tnefMsg, MAPI_TAG_PR_OTHER_ADDRESS_PO_BOX ) );
-        adr.setStreet( stringProp(tnefMsg, MAPI_TAG_PR_OTHER_ADDRESS_STREET ) );
-        adr.setLocality( stringProp(tnefMsg, MAPI_TAG_PR_OTHER_ADDRESS_CITY ) );
-        adr.setRegion( stringProp(tnefMsg, MAPI_TAG_PR_OTHER_ADDRESS_STATE_OR_PROVINCE ) );
-        adr.setPostalCode( stringProp(tnefMsg, MAPI_TAG_PR_OTHER_ADDRESS_POSTAL_CODE ) );
-        adr.setCountry( stringProp(tnefMsg, MAPI_TAG_PR_OTHER_ADDRESS_COUNTRY ) );
-        adr.setType( KABC::Address::Dom );
-        addressee.insertAddress(adr);
-
-        // problem: the 'other' address was stored by KOrganizer in
-        //          a line looking like the following one:
-        // vPart += "\nADR;TYPE=dom;TYPE=intl;TYPE=parcel;TYPE=postal;TYPE=work;TYPE=home:other_pobox;;other_str1\nother_str2;other_loc;other_region;other_pocode;other_country
-
-        QString nr;
-        nr = stringProp( tnefMsg, MAPI_TAG_PR_HOME_TELEPHONE_NUMBER );
-        addressee.insertPhoneNumber( KABC::PhoneNumber( nr, KABC::PhoneNumber::Home ) );
-        nr = stringProp( tnefMsg, MAPI_TAG_PR_BUSINESS_TELEPHONE_NUMBER );
-        addressee.insertPhoneNumber( KABC::PhoneNumber( nr, KABC::PhoneNumber::Work ) );
-        nr = stringProp( tnefMsg, MAPI_TAG_PR_MOBILE_TELEPHONE_NUMBER );
-        addressee.insertPhoneNumber( KABC::PhoneNumber( nr, KABC::PhoneNumber::Cell ) );
-        nr = stringProp( tnefMsg, MAPI_TAG_PR_HOME_FAX_NUMBER );
-        addressee.insertPhoneNumber( KABC::PhoneNumber( nr, KABC::PhoneNumber::Fax | KABC::PhoneNumber::Home ) );
-        nr = stringProp( tnefMsg, MAPI_TAG_PR_BUSINESS_FAX_NUMBER );
-        addressee.insertPhoneNumber( KABC::PhoneNumber( nr, KABC::PhoneNumber::Fax | KABC::PhoneNumber::Work ) );
-
-        s = tnefMsg->findProp( MAPI_TAG_PR_BIRTHDAY )
-          .replace( QChar( '-' ), QString() )
-          .replace( QChar( ':' ), QString() );
-        if( !s.isEmpty() )
-          addressee.setBirthday( QDateTime::fromString( s ) );
-
-        bOk = ( !addressee.isEmpty() );
-      } else if( "IPM.NOTE" == msgClass ) {
-
-      } // else if ... and so on ...
-    }
-  }
-
-  // Compose return string
-  QString iCal = calFormat.toString( &cal );
-  if( !iCal.isEmpty() )
-    // This was an iCal
-    return iCal;
-
-  // Not an iCal - try a vCard
-  KABC::VCardConverter converter;
-  return QString::fromUtf8( converter.createVCard( addressee ) );
-}
-
-
-QString IncidenceFormatter::formatTNEFInvitation( const QByteArray& tnef,
-        Calendar *mCalendar, InvitationFormatterHelper *helper )
-{
-  QString vPart = IncidenceFormatter::msTNEFToVPart( tnef );
-  QString iCal = IncidenceFormatter::formatICalInvitation( vPart, mCalendar, helper );
-  if( !iCal.isEmpty() )
-    return iCal;
-  return vPart;
-}
-
-
-
-
 /*******************************************************************
  *  Helper functions for the Incidence tooltips
  *******************************************************************/
 
-class IncidenceFormatter::ToolTipVisitor : public IncidenceBase::Visitor
+class KCal::IncidenceFormatter::ToolTipVisitor : public IncidenceBase::Visitor
 {
   public:
     ToolTipVisitor() : mRichText( true ), mResult( "" ) {}
@@ -1588,13 +1124,13 @@ QString IncidenceFormatter::ToolTipVisitor::dateRangeText( Event*event )
   QString tmp;
   if ( event->isMultiDay() ) {
 
-    if (event->doesFloat())
+    if (event->floats())
       tmp = event->dtStartDateStr().replace(" ", "&nbsp;");
     else
       tmp = event->dtStartStr().replace(" ", "&nbsp;");
     ret += "<br>" + i18nc("Event start", "<i>From:</i>&nbsp;%1", tmp);
 
-    if (event->doesFloat())
+    if (event->floats())
       tmp = event->dtEndDateStr().replace(" ", "&nbsp;");
     else
       tmp = event->dtEndStr().replace(" ", "&nbsp;");
@@ -1604,7 +1140,7 @@ QString IncidenceFormatter::ToolTipVisitor::dateRangeText( Event*event )
 
     ret += "<br>"+i18n("<i>Date:</i>&nbsp;%1",
          event->dtStartDateStr().replace(" ", "&nbsp;") );
-    if ( !event->doesFloat() ) {
+    if ( !event->floats() ) {
       if ( event->dtStartTimeStr() == event->dtEndTimeStr() ) { // to prevent 'Time: 17:00 - 17:00'
         tmp = "<br>" + i18nc("time for event, &nbsp; to prevent ugly line breaks",
         "<i>Time:</i>&nbsp;%1",
@@ -1625,7 +1161,7 @@ QString IncidenceFormatter::ToolTipVisitor::dateRangeText( Event*event )
 QString IncidenceFormatter::ToolTipVisitor::dateRangeText( Todo*todo )
 {
   QString ret;
-  bool floats( todo->doesFloat() );
+  bool floats( todo->floats() );
   if (todo->hasStartDate())
     // No need to add <i> here. This is separated issue and each line
     // is very visible on its own. On the other hand... Yes, I like it
@@ -1658,8 +1194,8 @@ QString IncidenceFormatter::ToolTipVisitor::dateRangeText( Journal*journal )
 
 QString IncidenceFormatter::ToolTipVisitor::dateRangeText( FreeBusy *fb )
 {
-  QString ret = "<br>" + i18n("<i>Period start:</i>&nbsp;%1", KGlobal::locale()->formatDateTime( fb->dtStart() ) );
-  ret += "<br>" + i18n("<i>Period start:</i>&nbsp;%1", KGlobal::locale()->formatDateTime( fb->dtEnd() ) );
+  QString ret = "<br>" + i18n("<i>Period start:</i>&nbsp;%1", KGlobal::locale()->formatDateTime( fb->dtStart().dateTime() ) );
+  ret += "<br>" + i18n("<i>Period start:</i>&nbsp;%1", KGlobal::locale()->formatDateTime( fb->dtEnd().dateTime() ) );
   return ret;
 }
 
@@ -1694,6 +1230,9 @@ bool IncidenceFormatter::ToolTipVisitor::visit( FreeBusy *fb )
 
 QString IncidenceFormatter::ToolTipVisitor::generateToolTip( Incidence* incidence, QString dtRangeText )
 {
+  if ( !incidence )
+    return QString();
+
   QString tmp = "<qt><b>"+ incidence->summary().replace("\n", "<br>")+"</b>";
 
   tmp += dtRangeText;
@@ -1730,7 +1269,7 @@ QString IncidenceFormatter::toolTipString( IncidenceBase *incidence, bool richTe
  *  Helper functions for the Incidence tooltips
  *******************************************************************/
 
-class IncidenceFormatter::MailBodyVisitor : public IncidenceBase::Visitor
+class KCal::IncidenceFormatter::MailBodyVisitor : public IncidenceBase::Visitor
 {
   public:
     MailBodyVisitor() : mResult( "" ) {}
@@ -1769,20 +1308,28 @@ static QString mailBodyIncidence( Incidence *incidence )
 
 bool IncidenceFormatter::MailBodyVisitor::visit( Event *event )
 {
-  QString recurrence[]= {i18nc("no recurrence", "None"),
-    i18n("Minutely"), i18n("Hourly"), i18n("Daily"),
-    i18n("Weekly"), i18n("Monthly Same Day"), i18n("Monthly Same Position"),
-    i18n("Yearly"), i18n("Yearly"), i18n("Yearly")};
+  QString recurrence[]= {
+    i18nc("no recurrence", "None"),
+    i18n("Minutely"),
+    i18n("Hourly"),
+    i18n("Daily"),
+    i18n("Weekly"),
+    i18n("Monthly Same Day"),
+    i18n("Monthly Same Position"),
+    i18n("Yearly"),
+    i18n("Yearly"),
+    i18n("Yearly")
+  };
 
   mResult = mailBodyIncidence( event );
   mResult += i18n("Start Date: %1\n", event->dtStartDateStr() );
-  if ( !event->doesFloat() ) {
+  if ( !event->floats() ) {
     mResult += i18n("Start Time: %1\n", event->dtStartTimeStr() );
   }
   if ( event->dtStart() != event->dtEnd() ) {
     mResult += i18n("End Date: %1\n", event->dtEndDateStr() );
   }
-  if ( !event->doesFloat() ) {
+  if ( !event->floats() ) {
     mResult += i18n("End Time: %1\n", event->dtEndTimeStr() );
   }
   if ( event->doesRecur() ) {
@@ -1794,16 +1341,16 @@ bool IncidenceFormatter::MailBodyVisitor::visit( Event *event )
                event->recurrence()->frequency() );
 
     if ( recur->duration() > 0 ) {
-      mResult += i18np ("Repeats once", "Repeats %n times", recur->duration());
+      mResult += i18np ("Repeats once", "Repeats %1 times", recur->duration());
       mResult += '\n';
     } else {
       if ( recur->duration() != -1 ) {
 // TODO_Recurrence: What to do with floating
         QString endstr;
-        if ( event->doesFloat() ) {
+        if ( event->floats() ) {
           endstr = KGlobal::locale()->formatDate( recur->endDate() );
         } else {
-          endstr = KGlobal::locale()->formatDateTime( recur->endDateTime() );
+          endstr = KGlobal::locale()->formatDateTime( recur->endDateTime().dateTime() );
         }
         mResult += i18n("Repeat until: %1\n", endstr );
       } else {
@@ -1824,13 +1371,13 @@ bool IncidenceFormatter::MailBodyVisitor::visit( Todo *todo )
 
   if ( todo->hasStartDate() ) {
     mResult += i18n("Start Date: %1\n", todo->dtStartDateStr() );
-    if ( !todo->doesFloat() ) {
+    if ( !todo->floats() ) {
       mResult += i18n("Start Time: %1\n", todo->dtStartTimeStr() );
     }
   }
   if ( todo->hasDueDate() ) {
     mResult += i18n("Due Date: %1\n", todo->dtDueDateStr() );
-    if ( !todo->doesFloat() ) {
+    if ( !todo->floats() ) {
       mResult += i18n("Due Time: %1\n", todo->dtDueTimeStr() );
     }
   }
@@ -1845,7 +1392,7 @@ bool IncidenceFormatter::MailBodyVisitor::visit( Journal *journal )
 {
   mResult = mailBodyIncidence( journal );
   mResult += i18n("Date: %1\n", journal->dtStartDateStr() );
-  if ( !journal->doesFloat() ) {
+  if ( !journal->floats() ) {
     mResult += i18n("Time: %1\n", journal->dtStartTimeStr() );
   }
   if ( !journal->description().isEmpty() )
