@@ -2,7 +2,7 @@
   FILE: icalcomponent.c
   CREATOR: eric 28 April 1999
   
-  $Id: icalcomponent.c 564286 2006-07-19 19:25:58Z chehrlic $
+  $Id: icalcomponent.c 627441 2007-01-26 15:59:10Z kainhofe $
 
  (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
      http://www.softwarestudio.org
@@ -23,10 +23,6 @@
 ======================================================================*/
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "icalcomponent.h"
 #include "pvl.h" /* "Pointer-to-void list" */
 #include "icalerror.h"
@@ -281,11 +277,11 @@ icalcomponent_as_ical_string (icalcomponent* impl)
    char* buf_ptr = 0;
     pvl_elem itr;
 
+/* RFC 2445 explicitly says that the newline is *ALWAYS* a \r\n (CRLF)!!!! */
+char newline[] = "\r\n";
 /* WIN32 automatically adds the \r, Anybody else need it?
 #ifdef ICAL_UNIX_NEWLINE    
-*/
     char newline[] = "\n";
-/*
 #else
     char newline[] = "\r\n";
 #endif
