@@ -4,7 +4,7 @@
   FILE: icalproperty.c
   CREATOR: eric 28 April 1999
   
-  $Id: icalproperty.c 537971 2006-05-06 11:24:08Z kainhofe $
+  $Id: icalproperty.c 627441 2007-01-26 15:59:10Z kainhofe $
 
 
  (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>
@@ -342,7 +342,7 @@ fold_property_line (char *text)
 	/* If this isn't the first line, we need to output a newline and space
 	   first. */
 	if (!first_line) {
-	    icalmemory_append_string (&buf, &buf_ptr, &buf_size, "\n ");
+	    icalmemory_append_string (&buf, &buf_ptr, &buf_size, "\r\n ");
 	}
 	first_line = 0;
 
@@ -434,7 +434,7 @@ icalproperty_as_ical_string (icalproperty* prop)
     char *out_buf;
     const char* kind_string = 0;
 
-    char newline[] = "\n";
+    char newline[] = "\r\n";
 
     
     icalerror_check_arg_rz( (prop!=0),"prop");
