@@ -4,7 +4,7 @@
  CREATOR: eric 21 Aug 2000
 
 
- $Id: icalclassify.h 537971 2006-05-06 11:24:08Z kainhofe $
+ $Id: icalclassify.h 618274 2006-12-31 21:00:36Z kainhofe $
  $Locker:  $
 
  (C) COPYRIGHT 2000, Eric Busboom <eric@softwarestudio.org>, http://www.softwarestudio.org
@@ -27,16 +27,10 @@
 #define ICALCLASSIFY_H
 
 #include "ical.h"
-#include "icalset.h"
 
 icalproperty_xlicclass icalclassify(icalcomponent* c,icalcomponent* match, 
 			      const char* user);
 
-icalcomponent* icalclassify_find_overlaps(icalset* set, icalcomponent* comp);
-
-char* icalclassify_class_to_string(icalproperty_xlicclass c);
-
-
 #endif /* ICALCLASSIFY_H*/
 
 
