@@ -22,21 +22,20 @@
 #include <stdlib.h>
 
 #include <qtest_kde.h>
-#include <QtTest/QtTest>
 #include <QDateTime>
 #include <QTextStream>
 
-#include <kaboutdata.h>
-#include <kcmdlineargs.h>
-#include <kapplication.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
+
+#include "testicaltimezones.h"
+#include "testicaltimezones.moc"
+
+QTEST_KDEMAIN( ICalTimeZonesTest, NoGUI )
 
 extern "C" {
   #include "kcal/libical/src/libical/ical.h"
 }
 #include "kcal/icaltimezones.h"
-#include "testicaltimezones.moc"
-
 using namespace KCal;
 
 static icalcomponent *loadCALENDAR(const char *vcal);
@@ -73,22 +72,22 @@ static const char *VTZ_Western =
   "TZOFFSETFROM:-0400\n"
   "TZOFFSETTO:-0500\n"
   "TZNAME:WST\n"
-  "END:STANDARD\n"
+  "END:STANDARD\n\n"
   "BEGIN:DAYLIGHT\n"
   "DTSTART:19870405T020000\n"
-  "RRULE:FREQ=YEARLY;BYDAY=1SU;BYMONTH=4;UNTIL=19970406T070000Z\n"
+  "RRULE:FREQ=YEARLY;UNTIL=19970406T070000Z;BYDAY=1SU;BYMONTH=4\n"
   "TZOFFSETFROM:-0500\n"
   "TZOFFSETTO:-0400\n"
   "TZNAME:WDT1\n"
-  "END:DAYLIGHT\n"
+  "END:DAYLIGHT\n\n"
   "BEGIN:DAYLIGHT\n"
   "DTSTART:19990425T020000\n"
-  "RDATE:20000430T020000\n"
+  "RDATE;VALUE=DATE-TIME:20000430T020000\n"
   "TZOFFSETFROM:-0500\n"
   "TZOFFSETTO:-0400\n"
   "TZNAME:WDT2\n"
-  "END:DAYLIGHT\n"
-  "END:VTIMEZONE\n";
+  "END:DAYLIGHT\n\n"
+  "END:VTIMEZONE\n\n";
 
 // Standard time only
 static const char *VTZ_other =
@@ -98,12 +97,12 @@ static const char *VTZ_other =
   "X-LIC-LOCATION:Wyland/Tryburgh\n"
   "BEGIN:STANDARD\n"
   "DTSTART:19500101T000000\n"
-  "RDATE:19500101T000000\n"
-  "TZOFFSETFROM:0000\n"
-  "TZOFFSETTO:0300\n"
+  "RDATE;VALUE=DATE-TIME:19500101T000000\n"
+  "TZOFFSETFROM:+0000\n"
+  "TZOFFSETTO:+0300\n"
   "TZNAME:OST\n"
-  "END:STANDARD\n"
-  "END:VTIMEZONE\n";
+  "END:STANDARD\n\n"
+  "END:VTIMEZONE\n\n";
 
 // CALENDAR component header and footer
 static const char *calendarHeader =
@@ -113,19 +112,6 @@ static const char *calendarHeader =
 static const char *calendarFooter =
 "END:CALENDAR\n";
 
-
-int main(int argc, char *argv[])
-{
-    setenv("LC_ALL", "C", 1);
-    KAboutData aboutData("qttest", "qttest", "version");
-    KCmdLineArgs::init(argc, argv, &aboutData);
-    // KApplication::disableAutoDcopRegistration();
-    KApplication app(false);
-    ICalTimeZonesTest tc;
-    return QTest::qExec(&tc, argc, argv);
-}
-
-
 ///////////////////////////
 // ICalTimeZoneSource tests
 ///////////////////////////
@@ -137,24 +123,23 @@ void ICalTimeZonesTest::parse()
     text += VTZ_Western;
     text += VTZ_other;
     text += calendarFooter;
-    KTempFile tmpFile;
-    tmpFile.setAutoDelete(true);
-    QString path = tmpFile.name();
-    *tmpFile.textStream() << text.data();
-    tmpFile.close();
+    KTemporaryFile tmpFile;
+    tmpFile.open();
+    QString path = tmpFile.fileName();
+    QTextStream ts (&tmpFile);
+    ts << text.data();
+    ts.flush();
 
     // Parse the file, the CALENDAR text string and the individual VTIMEZONE strings,
     // and check that ICalTimeZone instances with the same names are created in each case.
     ICalTimeZoneSource src;
-    KTimeZones timezones1;
-#warning Temporary build hack
-    //    QVERIFY(src.parse(path, timezones1));
+    ICalTimeZones timezones1;
+    QVERIFY(src.parse(path, timezones1));
 
     icalcomponent *calendar = loadCALENDAR(text);
     QVERIFY(calendar);
-    KTimeZones timezones2;
-#warning Temporary build hack
-    // QVERIFY(src.parse(calendar, timezones2));
+    ICalTimeZones timezones2;
+    QVERIFY(src.parse(calendar, timezones2));
 
     icaltimezone *icaltz = icaltimezone_new();
     for (icalcomponent *ctz = icalcomponent_get_first_component(calendar, ICAL_VTIMEZONE_COMPONENT);
@@ -197,12 +182,14 @@ void ICalTimeZonesTest::general()
     QCOMPARE(tz->url(), QByteArray("http://tz.reference.net/dummies/western"));
     QCOMPARE(tz->city(), QString::fromLatin1("Zedland/Tryburgh"));
     QCOMPARE(tz->lastModified(), QDateTime(QDate(1987,1,1), QTime(0,0,0), Qt::UTC));
+    QCOMPARE(tz->vtimezone(), QByteArray(VTZ_Western));
 
     ICalTimeZone tz1(*tz);
     QCOMPARE(tz1.name(), tz->name());
     QCOMPARE(tz1.url(), tz->url());
     QCOMPARE(tz1.city(), tz->city());
     QCOMPARE(tz1.lastModified(), tz->lastModified());
+    QCOMPARE(tz1.vtimezone(), tz->vtimezone());
 
     QCOMPARE(itz->name(), tz->name());
     QCOMPARE(itz->url(), tz->url());
@@ -222,12 +209,14 @@ void ICalTimeZonesTest::general()
     QCOMPARE(tz2->url(), QByteArray("http://tz.reference.net/dummies/other"));
     QCOMPARE(tz2->city(), QString::fromLatin1("Wyland/Tryburgh"));
     QVERIFY(tz2->lastModified().isNull());
+    QCOMPARE(tz2->vtimezone(), QByteArray(VTZ_other));
 
     tz1 = *tz2;
     QCOMPARE(tz1.name(), tz2->name());
     QCOMPARE(tz1.url(), tz2->url());
     QCOMPARE(tz1.city(), tz2->city());
     QCOMPARE(tz1.lastModified(), tz2->lastModified());
+    QCOMPARE(tz1.vtimezone(), tz2->vtimezone());
 
     QCOMPARE(tz1.name(), itz2->name());
     QCOMPARE(tz1.url(), itz2->url());
@@ -250,6 +239,8 @@ void ICalTimeZonesTest::offsetAtUtc()
     QVERIFY(tz);
     icalcomponent_free(vtimezone);
 
+    QCOMPARE(tz->data(true)->previousUtcOffset(), -4*3600);
+    QCOMPARE(tz->transitions()[0].time(), start);
     QCOMPARE(tz->offsetAtUtc(start.addSecs(-1)), -4*3600);
     QCOMPARE(tz->offsetAtUtc(start), -5*3600);
     QCOMPARE(tz->offsetAtUtc(daylight87.addSecs(-1)), -5*3600);
