@@ -1,21 +1,21 @@
 /*
-    This file is part of libkldap.
-    Copyright (c) 2004-2006 Szombathelyi György <gyurco@freemail.hu>
+  This file is part of libkldap.
+  Copyright (c) 2004-2006 Szombathelyi György <gyurco@freemail.hu>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Library General Public
+  License as published by the Free Software Foundation; either
+  version 2 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Library General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+  You should have received a copy of the GNU Library General Public License
+  along with this library; see the file COPYING.LIB.  If not, write to
+  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA 02110-1301, USA.
 */
 
 #include <QCheckBox>
@@ -35,20 +35,22 @@
 #include <klineedit.h>
 #include <kmessagebox.h>
 
+#include "ldapsearch.h"
 #include "ldapconfigwidget.h"
 
 using namespace KLDAP;
 
-LdapConfigWidget::LdapConfigWidget( QWidget* parent,
-                                    Qt::WFlags fl ) : QWidget( parent, fl )
+LdapConfigWidget::LdapConfigWidget( QWidget *parent, Qt::WFlags fl )
+  : QWidget( parent, fl ), d( 0 )
 {
   mProg = 0;
   mFeatures = W_ALL;
   mainLayout = new QGridLayout( this );
 }
 
-LdapConfigWidget::LdapConfigWidget( LdapConfigWidget::WinFlags flags, QWidget* parent,
-                                    Qt::WFlags fl ) : QWidget( parent, fl )
+LdapConfigWidget::LdapConfigWidget( LdapConfigWidget::WinFlags flags,
+                                    QWidget *parent, Qt::WFlags fl )
+  : QWidget( parent, fl ), d( 0 )
 {
   mFeatures = flags;
   mProg = 0;
@@ -148,7 +150,9 @@ void LdapConfigWidget::initWidget()
     mainLayout->addWidget( label, row, col );
     mainLayout->addWidget( mVersion, row, col+1 );
   }
-  if ( mFeatures & ( W_PORT | W_VER ) ) row++;
+  if ( mFeatures & ( W_PORT | W_VER ) ) {
+    row++;
+  }
 
   col = 0;
   if ( mFeatures & W_SIZELIMIT ) {
@@ -176,7 +180,9 @@ void LdapConfigWidget::initWidget()
     mainLayout->addWidget( label, row, col );
     mainLayout->addWidget( mTimeLimit, row, col+1 );
   }
-  if ( mFeatures & ( W_SIZELIMIT | W_TIMELIMIT ) ) row++;
+  if ( mFeatures & ( W_SIZELIMIT | W_TIMELIMIT ) ) {
+    row++;
+  }
 
   if ( mFeatures & W_PAGESIZE ) {
     label = new QLabel( i18n( "Page size:" ), this );
@@ -192,7 +198,7 @@ void LdapConfigWidget::initWidget()
 
   if ( mFeatures & W_DN ) {
     label = new QLabel( i18nc( "Distinguished Name", "DN:" ), this );
-    mDn = new KLineEdit( this);
+    mDn = new KLineEdit( this );
     mDn->setObjectName("kcfg_ldapdn" );
 
     mainLayout->addWidget( label, row, 0 );
@@ -208,7 +214,7 @@ void LdapConfigWidget::initWidget()
 
   if ( mFeatures & W_FILTER ) {
     label = new QLabel( i18n( "Filter:" ), this );
-    mFilter = new KLineEdit( this);
+    mFilter = new KLineEdit( this );
     mFilter->setObjectName("kcfg_ldapfilter" );
 
     mainLayout->addWidget( label, row, 0 );
@@ -262,8 +268,8 @@ void LdapConfigWidget::initWidget()
     vbox->addLayout( hbox );
     label = new QLabel( i18n( "SASL mechanism:" ), authbox );
     hbox->addWidget( label );
-    mMech = new KComboBox( false, authbox);
-    mMech->setObjectName("kcfg_ldapsaslmech");
+    mMech = new KComboBox( false, authbox );
+    mMech->setObjectName( "kcfg_ldapsaslmech" );
     mMech->setEditable( true );
     mMech->addItem( "DIGEST-MD5" );
     mMech->addItem( "GSSAPI" );
@@ -279,41 +285,28 @@ void LdapConfigWidget::initWidget()
 
     mainLayout->addWidget( authbox, row, 0, 2, 4 );
 
-    connect( mAnonymous, SIGNAL( toggled(bool) ), SLOT( setAnonymous(bool) ) );
-    connect( mSimple, SIGNAL( toggled(bool) ), SLOT( setSimple(bool) ) );
-    connect( mSASL, SIGNAL( toggled(bool) ), SLOT( setSASL(bool) ) );
+    connect( mAnonymous, SIGNAL( toggled( bool ) ), SLOT( setAnonymous( bool ) ) );
+    connect( mSimple, SIGNAL( toggled( bool ) ), SLOT( setSimple( bool ) ) );
+    connect( mSASL, SIGNAL( toggled( bool ) ), SLOT( setSASL( bool ) ) );
 
     mAnonymous->setChecked( true );
   }
-
 }
 
-void LdapConfigWidget::loadData( KIO::Job*, const QByteArray& d )
+void LdapConfigWidget::loadData( LdapSearch*, const LdapObject &object )
 {
-  Ldif::ParseValue ret;
-
-  if ( d.size() ) {
-    mLdif.setLdif( d );
-  } else {
-    mLdif.endLdif();
-  }
-  do {
-    ret = mLdif.nextItem();
-    if ( ret == Ldif::Item && mLdif.attr().toLower() == mAttr ) {
-      mProg->setValue( mProg->value() + 1 );
-      mQResult.push_back( QString::fromUtf8( mLdif.value(), mLdif.value().size() ) );
+  kDebug() << "loadData() object: " << object.toString() << endl;
+  mProg->setValue( mProg->value() + 1 );
+  for ( LdapAttrMap::ConstIterator it = object.attributes().constBegin(); it != object.attributes().constEnd(); ++it ) {
+    for ( LdapAttrValue::ConstIterator it2 = (*it).constBegin(); it2 != (*it).constEnd(); ++it2 ) {
+      mQResult.push_back( QString::fromUtf8( *it2 ) );
     }
-  } while ( ret != Ldif::MoreData );
+  }
 }
 
-void LdapConfigWidget::loadResult( KJob* job)
+void LdapConfigWidget::loadResult( LdapSearch *search )
 {
-  int error = job->error();
-  if ( error && error != KIO::ERR_USER_CANCELED )
-    mErrorMsg = job->errorString();
-  else
-    mErrorMsg = "";
-
+  Q_UNUSED( search );
   mCancelled = false;
   mProg->close();
 }
@@ -326,25 +319,35 @@ void LdapConfigWidget::sendQuery()
   mCancelled = true;
 
   _url.setProtocol( ( mSecSSL && mSecSSL->isChecked() ) ? "ldaps" : "ldap" );
-  if ( mHost ) _url.setHost( mHost->text() );
-  if ( mPort ) _url.setPort( mPort->value() );
+  if ( mHost ) {
+    _url.setHost( mHost->text() );
+  }
+  if ( mPort ) {
+    _url.setPort( mPort->value() );
+  }
   _url.setDn( "" );
   _url.setAttributes( QStringList( mAttr ) );
   _url.setScope( LdapUrl::Base );
-  if ( mVersion ) _url.setExtension( "x-ver", QString::number( mVersion->value() ) );
-  if ( mSecTLS && mSecTLS->isChecked() ) _url.setExtension( "x-tls", "" );
+  if ( mVersion ) {
+    _url.setExtension( "x-ver", QString::number( mVersion->value() ) );
+  }
+  if ( mSecTLS && mSecTLS->isChecked() ) {
+    _url.setExtension( "x-tls", "" );
+  }
 
   kDebug(5700) << "sendQuery url: " << _url.prettyUrl() << endl;
-  mLdif.startParsing();
-  KIO::Job *job = KIO::get( _url, true, false );
-  job->addMetaData("no-auth-prompt","true");
-  connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
-    this, SLOT( loadData( KIO::Job*, const QByteArray& ) ) );
-  connect( job, SIGNAL( result( KJob* ) ),
-    this, SLOT( loadResult( KJob* ) ) );
 
-  if ( mProg == NULL )
-  {
+  LdapSearch search;
+  connect( &search, SIGNAL( data( LdapSearch*, const LdapObject& ) ),
+           this, SLOT( loadData( LdapSearch*, const LdapObject& ) ) );
+  connect( &search, SIGNAL( result( LdapSearch* ) ),
+           this, SLOT( loadResult( LdapSearch* ) ) );
+
+  if ( !search.search( _url ) ) {
+    return;
+  }
+
+  if ( mProg == NULL ) {
     mProg = new QProgressDialog( this );
     mProg->setWindowTitle( i18n("LDAP Query") );
     mProg->setModal( true );
@@ -355,9 +358,11 @@ void LdapConfigWidget::sendQuery()
   mProg->exec();
   if ( mCancelled ) {
     kDebug(5700) << "query canceled!" << endl;
-    job->kill( KJob::Quietly );
+    search.abandon();
   } else {
-    if ( !mErrorMsg.isEmpty() ) KMessageBox::error( this, mErrorMsg );
+    if ( search.error() ) {
+      KMessageBox::error( this, search.errorString() );
+    }
   }
 }
 
@@ -376,40 +381,84 @@ void LdapConfigWidget::mQueryDNClicked()
 {
   mAttr = "namingcontexts";
   sendQuery();
-  if ( !mQResult.isEmpty() ) mDn->setText( mQResult.first() );
+  if ( !mQResult.isEmpty() ) {
+    mDn->setText( mQResult.first() );
+  }
 }
 
 void LdapConfigWidget::setAnonymous( bool on )
 {
-  if ( !on ) return;
-  if ( mUser ) mUser->setEnabled(false);
-  if ( mPassword ) mPassword->setEnabled(false);
-  if ( mBindDn ) mBindDn->setEnabled(false);
-  if ( mRealm ) mRealm->setEnabled(false);
-  if ( mMech ) mMech->setEnabled(false);
-  if ( mQueryMech ) mQueryMech->setEnabled(false);
+  if ( !on ) {
+    return;
+  }
+  if ( mUser ) {
+    mUser->setEnabled( false );
+  }
+  if ( mPassword ) {
+    mPassword->setEnabled( false );
+  }
+  if ( mBindDn ) {
+    mBindDn->setEnabled( false );
+  }
+  if ( mRealm ) {
+    mRealm->setEnabled( false );
+  }
+  if ( mMech ) {
+    mMech->setEnabled( false );
+  }
+  if ( mQueryMech ) {
+    mQueryMech->setEnabled( false );
+  }
 }
 
 void LdapConfigWidget::setSimple( bool on )
 {
-  if ( !on ) return;
-  if ( mUser ) mUser->setEnabled(false);
-  if ( mPassword ) mPassword->setEnabled(true);
-  if ( mBindDn ) mBindDn->setEnabled(true);
-  if ( mRealm ) mRealm->setEnabled(false);
-  if ( mMech ) mMech->setEnabled(false);
-  if ( mQueryMech ) mQueryMech->setEnabled(false);
+  if ( !on ) {
+    return;
+  }
+  if ( mUser ) {
+    mUser->setEnabled( false );
+  }
+  if ( mPassword ) {
+    mPassword->setEnabled( true );
+  }
+  if ( mBindDn ) {
+    mBindDn->setEnabled( true );
+  }
+  if ( mRealm ) {
+    mRealm->setEnabled( false );
+  }
+  if ( mMech ) {
+    mMech->setEnabled( false );
+  }
+  if ( mQueryMech ) {
+    mQueryMech->setEnabled( false );
+  }
 }
 
 void LdapConfigWidget::setSASL( bool on )
 {
-  if ( !on ) return;
-  if ( mUser ) mUser->setEnabled(true);
-  if ( mPassword ) mPassword->setEnabled(true);
-  if ( mBindDn ) mBindDn->setEnabled(true);
-  if ( mRealm ) mRealm->setEnabled(true);
-  if ( mMech ) mMech->setEnabled(true);
-  if ( mQueryMech ) mQueryMech->setEnabled(true);
+  if ( !on ) {
+    return;
+  }
+  if ( mUser ) {
+    mUser->setEnabled( true );
+  }
+  if ( mPassword ) {
+    mPassword->setEnabled( true );
+  }
+  if ( mBindDn ) {
+    mBindDn->setEnabled( true );
+  }
+  if ( mRealm ) {
+    mRealm->setEnabled( true );
+  }
+  if ( mMech ) {
+    mMech->setEnabled( true );
+  }
+  if ( mQueryMech ) {
+    mQueryMech->setEnabled( true );
+  }
 }
 
 void LdapConfigWidget::setLDAPPort()
@@ -422,223 +471,342 @@ void LdapConfigWidget::setLDAPSPort()
   mPort->setValue( 636 );
 }
 
-
 LdapUrl LdapConfigWidget::url() const
 {
-  LdapUrl _url;
-  if ( mSecSSL && mSecSSL->isChecked() )
-    _url.setProtocol( "ldaps" );
-  else
-    _url.setProtocol( "ldap" );
+  return server().url();
+}
 
-  if ( mUser ) _url.setUser( mUser->text() );
-  if ( mPassword ) _url.setPass( mPassword->text() );
-  if ( mHost ) _url.setHost( mHost->text() );
-  if ( mPort ) _url.setPort( mPort->value() );
-  if ( mDn ) _url.setDn( mDn->text() );
-  if ( mVersion ) _url.setExtension( "x-ver", QString::number( mVersion->value() ) );
-  if ( mSizeLimit && mSizeLimit->value() != 0 )
-    _url.setExtension( "x-sizelimit", QString::number( mSizeLimit->value() ) );
-  if ( mTimeLimit && mTimeLimit->value() != 0 )
-    _url.setExtension( "x-timelimit", QString::number( mTimeLimit->value() ) );
-  if ( mPageSize && mPageSize->value() != 0 )
-    _url.setExtension( "x-pagesize", QString::number( mPageSize->value() ) );
-  if ( mSecTLS && mSecTLS->isChecked() ) _url.setExtension( "x-tls","" );
-  if ( mFilter && !mFilter->text().isEmpty() )
-    _url.setFilter( mFilter->text() );
-  if ( mSASL && mSASL->isChecked() ) {
-    _url.setExtension( "x-sasl", "" );
-    _url.setExtension( "x-mech", mMech->currentText() );
-    if ( mBindDn && !mBindDn->text().isEmpty() )
-      _url.setExtension( "bindname", mBindDn->text() );
-    if ( mRealm && !mRealm->text().isEmpty() )
-      _url.setExtension( "x-realm", mRealm->text() );
+void LdapConfigWidget::setUrl( const LdapUrl &url )
+{
+  LdapServer _server;
+  _server.setUrl( url );
+  setServer( _server );
+}
+
+LdapServer LdapConfigWidget::server() const
+{
+  LdapServer _server;
+  if ( mSecSSL && mSecSSL->isChecked() ) {
+    _server.setSecurity( LdapServer::SSL );
+  } else if ( mSecTLS && mSecTLS->isChecked() ) {
+    _server.setSecurity( LdapServer::TLS );
+  } else {
+    _server.setSecurity( LdapServer::None );
   }
-  return ( _url );
+
+  if ( mUser ) {
+    _server.setUser( mUser->text() );
+  }
+  if ( mBindDn ) {
+    _server.setBindDn( mBindDn->text() );
+  }
+  if ( mPassword ) {
+    _server.setPassword( mPassword->text() );
+  }
+  if ( mRealm ) {
+    _server.setRealm( mRealm->text() );
+  }
+  if ( mHost ) {
+    _server.setHost( mHost->text() );
+  }
+  if ( mPort ) {
+    _server.setPort( mPort->value() );
+  }
+  if ( mDn ) {
+    _server.setBaseDn( mDn->text() );
+  }
+  if ( mFilter ) {
+    _server.setFilter( mFilter->text() );
+  }
+  if ( mVersion ) {
+    _server.setVersion( mVersion->value() );
+  }
+  if ( mSizeLimit && mSizeLimit->value() != 0 ) {
+    _server.setSizeLimit( mSizeLimit->value() );
+  }
+  if ( mTimeLimit && mTimeLimit->value() != 0 ) {
+    _server.setTimeLimit( mTimeLimit->value() );
+  }
+  if ( mPageSize && mPageSize->value() != 0 ) {
+    _server.setPageSize( mPageSize->value() );
+  }
+  if ( mAnonymous && mAnonymous->isChecked() ) {
+    _server.setAuth( LdapServer::Anonymous );
+  } else if ( mSimple && mSimple->isChecked() ) {
+    _server.setAuth( LdapServer::Simple );
+  } else if ( mSASL && mSASL->isChecked() ) {
+    _server.setAuth( LdapServer::SASL );
+    _server.setMech( mMech->currentText() );
+  }
+  return _server;
+}
+
+void LdapConfigWidget::setServer( const LdapServer &server )
+{
+  switch ( server.security() ) {
+  case LdapServer::SSL:
+    if ( mSecSSL ) {
+      mSecSSL->setChecked( true );
+    }
+  case LdapServer::TLS:
+    if ( mSecTLS ) {
+      mSecTLS->setChecked( true );
+    }
+  case LdapServer::None:
+    if ( mSecNo ) {
+      mSecNo->setChecked( true );
+    }
+  }
+
+  switch ( server.auth() ) {
+  case LdapServer::Anonymous:
+    if ( mAnonymous ) {
+      mAnonymous->setChecked( true );
+    }
+  case LdapServer::Simple:
+    if ( mSimple ) {
+      mSimple->setChecked( true );
+    }
+  case LdapServer::SASL:
+    if ( mSASL ) {
+      mSASL->setChecked( true );
+    }
+  }
+
+  setUser( server.user() );
+  setBindDn( server.bindDn() );
+  setPassword( server.password() );
+  setRealm( server.realm() );
+  setHost( server.host() );
+  setPort( server.port() );
+  setFilter( server.filter() );
+  setDn( server.baseDn() );
+  setVersion( server.version() );
+  setSizeLimit( server.sizeLimit() );
+  setTimeLimit( server.timeLimit() );
+  setPageSize( server.pageSize() );
+  setMech( server.mech() );
 }
 
 void LdapConfigWidget::setUser( const QString &user )
 {
-  if ( mUser ) mUser->setText( user );
+  if ( mUser ) {
+    mUser->setText( user );
+  }
 }
 
 QString LdapConfigWidget::user() const
 {
-  return ( mUser ? mUser->text() : QString() );
+  return mUser ? mUser->text() : QString();
 }
 
 void LdapConfigWidget::setPassword( const QString &password )
 {
-  if ( mPassword ) mPassword->setText( password );
+  if ( mPassword ) {
+    mPassword->setText( password );
+  }
 }
 
 QString LdapConfigWidget::password() const
 {
-  return ( mPassword ? mPassword->text() : QString() );
+  return mPassword ? mPassword->text() : QString();
 }
 
 void LdapConfigWidget::setBindDn( const QString &binddn )
 {
-  if ( mBindDn ) mBindDn->setText( binddn );
+  if ( mBindDn ) {
+    mBindDn->setText( binddn );
+  }
 }
 
 QString LdapConfigWidget::bindDn() const
 {
-  return ( mBindDn ? mBindDn->text() : QString() );
+  return mBindDn ? mBindDn->text() : QString();
 }
 
 void LdapConfigWidget::setRealm( const QString &realm )
 {
-  if ( mRealm ) mRealm->setText( realm );
+  if ( mRealm ) {
+    mRealm->setText( realm );
+  }
 }
 
 QString LdapConfigWidget::realm() const
 {
-  return ( mRealm ? mRealm->text() : QString() );
+  return mRealm ? mRealm->text() : QString();
 }
 
 void LdapConfigWidget::setHost( const QString &host )
 {
-  if ( mHost ) mHost->setText( host );
+  if ( mHost ) {
+    mHost->setText( host );
+  }
 }
 
 QString LdapConfigWidget::host() const
 {
-  return ( mHost ? mHost->text() : QString() );
+  return mHost ? mHost->text() : QString();
 }
 
 void LdapConfigWidget::setPort( int port )
 {
-  if ( mPort ) mPort->setValue( port );
+  if ( mPort ) {
+    mPort->setValue( port );
+  }
 }
 
 int LdapConfigWidget::port() const
 {
-  return ( mPort ? mPort->value() : 389 );
+  return mPort ? mPort->value() : 389;
 }
 
 void LdapConfigWidget::setVersion( int version )
 {
-  if ( mVersion ) mVersion->setValue( version );
+  if ( mVersion ) {
+    mVersion->setValue( version );
+  }
 }
 
 int LdapConfigWidget::version() const
 {
-  return ( mVersion ? mVersion->value() : 3 );
+  return mVersion ? mVersion->value() : 3;
 }
 
 void LdapConfigWidget::setDn( const QString &dn )
 {
-  if ( mDn ) mDn->setText( dn );
+  if ( mDn ) {
+    mDn->setText( dn );
+  }
 }
 
 QString LdapConfigWidget::dn() const
 {
-  return ( mDn ? mDn->text() : QString() );
+  return mDn ? mDn->text() : QString();
 }
 
 void LdapConfigWidget::setFilter( const QString &filter )
 {
-  if ( mFilter ) mFilter->setText( filter );
+  if ( mFilter ) {
+    mFilter->setText( filter );
+  }
 }
 
 QString LdapConfigWidget::filter() const
 {
-  return ( mFilter ? mFilter->text() : QString() );
+  return mFilter ? mFilter->text() : QString();
 }
 
 void LdapConfigWidget::setMech( const QString &mech )
 {
-  if ( mMech == 0 ) return;
+  if ( mMech == 0 ) {
+    return;
+  }
   if ( !mech.isEmpty() ) {
     int i = 0;
     while ( i < mMech->count() ) {
-      if ( mMech->itemText( i ) == mech ) break;
+      if ( mMech->itemText( i ) == mech ) {
+        break;
+      }
       i++;
     }
-    if ( i == mMech->count() ) mMech->addItem( mech );
+    if ( i == mMech->count() ) {
+      mMech->addItem( mech );
+    }
     mMech->setCurrentIndex( i );
   }
 }
 
 QString LdapConfigWidget::mech() const
 {
-  return ( mMech ? mMech->currentText() : QString() );
+  return mMech ? mMech->currentText() : QString();
 }
 
 void LdapConfigWidget::setSecurity( Security security )
 {
   switch ( security ) {
-    case None: 
-      mSecNo->setChecked( true );
-      break;
-    case SSL:
-      mSecSSL->setChecked( true );
-      break;
-    case TLS:
-      mSecTLS->setChecked( true );
-      break;
+  case None:
+    mSecNo->setChecked( true );
+    break;
+  case SSL:
+    mSecSSL->setChecked( true );
+    break;
+  case TLS:
+    mSecTLS->setChecked( true );
+    break;
   }
 }
 
 LdapConfigWidget::Security LdapConfigWidget::security() const
 {
-  if ( mSecTLS->isChecked() ) return TLS;
-  if ( mSecSSL->isChecked() ) return SSL;
+  if ( mSecTLS->isChecked() ) {
+    return TLS;
+  }
+  if ( mSecSSL->isChecked() ) {
+    return SSL;
+  }
   return None;
 }
 
-
 void LdapConfigWidget::setAuth( Auth auth )
 {
   switch ( auth ) {
-    case Anonymous:
-      mAnonymous->setChecked( true );
-      break;
-    case Simple:
-      mSimple->setChecked( true );
-      break;
-    case SASL:
-      mSASL->setChecked( true );
-      break;
+  case Anonymous:
+    mAnonymous->setChecked( true );
+    break;
+  case Simple:
+    mSimple->setChecked( true );
+    break;
+  case SASL:
+    mSASL->setChecked( true );
+    break;
   }
 }
 
 LdapConfigWidget::Auth LdapConfigWidget::auth() const
 {
-  if ( mSimple->isChecked() ) return Simple;
-  if ( mSASL->isChecked() ) return SASL;
+  if ( mSimple->isChecked() ) {
+    return Simple;
+  }
+  if ( mSASL->isChecked() ) {
+    return SASL;
+  }
   return Anonymous;
 }
 
 void LdapConfigWidget::setSizeLimit( int sizelimit )
 {
-  if ( mSizeLimit ) mSizeLimit->setValue( sizelimit );
+  if ( mSizeLimit ) {
+    mSizeLimit->setValue( sizelimit );
+  }
 }
 
 int LdapConfigWidget::sizeLimit() const
 {
-  return ( mSizeLimit ? mSizeLimit->value() : 0 );
+  return mSizeLimit ? mSizeLimit->value() : 0;
 }
 
 void LdapConfigWidget::setTimeLimit( int timelimit )
 {
-  if ( mTimeLimit ) mTimeLimit->setValue( timelimit );
+  if ( mTimeLimit ) {
+    mTimeLimit->setValue( timelimit );
+  }
 }
 
 int LdapConfigWidget::timeLimit() const
 {
-  return ( mTimeLimit ? mTimeLimit->value() : 0 );
+  return mTimeLimit ? mTimeLimit->value() : 0;
 }
 
 void LdapConfigWidget::setPageSize( int pagesize )
 {
-  if ( mPageSize ) mPageSize->setValue( pagesize );
+  if ( mPageSize ) {
+    mPageSize->setValue( pagesize );
+  }
 }
 
 int LdapConfigWidget::pageSize() const
 {
-  return ( mPageSize ? mPageSize->value() : 0 );
+  return mPageSize ? mPageSize->value() : 0;
 }
 
 LdapConfigWidget::WinFlags LdapConfigWidget::features() const
@@ -656,8 +824,9 @@ void LdapConfigWidget::setFeatures( LdapConfigWidget::WinFlags features )
 
   for ( int i = 0; i < ch.count(); ++i ) {
     QWidget *widget = dynamic_cast<QWidget*>( ch[ i ] );
-    if ( widget && widget->parent() == this )
+    if ( widget && widget->parent() == this ) {
       delete ( widget );
+    }
   }
 
   // Re-create child widgets according to the new flags
