@@ -1,17 +1,17 @@
 /*
   This file is part of libkldap.
   Copyright (c) 2004-2006 Szombathelyi György <gyurco@freemail.hu>
-    
+
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General  Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
-            
+
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
-                    
+
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
@@ -20,7 +20,7 @@
 
 #include "ldapoperation.h"
 
-#include <malloc.h>
+#include <stdlib.h>
 
 #include <kdebug.h>
 
@@ -35,10 +35,13 @@ using namespace KLDAP;
 
 class LdapOperation::LdapOperationPrivate {
   public:
-    LdapControls mClientCtrls,mServerCtrls, mControls;
-    LdapObject mObject;
-            
-    LdapConnection *mConnection;
+  LdapControls mClientCtrls, mServerCtrls, mControls;
+  LdapObject mObject;
+  QByteArray mExtOid, mExtData;
+  QString mMatchedDn;
+  QList<QByteArray> mReferrals;
+
+  LdapConnection *mConnection;
 };
 
 LdapOperation::LdapOperation()
@@ -57,7 +60,7 @@ LdapOperation::~LdapOperation()
 {
   delete d;
 }
-                  
+
 void LdapOperation::setConnection( LdapConnection &conn )
 {
   d->mConnection = &conn;
@@ -69,97 +72,125 @@ LdapConnection &LdapOperation::connection()
 }
 
 void LdapOperation::setClientControls( const LdapControls &ctrls )
-{ 
+{
   d->mClientCtrls = ctrls;
 }
 
 void LdapOperation::setServerControls( const LdapControls &ctrls )
-{ 
+{
   d->mServerCtrls = ctrls;
 }
 
-const LdapControls &LdapOperation::clientControls() const 
-{ 
+const LdapControls &LdapOperation::clientControls() const
+{
   return d->mClientCtrls;
 }
 
-const LdapControls &LdapOperation::serverControls() const 
-{ 
+const LdapControls &LdapOperation::serverControls() const
+{
   return d->mServerCtrls;
 }
 
 const LdapObject &LdapOperation::object() const
-{ 
+{
   return d->mObject;
 }
 
-const LdapControls &LdapOperation::controls() const 
-{ 
+const LdapControls &LdapOperation::controls() const
+{
   return d->mControls;
 }
-                  
+
+QByteArray LdapOperation::extendedOid() const
+{
+  return d->mExtOid;
+}
+
+QByteArray LdapOperation::extendedData() const
+{
+  return d->mExtData;
+}
+
+QString LdapOperation::matchedDn() const
+{
+  return d->mMatchedDn;
+}
+
+QList<QByteArray> LdapOperation::referrals() const
+{
+  return d->mReferrals;
+}
+
 #ifdef LDAP_FOUND
 
 static void addModOp( LDAPMod ***pmods, int mod_type, const QString &attr,
-  const QByteArray &value )
+                      const QByteArray &value )
 {
   //  kDebug() << "type: " << mod_type << " attr: " << attr <<
   //    " value: " << QString::fromUtf8(value,value.size()) <<
   //    " size: " << value.size() << endl;
   LDAPMod **mods;
-    
+
   mods = *pmods;
-      
+
   uint i = 0;
-        
+
   if ( mods == 0 ) {
-    mods = (LDAPMod **) malloc ( 2 * sizeof( LDAPMod* ) );
-    mods[ 0 ] = (LDAPMod*) malloc( sizeof( LDAPMod ) );
+    mods = (LDAPMod **) malloc ( 2 * sizeof( LDAPMod * ) );
+    mods[ 0 ] = (LDAPMod *) malloc( sizeof( LDAPMod ) );
     mods[ 1 ] = 0;
     memset( mods[ 0 ], 0, sizeof( LDAPMod ) );
   } else {
-    while( mods[ i ] != 0 &&
-      ( strcmp( attr.toUtf8(),mods[i]->mod_type ) != 0 ||
-      ( mods[ i ]->mod_op & ~LDAP_MOD_BVALUES ) != mod_type ) ) i++;
-                                            
+    while ( mods[ i ] != 0 &&
+            ( strcmp( attr.toUtf8(), mods[i]->mod_type ) != 0 ||
+              ( mods[ i ]->mod_op & ~LDAP_MOD_BVALUES ) != mod_type ) ) i++;
+
     if ( mods[ i ] == 0 ) {
-      mods = ( LDAPMod ** )realloc( mods, (i + 2) * sizeof( LDAPMod * ) );
+      mods = (LDAPMod **)realloc( mods, ( i + 2 ) * sizeof( LDAPMod * ) );
       if ( mods == 0 ) {
         kError() << "addModOp: realloc" << endl;
         return;
       }
       mods[ i + 1 ] = 0;
-      mods[ i ] = ( LDAPMod* ) malloc( sizeof( LDAPMod ) );
+      mods[ i ] = (LDAPMod *) malloc( sizeof( LDAPMod ) );
       memset( mods[ i ], 0, sizeof( LDAPMod ) );
     }
   }
 
   mods[ i ]->mod_op = mod_type | LDAP_MOD_BVALUES;
-  if ( mods[ i ]->mod_type == 0 ) mods[ i ]->mod_type = strdup( attr.toUtf8() );
+  if ( mods[ i ]->mod_type == 0 ) {
+    mods[ i ]->mod_type = strdup( attr.toUtf8() );
+  }
 
   *pmods = mods;
-  
+
   int vallen = value.size();
-  if ( vallen == 0 ) return;
+  if ( vallen == 0 ) {
+    return;
+  }
   BerValue *berval;
-  berval = ( BerValue* ) malloc( sizeof( BerValue ) );
+  berval = (BerValue*) malloc( sizeof( BerValue ) );
   berval -> bv_val = (char*) malloc( vallen );
   berval -> bv_len = vallen;
   memcpy( berval -> bv_val, value.data(), vallen );
-                
+
   if ( mods[ i ] -> mod_vals.modv_bvals == 0 ) {
-    mods[ i ]->mod_vals.modv_bvals = ( BerValue** ) malloc( sizeof( BerValue* ) * 2 );
+    mods[ i ]->mod_vals.modv_bvals =
+      (BerValue**) malloc( sizeof( BerValue * ) * 2 );
     mods[ i ]->mod_vals.modv_bvals[ 0 ] = berval;
     mods[ i ]->mod_vals.modv_bvals[ 1 ] = 0;
     kDebug() << "addModOp: new bervalue struct " << endl;
   } else {
     uint j = 0;
-    while ( mods[ i ]->mod_vals.modv_bvals[ j ] != 0 ) j++;
-    mods[ i ]->mod_vals.modv_bvals = ( BerValue ** )
-    realloc( mods[ i ]->mod_vals.modv_bvals, (j + 2) * sizeof( BerValue* ) );
+    while ( mods[ i ]->mod_vals.modv_bvals[ j ] != 0 ) {
+      j++;
+    }
+    mods[ i ]->mod_vals.modv_bvals =
+      (BerValue **) realloc( mods[ i ]->mod_vals.modv_bvals,
+                             ( j + 2 ) * sizeof( BerValue * ) );
     if ( mods[ i ]->mod_vals.modv_bvals == 0 ) {
       kError() << "addModOp: realloc" << endl;
-      free(berval);
+      free( berval );
       return;
     }
     mods[ i ]->mod_vals.modv_bvals[ j ] = berval;
@@ -169,13 +200,13 @@ static void addModOp( LDAPMod ***pmods, int mod_type, const QString &attr,
 }
 
 static void addControlOp( LDAPControl ***pctrls, const QString &oid,
-  const QByteArray &value, bool critical )
+                          const QByteArray &value, bool critical )
 {
   LDAPControl **ctrls;
   LDAPControl *ctrl = (LDAPControl *) malloc( sizeof( LDAPControl ) );
-      
+
   ctrls = *pctrls;
-        
+
   kDebug() << "addControlOp: oid:'" << oid << "' val: '" << value << "'" << endl;
   int vallen = value.size();
   ctrl->ldctl_value.bv_len = vallen;
@@ -191,13 +222,16 @@ static void addControlOp( LDAPControl ***pctrls, const QString &oid,
   uint i = 0;
 
   if ( ctrls == 0 ) {
-    ctrls = (LDAPControl **) malloc ( 2 * sizeof( LDAPControl* ) );
+    ctrls = (LDAPControl **) malloc ( 2 * sizeof( LDAPControl * ) );
     ctrls[ 0 ] = 0;
     ctrls[ 1 ] = 0;
   } else {
-    while ( ctrls[ i ] != 0 ) i++;
+    while ( ctrls[ i ] != 0 ) {
+      i++;
+    }
     ctrls[ i + 1 ] = 0;
-    ctrls = (LDAPControl **) realloc( ctrls, (i + 2) * sizeof( LDAPControl * ) );
+    ctrls =
+      (LDAPControl **) realloc( ctrls, ( i + 2 ) * sizeof( LDAPControl * ) );
   }
   ctrls[ i ] = ctrl;
   *pctrls = ctrls;
@@ -219,8 +253,8 @@ static void extractControls( LdapControls &ctrls, LDAPControl **pctrls )
   while ( pctrls[i] ) {
     ctrl = pctrls[ i ];
     control.setOid( QString::fromUtf8( ctrl->ldctl_oid ) );
-    control.setValue( QByteArray::fromRawData( ctrl->ldctl_value.bv_val,
-      ctrl->ldctl_value.bv_len ) );
+    control.setValue( QByteArray( ctrl->ldctl_value.bv_val,
+                                  ctrl->ldctl_value.bv_len ) );
     control.setCritical( ctrl->ldctl_iscritical );
     ctrls.append( control );
     i++;
@@ -228,7 +262,7 @@ static void extractControls( LdapControls &ctrls, LDAPControl **pctrls )
 }
 
 int LdapOperation::search( const QString &base, LdapUrl::Scope scope,
-    const QString &filter, const QStringList& attributes )
+                           const QString &filter, const QStringList &attributes )
 {
   Q_ASSERT( d->mConnection );
   LDAP *ld = (LDAP*) d->mConnection->handle();
@@ -239,44 +273,51 @@ int LdapOperation::search( const QString &base, LdapUrl::Scope scope,
   LDAPControl **serverctrls = 0, **clientctrls = 0;
   createControls( &serverctrls, d->mServerCtrls );
   createControls( &serverctrls, d->mClientCtrls );
-      
+
   int count = attributes.count();
   if ( count > 0 ) {
-    attrs = static_cast<char**>( malloc((count+1) * sizeof(char*)) );
-    for (int i=0; i<count; i++)
+    attrs = static_cast<char**>( malloc( ( count + 1 ) * sizeof( char * ) ) );
+    for ( int i=0; i<count; i++ ) {
       attrs[i] = strdup( attributes.at(i).toUtf8() );
+    }
     attrs[count] = 0;
   }
-                              
+
   int lscope = LDAP_SCOPE_BASE;
   switch ( scope ) {
-    case LdapUrl::Base:
-      lscope = LDAP_SCOPE_BASE;
-      break;
-    case LdapUrl::One:
-      lscope = LDAP_SCOPE_ONELEVEL;
-      break;
-    case LdapUrl::Sub:
-      lscope = LDAP_SCOPE_SUBTREE;
-      break;
+  case LdapUrl::Base:
+    lscope = LDAP_SCOPE_BASE;
+    break;
+  case LdapUrl::One:
+    lscope = LDAP_SCOPE_ONELEVEL;
+    break;
+  case LdapUrl::Sub:
+    lscope = LDAP_SCOPE_SUBTREE;
+    break;
   }
-                                                                                    
-  kDebug() << "asyncSearch() base=\"" << base << "\" scope=" << scope << 
+
+  kDebug() << "asyncSearch() base=\"" << base << "\" scope=" << scope <<
     " filter=\"" << filter << "\" attrs=" << attributes << endl;
-  int retval = ldap_search_ext( ld, base.toUtf8(), lscope, 
-    filter.isEmpty() ? QByteArray("objectClass=*") : filter.toUtf8(),
-    attrs, 0, serverctrls, clientctrls, 0, d->mConnection->sizeLimit(), &msgid );
-                    
+  int retval =
+    ldap_search_ext( ld, base.toUtf8(), lscope,
+                     filter.isEmpty() ? QByteArray("objectClass=*") : filter.toUtf8(),
+                     attrs, 0, serverctrls, clientctrls, 0,
+                     d->mConnection->sizeLimit(), &msgid );
+
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
-                        
+
   // free the attributes list again
   if ( count > 0 ) {
-    for ( int i=0; i<count; i++ ) free( attrs[i] );
-    free(attrs);
+    for ( int i=0; i<count; i++ ) {
+      free( attrs[i] );
+    }
+    free( attrs );
+  }
+
+  if ( retval == 0 ) {
+    retval = msgid;
   }
-                                      
-  if ( retval == 0 ) retval = msgid;
   return retval;
 }
 
@@ -292,7 +333,7 @@ int LdapOperation::add( const LdapObject &object )
   createControls( &serverctrls, d->mServerCtrls );
   createControls( &serverctrls, d->mClientCtrls );
 
-  for ( LdapAttrMap::ConstIterator it = object.attributes().begin(); 
+  for ( LdapAttrMap::ConstIterator it = object.attributes().begin();
         it != object.attributes().end(); ++it ) {
     QString attr = it.key();
     for ( LdapAttrValue::ConstIterator it2 = (*it).begin(); it2 != (*it).end(); ++it2 ) {
@@ -300,12 +341,16 @@ int LdapOperation::add( const LdapObject &object )
     }
   }
 
-  int retval = ldap_add_ext( ld, object.dn().toUtf8(), lmod, serverctrls, clientctrls, &msgid );
+  int retval =
+    ldap_add_ext( ld, object.dn().toString().toUtf8(), lmod, serverctrls,
+                  clientctrls, &msgid );
 
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
   ldap_mods_free( lmod, 1 );
-  if ( retval == 0 ) retval = msgid;
+  if ( retval == 0 ) {
+    retval = msgid;
+  }
   return retval;
 }
 
@@ -320,7 +365,7 @@ int LdapOperation::add_s( const LdapObject &object )
   createControls( &serverctrls, d->mServerCtrls );
   createControls( &serverctrls, d->mClientCtrls );
 
-  for ( LdapAttrMap::ConstIterator it = object.attributes().begin(); 
+  for ( LdapAttrMap::ConstIterator it = object.attributes().begin();
         it != object.attributes().end(); ++it ) {
     QString attr = it.key();
     for ( LdapAttrValue::ConstIterator it2 = (*it).begin(); it2 != (*it).end(); ++it2 ) {
@@ -328,7 +373,9 @@ int LdapOperation::add_s( const LdapObject &object )
     }
   }
 
-  int retval = ldap_add_ext_s( ld, object.dn().toUtf8(), lmod, serverctrls, clientctrls );
+  int retval =
+    ldap_add_ext_s( ld, object.dn().toString().toUtf8(), lmod, serverctrls,
+                    clientctrls );
 
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
@@ -337,7 +384,7 @@ int LdapOperation::add_s( const LdapObject &object )
 }
 
 int LdapOperation::rename( const QString &dn, const QString &newRdn,
-      const QString &newSuperior, bool deleteold )
+                           const QString &newSuperior, bool deleteold )
 {
   Q_ASSERT( d->mConnection );
   LDAP *ld = (LDAP*) d->mConnection->handle();
@@ -348,18 +395,21 @@ int LdapOperation::rename( const QString &dn, const QString &newRdn,
   createControls( &serverctrls, d->mServerCtrls );
   createControls( &serverctrls, d->mClientCtrls );
 
-  int retval = ldap_rename( ld, dn.toUtf8(), newRdn.toUtf8(), newSuperior.toUtf8(), 
-    deleteold, serverctrls, clientctrls, &msgid );
+  int retval = ldap_rename( ld, dn.toUtf8(), newRdn.toUtf8(),
+                            newSuperior.isEmpty() ? (char *) 0 : newSuperior.toUtf8().data(),
+                            deleteold, serverctrls, clientctrls, &msgid );
 
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
 
-  if ( retval == 0 ) retval = msgid;
+  if ( retval == 0 ) {
+    retval = msgid;
+  }
   return retval;
 }
 
 int LdapOperation::rename_s( const QString &dn, const QString &newRdn,
-      const QString &newSuperior, bool deleteold )
+                             const QString &newSuperior, bool deleteold )
 {
   Q_ASSERT( d->mConnection );
   LDAP *ld = (LDAP*) d->mConnection->handle();
@@ -368,8 +418,9 @@ int LdapOperation::rename_s( const QString &dn, const QString &newRdn,
   createControls( &serverctrls, d->mServerCtrls );
   createControls( &serverctrls, d->mClientCtrls );
 
-  int retval = ldap_rename_s( ld, dn.toUtf8(), newRdn.toUtf8(), newSuperior.toUtf8(), 
-    deleteold, serverctrls, clientctrls );
+  int retval = ldap_rename_s( ld, dn.toUtf8(), newRdn.toUtf8(),
+                              newSuperior.isEmpty() ? (char *) 0 : newSuperior.toUtf8().data(),
+                              deleteold, serverctrls, clientctrls );
 
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
@@ -393,7 +444,9 @@ int LdapOperation::del( const QString &dn )
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
 
-  if ( retval == 0 ) retval = msgid;
+  if ( retval == 0 ) {
+    retval = msgid;
+  }
   return retval;
 }
 
@@ -429,18 +482,18 @@ int LdapOperation::modify( const QString &dn, const ModOps &ops )
   for ( int i = 0; i < ops.count(); ++i ) {
     int mtype = 0;
     switch ( ops[i].type ) {
-      case Mod_None:
-        mtype = 0;
-        break;
-      case Mod_Add: 
-        mtype = LDAP_MOD_ADD;
-        break;
-      case Mod_Replace: 
-        mtype = LDAP_MOD_REPLACE;
-        break;
-      case Mod_Del:
-        mtype = LDAP_MOD_DELETE;
-        break;
+    case Mod_None:
+      mtype = 0;
+      break;
+    case Mod_Add:
+      mtype = LDAP_MOD_ADD;
+      break;
+    case Mod_Replace:
+      mtype = LDAP_MOD_REPLACE;
+      break;
+    case Mod_Del:
+      mtype = LDAP_MOD_DELETE;
+      break;
     }
     for ( int j = 0; j < ops[i].values.count(); ++j ) {
       addModOp( &lmod, mtype, ops[i].attr, ops[i].values[j] );
@@ -452,7 +505,9 @@ int LdapOperation::modify( const QString &dn, const ModOps &ops )
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
   ldap_mods_free( lmod, 1 );
-  if ( retval == 0 ) retval = msgid;
+  if ( retval == 0 ) {
+    retval = msgid;
+  }
   return retval;
 }
 
@@ -470,18 +525,18 @@ int LdapOperation::modify_s( const QString &dn, const ModOps &ops )
   for ( int i = 0; i < ops.count(); ++i ) {
     int mtype = 0;
     switch ( ops[i].type ) {
-      case Mod_None:
-        mtype = 0;
-        break;
-      case Mod_Add: 
-        mtype = LDAP_MOD_ADD;
-        break;
-      case Mod_Replace: 
-        mtype = LDAP_MOD_REPLACE;
-        break;
-      case Mod_Del:
-        mtype = LDAP_MOD_DELETE;
-        break;
+    case Mod_None:
+      mtype = 0;
+      break;
+    case Mod_Add:
+      mtype = LDAP_MOD_ADD;
+      break;
+    case Mod_Replace:
+      mtype = LDAP_MOD_REPLACE;
+      break;
+    case Mod_Del:
+      mtype = LDAP_MOD_DELETE;
+      break;
     }
     for ( int j = 0; j < ops[i].values.count(); ++j ) {
       addModOp( &lmod, mtype, ops[i].attr, ops[i].values[j] );
@@ -508,19 +563,21 @@ int LdapOperation::compare( const QString &dn, const QString &attr, const QByteA
 
   int vallen = value.size();
   BerValue *berval;
-  berval = ( BerValue* ) malloc( sizeof( BerValue ) );
+  berval = (BerValue*) malloc( sizeof( BerValue ) );
   berval -> bv_val = (char*) malloc( vallen );
   berval -> bv_len = vallen;
   memcpy( berval -> bv_val, value.data(), vallen );
 
-  int retval = ldap_compare_ext( ld, dn.toUtf8(), attr.toUtf8(), berval, 
-    serverctrls, clientctrls, &msgid );
-  
+  int retval = ldap_compare_ext( ld, dn.toUtf8(), attr.toUtf8(), berval,
+                                 serverctrls, clientctrls, &msgid );
+
   ber_bvfree( berval );
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
 
-  if ( retval == 0 ) retval = msgid;
+  if ( retval == 0 ) {
+    retval = msgid;
+  }
   return retval;
 }
 
@@ -535,14 +592,14 @@ int LdapOperation::compare_s( const QString &dn, const QString &attr, const QByt
 
   int vallen = value.size();
   BerValue *berval;
-  berval = ( BerValue* ) malloc( sizeof( BerValue ) );
+  berval = (BerValue*) malloc( sizeof( BerValue ) );
   berval -> bv_val = (char*) malloc( vallen );
   berval -> bv_len = vallen;
   memcpy( berval -> bv_val, value.data(), vallen );
 
-  int retval = ldap_compare_ext_s( ld, dn.toUtf8(), attr.toUtf8(), berval, 
-    serverctrls, clientctrls );
-  
+  int retval = ldap_compare_ext_s( ld, dn.toUtf8(), attr.toUtf8(), berval,
+                                   serverctrls, clientctrls );
+
   ber_bvfree( berval );
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
@@ -562,19 +619,21 @@ int LdapOperation::exop( const QString &oid, const QByteArray &data )
 
   int vallen = data.size();
   BerValue *berval;
-  berval = ( BerValue* ) malloc( sizeof( BerValue ) );
+  berval = (BerValue*) malloc( sizeof( BerValue ) );
   berval -> bv_val = (char*) malloc( vallen );
   berval -> bv_len = vallen;
   memcpy( berval -> bv_val, data.data(), vallen );
 
-  int retval = ldap_extended_operation( ld, oid.toUtf8(), berval, 
-    serverctrls, clientctrls, &msgid );
-  
+  int retval = ldap_extended_operation( ld, oid.toUtf8(), berval,
+                                        serverctrls, clientctrls, &msgid );
+
   ber_bvfree( berval );
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
 
-  if ( retval == 0 ) retval = msgid;
+  if ( retval == 0 ) {
+    retval = msgid;
+  }
   return retval;
 }
 
@@ -584,21 +643,21 @@ int LdapOperation::exop_s( const QString &oid, const QByteArray &data )
   LDAP *ld = (LDAP*) d->mConnection->handle();
   BerValue *retdata;
   char *retoid;
-  
+
   LDAPControl **serverctrls = 0, **clientctrls = 0;
   createControls( &serverctrls, d->mServerCtrls );
   createControls( &serverctrls, d->mClientCtrls );
 
   int vallen = data.size();
   BerValue *berval;
-  berval = ( BerValue* ) malloc( sizeof( BerValue ) );
+  berval = (BerValue*) malloc( sizeof( BerValue ) );
   berval -> bv_val = (char*) malloc( vallen );
   berval -> bv_len = vallen;
   memcpy( berval -> bv_val, data.data(), vallen );
 
-  int retval = ldap_extended_operation_s( ld, oid.toUtf8(), berval, 
-    serverctrls, clientctrls, &retoid, &retdata );
-  
+  int retval = ldap_extended_operation_s( ld, oid.toUtf8(), berval,
+                                          serverctrls, clientctrls, &retoid, &retdata );
+
   ber_bvfree( berval );
   ber_bvfree( retdata );
   free( retoid );
@@ -618,7 +677,7 @@ int LdapOperation::abandon( int id )
   createControls( &serverctrls, d->mClientCtrls );
 
   int retval = ldap_abandon_ext( ld, id, serverctrls, clientctrls );
-  
+
   ldap_controls_free( serverctrls );
   ldap_controls_free( clientctrls );
 
@@ -632,66 +691,101 @@ int LdapOperation::result( int id )
 
   LDAPMessage *msg;
   int rescode, retval;
-  
+
   rescode = ldap_result( ld, id, 0, NULL, &msg );
-  if ( rescode == -1 ) return -1;
+  if ( rescode == -1 ) {
+    return -1;
+  }
 
   switch ( rescode ) {
-    case LDAP_RES_SEARCH_ENTRY: {
-      d->mObject.clear();
-      LdapAttrMap attrs;
-      char *name;
-      struct berval **bvals;
-      BerElement     *entry;
-            
-      char *dn = ldap_get_dn( ld, msg );
-      d->mObject.setDn( QString::fromUtf8( dn ) );
-      ldap_memfree( dn );
-                    
-      // iterate over the attributes
-      name = ldap_first_attribute(ld, msg, &entry);
-      while ( name != 0 ) {
-        // print the values
-        bvals = ldap_get_values_len(ld, msg, name);
-        LdapAttrValue values;
-        if ( bvals ) {
-          for ( int i = 0; bvals[i] != 0; i++ ) {
-            char* val = bvals[i]->bv_val;
-            unsigned long len = bvals[i]->bv_len;
-            values.append( QByteArray( val, len ) );
-          }
-          ldap_value_free_len(bvals);
+  case RES_SEARCH_ENTRY:
+  {
+    d->mObject.clear();
+    LdapAttrMap attrs;
+    char *name;
+    struct berval **bvals;
+    BerElement     *entry;
+
+    char *dn = ldap_get_dn( ld, msg );
+    d->mObject.setDn( QString::fromUtf8( dn ) );
+    ldap_memfree( dn );
+
+    // iterate over the attributes
+    name = ldap_first_attribute( ld, msg, &entry );
+    while ( name != 0 ) {
+      // print the values
+      bvals = ldap_get_values_len( ld, msg, name );
+      LdapAttrValue values;
+      if ( bvals ) {
+        for ( int i = 0; bvals[i] != 0; i++ ) {
+          char *val = bvals[i]->bv_val;
+          unsigned long len = bvals[i]->bv_len;
+          values.append( QByteArray( val, len ) );
         }
-        attrs[ QString::fromLatin1( name ) ] = values;
-        // next attribute
-        name = ldap_next_attribute(ld, msg, entry);
+        ldap_value_free_len( bvals );
       }
-      ber_free( entry , 0 );
-      d->mObject.setAttributes( attrs );
-      break;
+      attrs[ QString::fromLatin1( name ) ] = values;
+      // next attribute
+      name = ldap_next_attribute( ld, msg, entry );
     }
-    default: {
-      LDAPControl **serverctrls = 0;
-      char *matcheddn = 0, *errmsg = 0;
-      char **referralsp;
-      int errcodep;
-      retval = ldap_parse_result( ld, msg, &errcodep, &matcheddn, &errmsg, &referralsp, &serverctrls, 0 );
-      kDebug() << "rescode " << rescode << " retval: " << retval << " matcheddn: " << matcheddn << " errcode: " 
-        << errcodep << " errmsg: " << errmsg << endl;
-      if ( retval != LDAP_SUCCESS ) {
-        ldap_msgfree( msg );
-        return -1;
-      }
-      d->mControls.clear();
-      if ( serverctrls ) {
-        extractControls( d->mControls, serverctrls );
-        ldap_controls_free( serverctrls );
-      }
-//FIXME: ldap_value_free deprecated, what to use?
-//      if ( referralsp ) ldap_value_free( referralsp );
-      if ( matcheddn ) ldap_memfree( matcheddn );
-      if ( errmsg ) ldap_memfree( errmsg );
+    ber_free( entry , 0 );
+    d->mObject.setAttributes( attrs );
+    break;
+  }
+  case RES_EXTENDED:
+  {
+    char *retoid;
+    struct berval *retdata;
+    retval = ldap_parse_extended_result( ld, msg, &retoid, &retdata, 0 );
+    if ( retval != LDAP_SUCCESS ) {
+      ldap_msgfree( msg );
+      return -1;
     }
+    d->mExtOid = retoid ? QByteArray( retoid ) : QByteArray();
+    d->mExtData = retdata ? QByteArray( retdata->bv_val, retdata->bv_len ) : QByteArray();
+    ldap_memfree( retoid );
+    ber_bvfree( retdata );
+    break;
+  }
+  default:
+  {
+    LDAPControl **serverctrls = 0;
+    char *matcheddn = 0, *errmsg = 0;
+    char **referralsp;
+    int errcodep;
+    retval =
+      ldap_parse_result( ld, msg, &errcodep, &matcheddn, &errmsg, &referralsp,
+                         &serverctrls, 0 );
+    kDebug() << "rescode " << rescode << " retval: " << retval
+             << " matcheddn: " << matcheddn << " errcode: "
+             << errcodep << " errmsg: " << errmsg << endl;
+    if ( retval != LDAP_SUCCESS ) {
+      ldap_msgfree( msg );
+      return -1;
+    }
+    d->mControls.clear();
+    if ( serverctrls ) {
+      extractControls( d->mControls, serverctrls );
+      ldap_controls_free( serverctrls );
+    }
+    d->mReferrals.clear();
+    if ( referralsp ) {
+      char **tmp = referralsp;
+      while ( *tmp ) {
+        d->mReferrals.append( QByteArray( *tmp ) );
+        tmp++;
+      }
+      ber_memvfree( (void **) referralsp );
+    }
+    d->mMatchedDn = QString();
+    if ( matcheddn ) {
+      d->mMatchedDn = QString::fromUtf8( matcheddn );
+      ldap_memfree( matcheddn );
+    }
+    if ( errmsg ) {
+      ldap_memfree( errmsg );
+    }
+  }
   }
 
   ldap_msgfree( msg );
@@ -701,7 +795,7 @@ int LdapOperation::result( int id )
 
 #else
 int LdapOperation::search( const QString &base, LdapUrl::Scope scope,
-    const QString &filter, const QStringList& attributes )
+                           const QString &filter, const QStringList &attributes )
 {
   kError() << "LDAP support not compiled" << endl;
   return -1;
@@ -720,14 +814,14 @@ int LdapOperation::add_s( const LdapObject &object )
 }
 
 int LdapOperation::rename( const QString &dn, const QString &newRdn,
-      const QString &newSuperior, bool deleteold )
+                           const QString &newSuperior, bool deleteold )
 {
   kError() << "LDAP support not compiled" << endl;
   return -1;
 }
 
 int LdapOperation::rename_s( const QString &dn, const QString &newRdn,
-      const QString &newSuperior, bool deleteold )
+                             const QString &newSuperior, bool deleteold )
 {
   kError() << "LDAP support not compiled" << endl;
   return -1;
@@ -787,12 +881,10 @@ int LdapOperation::result( int id )
   return -1;
 }
 
-
 int LdapOperation::abandon( int id )
 {
   kError() << "LDAP support not compiled" << endl;
   return -1;
 }
 
-
 #endif
