@@ -29,7 +29,7 @@ class TestPersonImpl : public QObject
 {
     Q_OBJECT
         
-    private slots:
+    private Q_SLOTS:
     
         void fromString();
 };
